;######################################
;
; ROBOT BUILDER
;
;######################################
;
; The robot building function is
; conducted in 3 major phases:
;
; 1. Name
; 2. Choose Image
; 3. Allocate
;
; In the NAME section, you use
; UP/DOWN to choose between
; characters; and LEFT/RIGHT to
; shift between letters.
;
; The the IMAGE section, you
; use LEFT/RIGHT to cycle
; between robot sprites
;
; The allocater phases uses the
; LIVE MENU system to allocate
; stat points.  The following
; stats may be affected in this
; phase:
;
; 1. Hitpoints          (0-255)
; 2. Energy Points      (0-100)
; 3. Attack             (0-100)
; 4. Defense            (0-100)
; 5. Speed              (0-100)
; 6. TYPE               (fire-laser)
; 7. ROLE               (des-def)
;
; The User uses LEFT/RIGHT to
; decreases/increase the appropriate
; stat.
;
; The User can use UP/DOWN move
; between the different stats
;
; The final "BUILD" stage accepts the
; stat preferences and loads them to
; the next available data slot in the
; TEXT_MEM data space.
;
;######################################

;________ROBOT TEMPORARY (BATTLE/BUILDER ONLY) DATA
;________ROBOT 0_

_hp             = mapbuffer+544+199+6+6+8+8+8+8
_hpl            = _hp + 1
_cp             = _hp + 2
_cpl            = _hp + 3
_statush        = _hp + 4
_statusl        = _hp + 5
_hpmax          = _hp + 6
_hpmaxl         = _hp + 7
_attack         = _hp + 8
_defense        = _hp + 9
_accessory      = _hp + 10
_arms           = _hp + 11
_techa          = _hp + 12
_techb          = _hp + 13
_techc          = _hp + 14
_typeclass      = _hp + 15
_armor          = _hp + 16
_level          = _hp + 17
_image          = _hp + 18
_togo           = _hp + 19
_togol          = _hp + 20



BUILDER:
 ld a,(BOTS)
 inc a
 ld e,a
 ld a,(ALLIES)
 cp e
 jp nc,start_builder

 ld hl,msg_too_many_robots
 call TALKBOX
 jp game_loop_entry

start_builder:

 call TEMPS_CLEAR2
 ld hl,OP1
 ld de,OP1+1
 ld (hl),0
 ld bc,11 - 1
 ldir

 ld hl,0
 ld (SPEND),hl

NAMER:
 call GRAPH_CLEAR

 call LETTERBOX

 set 3,(IY+$05)                 ; White text

 ld hl,str_builder              ; Display "Builder"
 ld de,$0023                    ;
 call DMX2                      ;

 res 3,(IY+$05)                 ; Normal text

 ld hl,str_name                 ; Display "Name:"
 ld de,$0F03                    ;
 call DMX2                      ;
 call CR_GRBCopy                ; Display now.Big text doesn't affect GRAPH

 ld hl,OP1
 ld de,$0304
namer_loop1:
 push hl
 push de
 ld hl,OP1                      ; Display contents of OP1 (name)
 ld de,$0303                    ;
 ld (CURSOR_POS),de             ;
 ROM_CALL(D_ZT_STR)             ; Display in large text
 pop de
 pop hl
namer_loop0:
 push hl
 ld hl,str_blank                ; Display Namer's cursor (an underscore)
 ld (CURSOR_POS),de             ;
 ROM_CALL(D_ZT_STR)             ;
 ld hl,blit2                    ; Now blit the cursor so it can be moved.
 ld (CURSOR_POS),de             ; Blitting will also make the cursor blink.
 ROM_CALL(D_ZT_STR)             ;
 call GET_KEY
 pop hl
 cp G_ALPHA
 jp z,LOAD_MAP
 cp G_LEFT
 jr z,namer_cursor_left
 cp G_2ND
 jr z,IMAGER
 cp G_RIGHT
 jr z,namer_cursor_right
 cp G_UP
 jr z,namer_letter_dec  ; Actually decrement so, UP moves you towards 'A'
 cp G_DOWN
 jr z,namer_letter_inc  ; Actually increment so DOWN moves you towards 'Z'
 jr namer_loop0
namer_cursor_left:
 ld a,d                         ; Find cursor position
 cp 3                           ; 3 = minimum cursor position
 jr z,namer_loop0
 dec hl                         ; Target letter to the left
 dec d
 jr namer_loop1
namer_cursor_right:
 ld a,d                         ; Find cursor's position
 cp 12                          ; 13 = rightmost cursor position
 jr z,namer_loop0
 inc hl                         ; Target letter to the right
 ld a,(hl)
 or a
 jr nz,no_fill
 ld (hl),' '
no_fill:
 inc d
 jr namer_loop1
namer_letter_inc:
 ld a,(hl)
 or a
 jr z,namer_a
 cp 'Z'                         ; Stop at big zero
 jr z,namer_loop1
 cp ' '
 jr nz,no_namer_inc_skip
namer_a:
 ld a,'A' - 1
no_namer_inc_skip
 inc a
 ld (hl),a
 jr namer_loop1
namer_letter_dec:
 ld a,(hl)
 or a
 jr z,namer_a
 cp ' '
 jr z,namer_loop1
 cp 'A'
 jr nz,no_namer_dec_skip
 ld a,' ' + 1
no_namer_dec_skip:
 dec a
 ld (hl),a
 jp namer_loop1

IMAGER:
 ld hl,mapbuffer
 ld de,mapbuffer+1
 ld bc,1024 - 1
 ld (hl),0
 ldir

 ld de,SPRITES_0        ; Load battle sprites and robot sprites
 ld hl,mapbuffer        ;
 call DecodeZCP         ;

 ld hl,str_image
 ld de,$2703
 call DMX2

 ld a,1
 ld (_image),a

 ld hl,spr_robot0
imager_loop1:
 ld bc,$282D
 push hl
 call PutSprClp
 pop hl
 call CR_GRBCopy

imager_loop0:
 push hl
 call GET_KEY
 pop hl
 cp G_RIGHT
 jr z,image_inc
 cp G_LEFT
 jr z,image_dec
 cp G_ALPHA
 jp z,NAMER
 cp G_2ND
 jr z,ALLOCATE
 jr imager_loop0

image_inc:
 ld a,(_image)
 cp 8
 jr z,imager_loop0
 ld de,16
 add hl,de
 inc a
 ld (_image),a
 jr imager_loop1
image_dec:
 ld a,(_image)
 cp 1
 jr z,imager_loop0
 ld de,-16
 add hl,de
 dec a
 ld (_image),a
 jr imager_loop1

ALLOCATE:
 ld hl,(GOLD)
 ld (SPEND),hl
 ld c,1
 ld de,$080F
 ld (allocate_pos),de
allocate_loop1:
 ld a,c
 ld (allocate_cursor),a

 call GRAPH_CLEAR
 call LETTERBOX

 set 3,(IY+$05)                 ; White text

 ld hl,str_gold
 ld de,$3920
 call DMX2

 ld hl,GOLD
 ROM_CALL($0033-$1A)
 ld de,$3933
 call DMD2

 ld hl,OP1                      ; Show robot's name
 ld de,$0001                    ;
 call DMX2                      ;

 res 3,(IY+$05)                 ; Normal text

 ld hl,msg_allocate             ; "HP/EP/ATK/DFN/SPE/TYPE/CLASS/done"
 ld de,$0814                    ;
 call PARAGRAPH                 ;

;---show stats
 ld hl,tbl_allocated_stats              ; Show latest EP,ATK,DFN, and SPE
 ld de,tbl_allocated_stats_locations    ;
 ld b,5                                 ;
 call VALUE_ARRAY                       ;

 ld a,(_typeclass)              ; Show TYPE
 srl a                          ;
 srl a                          ;
 srl a                          ;
 srl a                          ;
 add a,first_power              ; [see robot.h]
 ld b,a                         ;
 ld de,$2628                    ;
 call LIBRARY                   ;

 ld a,(_typeclass)              ; Show ROLE
 and %00001111                  ;
 add a,first_role               ;
 ld b,a                         ;
 ld de,$2C28                    ;
 call LIBRARY                   ;
;---
 ld a,(allocate_cursor)
 ld c,a
allocate_loop0:
 ld a,8                         ; 8 = Lenght of menu
 ld de,(allocate_pos)
 call MENU_LIVE                 ; Menu ; Out: A = key C = cursor
 ld (allocate_pos),de
 ld hl,GOLD
 cp G_2ND
 jp z,BUILD
 cp G_ALPHA
 jp z,NAMER
 cp G_RIGHT
 jr z,stat_inc
 cp G_LEFT
 jp z,stat_dec
 jr allocate_loop0

allocate_cursor:
.db 0
allocate_pos:
.dw $0000

stat_inc:
 ld a,c
 cp 6
 jr z,inc_type
 cp 7
 jp z,inc_role
 ld a,(hl)
 or a
 jr nz,stat_inc_costly
 inc hl
 ld a,(hl)
 dec hl
 or a
 jr z,allocate_loop0
stat_inc_costly:
 call Gold_dec
 ld a,c
 cp 1
 jr z,inc_hp
 cp 2
 jr z,inc_cp
 cp 3
 jr z,inc_atk
 cp 4
 jr z,inc_dfn
 cp 5
 jr z,inc_spe
 jp allocate_loop0

inc_limit:
 cp 100
 ret nz
 pop hl                         ; Fix the stack
undo_gold_dec:
 call Gold_inc                  ; Undo the money subtraction
 jp allocate_loop0

inc_hp:
 ld a,(_hpl)
 cp 255
 jr z,undo_gold_dec
 inc a
 ld (_hpl),a
 jp allocate_loop1
inc_cp:
 ld a,(_cpl)
 call inc_limit
 inc a
 ld (_cpl),a
 jp allocate_loop1
inc_atk:
 ld a,(_attack)
 call inc_limit
 inc a
 ld (_attack),a
 jp allocate_loop1
inc_dfn:
 ld a,(_defense)
 call inc_limit
 inc a
 ld (_defense),a
 jp allocate_loop1
inc_spe:
 ld a,(_accessory)
 call inc_limit
 inc a
 ld (_accessory),a
 jp allocate_loop1
inc_type:
 ld a,(_typeclass)
 cp 64
 jp nc,allocate_loop0
 add a,16
 ld (_typeclass),a
 jp allocate_loop1
inc_role:
 ld a,(_typeclass)
 and %00000011
 cp 3
 jp nc,allocate_loop0
 inc a
 ld e,a
 ld a,(_typeclass)
 and %11110000
 or e
 ld (_typeclass),a
 jp allocate_loop1

stat_dec:
 ld a,c
 cp 6
 jr z,dec_type
 cp 7
 jr z,dec_role
 call Gold_inc
 cp 1
 jr z,dec_hp
 cp 2
 jr z,dec_cp
 cp 3
 jr z,dec_atk
 cp 4
 jr z,dec_dfn
 cp 5
 jr z,dec_spe
 jp allocate_loop0

dec_limiter:
 or a
 ret nz
 call Gold_dec                  ; Undo gold increment
 pop hl                         ; Fix the stack
 jp allocate_loop0

dec_hp:
 ld a,(_hpl)
 call dec_limiter
 dec a
 ld (_hpl),a
 jp allocate_loop1
dec_cp:
 ld a,(_cpl)
 call dec_limiter
 dec a
 ld (_cpl),a
 jp allocate_loop1
dec_atk:
 ld a,(_attack)
 call dec_limiter
 dec a
 ld (_attack),a
 jp allocate_loop1
dec_dfn:
 ld a,(_defense)
 call dec_limiter
 dec a
 ld (_defense),a
 jp allocate_loop1
dec_spe:
 ld a,(_accessory)
 call dec_limiter
 dec a
 ld (_accessory),a
 jp allocate_loop1
dec_type:
 ld a,(_typeclass)
 and %11110000
 jp z,allocate_loop0
 sub 16
 ld (_typeclass),a
 jp allocate_loop1
dec_role:
 ld a,(_typeclass)
 and %00001111
 jp z,allocate_loop0
 ld a,(_typeclass)
 dec a
 ld (_typeclass),a
 jp allocate_loop1

tbl_allocated_stats:
.dw _hpl
.dw _cpl
.dw _attack
.dw _defense
.dw _accessory

tbl_allocated_stats_locations:
.dw $0828       ; Location for HP
.dw $0E28       ; EP
.dw $1428       ; ATK
.dw $1A28       ; DFN
.dw $2028       ; SPE

BUILD:
 ld a,c                 ; Check cursor
 cp 8                   ; Selected "done?"
 jp nz,ALLOCATE

 ld a,(_hpl)            ; Set up a maximum HP
 ld (_hpmaxl),a         ;

 ld a,first_melee       ; Set default weapon (Fist)
 ld (_arms),a           ;

 ld a,(_cpl)
 ld (_cp),a

 ld a,1
 ld (_level),a

 ld a,(_typeclass)
 add a,16
 ld (_typeclass),a
 sub 16
 srl a
 srl a
 srl a
 srl a
 add a,first_power
 ld (_techa),a
set_level:
 ld hl,SPEND            ; HL = SPEND - GOLD
 call ld_de_hlPTR       ;        
 push de                ;
 ld hl,GOLD             ;
 call ld_de_hlPTR       ;
 pop hl                 ;
 sbc hl,de              ; Now HL = Total gold spend on allocation

 srl h
 rr l
 srl h
 rr l
 srl h
 rr l
 ld a,l
 or a
 jr nz,non_zero_level   ; Are robot's levels so low, it's zero?
 inc a                  ; Make level = 1 or else it can't evolve
non_zero_level:
 ld (_level),a
 ld hl,0
 ld b,a
 ld e,a
 ld d,0
togo_loop:
 add hl,de
 djnz togo_loop
 ld (_togo),hl

 ld de,HP1              
 ld a,(IMAGE1)          ; Is slot 1 already filled?
 or a
 push af                ; Save Z flag
 jr z,copy_robot_now
 ld de,HP2
copy_robot_now:
 ld hl,_hp              ; Copy stats to file
 ld bc,21               ;
 ldir                   ;

 ld de,NAME1            ;
 pop af                 ; Restore Z flag
 jr z,copy_name_now
 ld de,NAME2
copy_name_now:
 ld hl,OP1              ; Copy name to file
 ld bc,11               ;
 ldir                   ;

 ld hl,BOTS             ; Increment bot count
 inc (hl)               ;

 jp LOAD_MAP            ; Go back to free mode ; Restore map data


Gold_inc:
 call ld_de_hlPTR               ; Increment 1 gold
 inc de                         ;
 call ld_hlPTR_de               ;
 ret
Gold_dec:
 call ld_de_hlPTR               ; Decrement 1 gold
 dec de                         ;
 call ld_hlPTR_de               ;
 ret
