;################################################
;                                               #
;               COMMUNICATIONS                  #
;                                               #
; This file contains codes for linking and      #
; coordinating calculators in 2-player mode     #
;                                               #
;################################################
;               SYNCHRONISE                     #
; Synchronises two calculators so they will     #
; send/receive at the same time                 #
;################################################
;               BLOCK SEND                      #
; Send data string continiously                 #
;################################################
;               BLOCK RECIEVE                   #
; Receive data into a string continiously       #
;################################################
;               COMMS DELAY                     #
; Communications delay.  Short than a halt, and #
; interrupts can be disabled.                   #
;------------------------------------------------

synchronise:
 ld hl,link_verify3              ; HL -> 'r'
 call linkSend                  ; Send the letter 'r'
sync_hold:
 call GET_KEY                   ; Check for key activity
 cp G_MODE
 ret z
 call linkReceive               ; Try to get a byte from other calculator
 cp 'r'                         ; "Did other calc run LINK after me?"
 jp z,sync_escape
 cp 'w'                         ; "Did other calc run LINK before me?"
 jr nz,sync_hold                ; --If so, you are the GUEST
 call comms_delay2
 ret
sync_escape:
 call comms_delay2
 ld hl,link_verify4             ; HL -> 'w'
 call linkSend                  ; Send the letter 'w'
 ret


;------------------------------------------------
;                                               ;
blockSend:                                      ;
;                                               ;
; IN: HL = Starting address to send from        ;
;      B = Number of bytes                      ;
; OUT:HL = Last address sent + 1                ;
;------------------------------------------------
 push bc
 call linkSend
 pop bc
 djnz blockSend
 ret

;------------------------------------------------
;                                               ;
blockReceive:                                   ;
;                                               ;
; IN: HL = Starting address to send to          ;
;      B = Number of bytes                      ;
; OUT:HL = Last address updated + 1             ; 
;------------------------------------------------
 push hl
 push bc
block_r:
 call GET_KEY2
 cp G_MODE
 jr z,link_failure
 call linkReceive
 jr c,block_r
 pop bc
 pop hl
 ld (hl),a
 inc hl
 djnz blockReceive
 ret

link_failure:
 pop hl         ; Trash saved BC
 pop hl         ; Trash saved HL
 pop hl         ; Trash call to blockReceive
; jp Exit
 jp (ix)        ; Return to FILE_SELECT, 

;------------------------------------------------
;                                               ;
comms_delay2:                                   ;
 ld bc,100                                      ;
;------------------------------------------------
;                                               ;
comms_delay:                                    ;
;                                               ;
; IN: BC = delay time                           ;
; NOTES: BC @ 831 equals one full halt          ;
;        BC @ 17  equals 1/9600 second  (aprox) ;
;                                               ;
; The executions times are comented below:      ;
;                                               ;
; Call "comms_delay2" to replace a halt         ;
;------------------------------------------------
 nop                    ; 0.67us ;
 dec bc                 ; 1.00us ;
 ld a,b                 ; 1.50us ;
 or c                   ; 0.67us ;
 jr nz,comms_delay      ; 2.17us ; 1.34us ;
 ret                    ; 1.67us ;
