;################################################
;                                               #
;               GENERAL                         #
;                                               #
; The following file contains frequently-used   #
; subroutines.                                  #
;                                               #
; routines in this file:                        #
;       - dm_hl_deci                            #
;       - random                                #
;       - delay                                 #
;       - graph_clear                           #
;       - 16 by 16                              #
;       - marquee                               #
;       - letterbox                             #
;                                               #
;################################################
; AFDEHL        DECIMAL-MENU                    #
;                                               #
; Written by Jimmy Mardell.                     #
;------------------------------------------------
;               RANDOM                          #
; Creates a random number between 0 and 255     #
;------------------------------------------------
;AFBCDEHL       16 by 16                        #
;                                               #
; Draw a 16 by 16 pixel icon                    #
; IN:   HL = points to icon data                #
;       DE = GRAPH_MEM spot to draw icon        #
; OUT:  -nothing-                               #
;------------------------------------------------
;AFHLDEB        TEXT ARRAY                      #
;                                               #
; Displays multiple strings at multiple         #
; locations                                     #
;                                               #
; IN:   HL = address of string table            #
;       DE = address of positions table         #
;        B = number of strings to show          #
;                                               #
; OUT:  -nothing-                               #
;                                               #
; EXAMPLE:                                      #
;                                               #
; ld hl,array0_strings                          #
; ld de,array0_positions                        #
; ld b,4                                        #
; call TEXT_ARRAY                               #
; ...                                           #
; array0_strings:                               #
; .db str_hello                                 #
; .db msg_world                                 #
; .db msg_words                                 #
; .db str_language                              #
;                                               #
; array0_positions:                             #
; .db $0000                                     #
; .db $000C                                     #
; .db $0B0B                                     #
; .db $170B                                     #
;                                               #
;------------------------------------------------
; AFHLDEB       VALUES ARRAY                    #
;                                               #
; Displays numbers (0-255) stored at address    #
; at mulitple or odd positions                  #
; (Similar to TEXT ARRAY)                       #
; (Only works on 8-bit numbers)                 #
;                                               #
; IN:   HL = address of pointers table          #
;       DE = address of positions table         #
;        B = number of values to show           #
; EXAMPLE:                                      #
;                                               #
; ld hl,array1_values                           #
; ld de,array1_positions                        #
; ld b,3                                        #
; call VALUE_ARRAY                              #
; ...                                           #
; array1_values:                                #
; .dw TEXT_MEM+20 ; score                       #
; .dw TEXT_MEM+21 ; time                        #
; .dw APD_BUF+300 ; lives                       #
;                                               #
; array1_positions:                             #
; .dw $0000                                     #
; .dw $000C                                     #
; .dw $0B0B                                     #
; .dw $170B                                     #
;------------------------------------------------
; AFHLDEBC      TALKBOX                         #
;                                               #
; Displays one or more lines of text in a black #
; box.  The black talkbox is used when a        #
; character is talking.                         #
;                                               #
;                                               #
; IN: HL = address of text paragraph            #
; NEEDS: Text must end with a "$FF"             #
; EXAMPLE:                                      #
;                                               #
; ld hl,test_text                               #
; call TALKBOX_BLACK                            #
; ...                                           #
; test_text:                                    #
; .db "This is",0                               #
; .db "a paragraph",0                           #
; .db "of text",0,$FF                           #
;                                               #
;------------------------------------------------
; AFHLDEBC      TALKBOX_PIC                     #
;                                               #
; Display a character portrait next to the text #
;                                               #
; IN: HL = Addess of 16x16 image                #
;     DE = Destination address for image        #
;          (Within GRAPH_MEM)                   #
;     BC = Text pointer                         #
;                                               #
;------------------------------------------------

;--------------------------------\
;Jimmy Mardell's DM_HL_DECI code |
;---------------------------------
DM_HL_DECI:
 push de
 push hl
 ld de,string+5
 xor a
 ld (de),a
DM_HL_DECI_Repeat:
 call UNPACK_HL
 add a,'0'
 dec de
 ld (de),a
 ld a,h
 or l
 jr nz,DM_HL_DECI_Repeat
 ex de,hl
 ROM_CALL(D_ZM_STR)
 pop hl
 pop de
 ret

RANDOM:
 push bc
 call RAND
 ld b,a
 call RAND
 rra
 xor b
 pop bc
 ret

DELAY:
 push bc
 ld b,a
delay_loop0:
 halt
 djnz delay_loop0
 pop bc
 ret

GRAPH_FILL:
 ld hl,GRAPH_MEM
 ld de,GRAPH_MEM+1
 ld (hl),$FF
 jr graph_clear_0

GRAPH_CLEAR:
 ld hl,GRAPH_MEM
 ld de,GRAPH_MEM+1
 ld (hl),0
graph_clear_0:
 ld bc,767
 ldir
 ret

APD_CLEAR
 ld hl,APD_BUF
 ld de,APD_BUF+1
 ld (hl),0
 ld bc,767
 ldir
 ret

TEXT_CLEAR:
 ld hl,TEXT_MEM         ; Erase TEXT_MEM's to prevent data from this...
 ld de,TEXT_MEM+1       ; ...file from "escaping" into other files.
 ld bc,SAVE_SIZE1 - 1   ;
 ld (hl),0              ;
 ldir                   ;

 ld hl,TEXT_MEM2+7              ; Clear TEXT_MEM2, but *DO NOT!* overwrite..
 ld de,TEXT_MEM2+7+1            ; ...the savegames name string or...
 ld bc,SAVE_SIZE2 - 7 - 1       ; ...its terminating zero.
 ld (hl),0                      ;
 ldir                           ;
 ret

TEMPS_CLEAR2:           ; Clears all battle data
 ld hl,mapbuffer
 ld de,mapbuffer+1
 ld bc,1024 - 1
 ld (hl),0
 ldir
TEMPS_CLEAR:            ; Clear only cursors and attacks
 ld hl,mapbuffer+544
 ld de,mapbuffer+544+1
 ld bc,199 - 1          ; 199 is size of attack_queue
 ld (hl),0
 ldir
 ret

_16BY16:
 ld b,16                        ; The sprite is 16 rows tall
_16by16_loop0:
 push bc                        ; Save row count
 ldi                            ; Copy one row
 ldi
 call ADD_DE_10                 ; Move down a row 
 pop bc                         ; Restore row count
 djnz _16by16_loop0
 ret

LETTERBOX:
 ld hl,GRAPH_MEM
 ld a,$FF
 ld (hl),a
 ld de,GRAPH_MEM+1
 ld bc,83
 ldir
MARQUEE:
 ld hl,GRAPH_MEM+684
 ld de,GRAPH_MEM+685
 ld a,$FF
 ld (hl),a
 ld bc,83
 ldir
 ret

RIGHT_ROTATE:			; Special thanks to CrASH_Man
 push hl
  SLA (HL)                      ; 96-bit shift left
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
  RL  (HL)
  DEC HL
 pop hl
 ret nc
 ld a,(hl)
 or %00000001
 ld (hl),a
 ret

TEXT_ARRAY:
 push hl                        ; Save pointers to tables
 push de                        ;
 ROM_CALL($0033-$1A)            ; HL <- (HL)...
 ex de,hl                       ; DE <- (DE)
 ROM_CALL($0033-$1A)            ;
 ex de,hl                       ;
 call DMX2                      ; Display one string
 pop de                         ; Restore pointers to tables
 pop hl                         ;
 inc hl                         ; Next string...
 inc hl                         ;
 inc de                         ; ...Next location
 inc de                         ;
 djnz TEXT_ARRAY
 ret

VALUE_ARRAY:
 push hl                        ; Save pointer to VALUES table
 push de                        ; Save pointer to POSITIONS table
 ROM_CALL($0033-$1A)            ; HL <- (HL)
 ROM_CALL($0033-$1A)            ; ...do it twice!
 ld h,0                         ; Clear upper byte. This is an 8-bit value!
 ex de,hl                       ; DE <- (DE)
 ROM_CALL($0033-$1A)            ;
 ex de,hl                       ;
 ld (GRAF_CURS),de              ; Update cursor
 call DM_HL_DECI                ; Display the number in menu font
 pop de                         ; Restore pointer to VALUES table
 pop hl                         ; Restore pointer to POSITIONS table
 inc hl                         ; Next value address
 inc hl
 inc de                         ; Next position
 inc de
 djnz VALUE_ARRAY               ; Display <B> values
 ret

GRAPH_INVERT:
 ld hl,GRAPH_MEM        ; Start at top-left
graph_inv_loop:
 ld a,(hl)              ; Load next 8 pixels
 cpl                    ; Invert!
 ld (hl),a              ; Update pixels
 inc hl                 ; Move to next 8 pixels
 ld de,GRAPH_MEM+768    ; (Stopping point)
 call CP_HL_DE          ; 
 jr nz,graph_inv_loop
 ret

TALKBOX:
 call TALKBOX_WINDOW
 ld de,$0101
TALKBOX_0:
 set 3,(IY+$05)         ; Enable white text
 call PARAGRAPH         ; Display the text
 res 3,(IY+$05)         ; Restore text properties
 call CR_GRBCopy        ; Update the LCD screen
 call CONFIRM_ALPHA     ; Pause for readability
 ret

TALKBOX_WINDOW:
 push hl                ; Save text pointer
 ld hl,GRAPH_MEM        ; Draw the black text box
 ld de,GRAPH_MEM+1      ;
 ld bc,251              ;
 ld (hl),%11111111      ;
 ldir                   ;
 pop hl                 ; Restore text pointer
 ret

talkbox_background:



TALKBOX_PIC:
 push bc                ; Save text pointer
 push hl                ; Save picture address
 push de                ; Save picture location
 ld hl,GRAPH_MEM        ; Draw the black text box
 ld de,GRAPH_MEM+1      ;
 ld bc,251              ;
 ld (hl),%11111111      ;
 ldir                   ;        
 call APD_CLEAR                 ; Prep APD buffer
 ld de,ICONS                    ; Decompress 16x16 icons to APD_BUF
 ld hl,APD_BUF                  ;
 call DecodeZCP                 ; Decompress (size=512)
 pop de                 ; Restore Picture location
 pop hl                 ; Restore Picture address
 call _16BY16
 pop hl                 ; Restore text pointer
 ld de,$0111            ; Position text to the left of the image
 jr TALKBOX_0

HL_SWAP:
 ld a,h
 ld h,l
 ld l,a
 ret

FADE:
 ld hl,GRAPH_MEM
 ld b,2
fade_loop0:
 push bc
 ld b,32
fade_loop1:
 push bc
 ld d,h
 ld e,l
 inc de
 ld bc,12 - 1
 ld (hl),%00000000
 ldir
 ld de,12 + 1
 add hl,de
 call CR_GRBCopy
 halt
 halt
 pop bc                 ; Restore loop counter
 djnz fade_loop1
 ld hl,GRAPH_MEM+12
 pop bc
 djnz fade_loop0
 ret

ld_de_hlPTR:
 ld e,(hl)
 inc hl
 ld d,(hl)
 dec hl
 ret
ld_hlPTR_de:
 ld (hl),e
 inc hl
 ld (hl),d
 dec hl
 ret
