;++++++++++++++++++++++++++++++++++++++++++++++++
; TEXT CONTROLLERS/DISPLAY                      +
;                                               +
; Routines on this file                         +
;       - dmx                                   +
;       - paragraph                             +
;       - confirm                               +
;------------------------------------------------
;                    DMX                        +
;                                               +
; DMX writes menu text to the screen and takes  +
; up less space than ROM_CALL(D_ZM_STR)         +
;                                               +
; DMX2 does more by setting the cursor position +
; IN: HL = points to string                     +
;     DE = cursor position (down:left)          +
; OUT: -nothing-                                +
;                                               +
;------------------------------------------------
;                       DMD                     +
; DMD write menu base10 digits to the screen.   +
;                                               +
; DMD2 does more by setting the cursor position +
;                                               +
; DMD3 does even more by clearing the upper byte+
;                                               +
; IN: HL = decimal value to display             +
;     DE = cursor position (down:left)          +
;                                               +
; OUT: -nothing-                                +
;                                               +
;------------------------------------------------
; AFDHL           PARAGRAPH                     +
;                                               +
; Display a paragraph of text.  Allow for       +
; returns, tabs, and "string shortcuts"         +
; $00       = end of line                       +
; $D0 - $FE = commonly used string              +
; $FF       = end of text                       +
;                                               +
; IN: HL=address of paragraph                   +
;     DE=verticle : left margin                 +
; OUT: -nothing-                                +
; STRING EXAMPLE:                               +
;       .db "An example",0                      +
;       .db "of paragraph",0                    +
;       .db "usage",0,$FF                       +
;------------------------------------------------
; AF            CONFIRM                         +
;                                               +
; Hold until 2ND key is pressed                 +
;------------------------------------------------
; AF          CONFIRM with ALPHA                +
;                                               +
; Hold until 2ND or AlPHA key is pressed        +
; (good for menus)                              +
;++++++++++++++++++++++++++++++++++++++++++++++++
DMX2:
 ld (GRAF_CURS),de
DMX:
 ROM_CALL(D_ZM_STR)
 ret

DMD3:
 ld h,0
DMD2:
 ld (GRAF_CURS),de
DMD:
 call DM_HL_DECI
 ret



PARAGRAPH:
 ld (GRAF_CURS),de      ; Update cursor position
 ROM_CALL(D_ZM_STR)     ; Display string
 ld a,(hl)              ; Checking for special characters
 cp $FF                 ; Check for end-of-string
 ret z                  ; Return at end-of-string
 ld a,d                 ; Move
 add a,6                ; Characters are 6 pixels tall
 ld d,a                 ; Update verticle position.  Keep left margins
 jr PARAGRAPH           ; Loop


CONFIRM:
 call GET_KEY           ; Scan for key activity
 cp G_2ND
 ret z
 jr CONFIRM



CONFIRM_ALPHA:
 call GET_KEY           ; Scan for key activity
 cp G_2ND
 ret z
 cp G_ALPHA
 ret z
 jr CONFIRM_ALPHA


GET_KEY2:               ; A Getkey that NEVER affects HL
 push hl
 call GET_KEY
 pop hl
 ret
