;--------------------------------------------------------
;                                                       ;
;                       DAMAGE RULES                    ;
;                                                       ;
; These tables tell how much damage an attack does      ;
; and how it behaves.                                   ;
;-------------------------------------------------------;
; DAMAGE: How much damage?  Or how many points?         ;
;                                                       ;
; STATUSH:(Part1)                                       ;
;         bit7: n/a                                     ;
;         bit6: n/a                                     ;
;         bit5: inflict STASIS                          ;
;         bit4: inflict STASIS                          ;
;         bit3: inflict WET                             ;
;         bit2: inflict WET                             ;
;         bit1: n/a                                     ;
;         bit0: n/a                                     ;
;                                                       ;
; STATUSL:(Part2)                                       ;
;         bit7: n/a                                     ;
;         bit6: stored (do not change)                  ;
;         bit5: inflict FORCE                           ;
;         bit4: inflict BURN                            ;
;         bit3: n/a                                     ;
;         bit2: n/a                                     ;
;         bit1: inflict SHIELD                          ;
;         bit0: inflict SHIELD                          ;
;                                                       ;
; Advantage: Double damage on robots of this type       ;
;         0 = no type                                   ;
;         1 = fire                                      ;
;         2 = ice                                       ;
;         3 = bomb                                      ;
;         4 = spark                                     ;
;         5 = laser                                     ;
;         F = means "no advantages"                     ;
;                                                       ;
; Algorithym: "DAMAGE" byte indicates what?             ;
;         0 = it indicates DAMAGE                       ;
;         1 = it indicates REVERSE DAMAGE (Healing)     ;
;         2 = nothing                                   ;
;         3 = it indicates DAMAGE (iff beckoning fails) ;
;         4 = powerup | stat affected                   ;
;         5 = nothing, pulse is applied                 ;
;         6 = nothing, a shield is applied              ;
;         7 = nothing, a renewal is applied             ;
;         8 = nothing, force is applied                 ;
;         9 = nothing, flux is applied                  ;
;         A = nothing, victim is healed partially       ;
;         B = nothing, stasis is applied                ;
;         C = nothing, teleport is applied              ;
;         D = nothing, rebuild is applied               ;
;         E = nothing, clone is applied                 ;
;         F = -unused-
;                                                       ;
; Menu: Do you get a menu with this attack so you can   ;
;       control it's behavior?                          ;
;        %00 = no menu                                  ;
;        %10 = power menu                               ;
;        %01 = scope menu                               ;
;        %11 = power menu (disallowed)                  ;
; Accuracy: What is this attack's accuracy?             ;
;       %00 = Can't miss! (100%)                        ;
;       %01 = low (62.5%)                               ;
;       %10 = medium (75.0%)                            ;
;       %11 = high (87.5%)                              ;
; Targeting: How does targeting work for this attack?   ;
;            Who can you target?                        ;
;       %00 = Target anywhere                           ;
;       %01 = Target Enemies only                       ;
;       %10 = Target Allies only                        ;
;       %11 = Target Allies only <disallowed>           ;
; Cost: Cost (in EP) to cast this attack?               ;
;       %00 = No cost                                   ;
;       %01 = Minimum cost                              ;
;       %10 = Double the minimum cost                   ;
;       %11 = Tripple the minimum cost                  ;
;                                                       ;
; Animation: Special Visual Effect? (X is "don't care") ;
;       0X = no animation                               ;
;       2X = fire animation                             ;
;       3X = explosion                                  ;
;       4X = lightning                                  ;
;       5X = Splash/Teleport                            ;
;       6X = Laser                                      ;
;       8X = Randomized Pattern                         ;
;       9X = Randomized Pattern                         ;
;       AX = Teleport (longer than splash)              ;
;       BX = Shield                                     ;
;       CX = Plus sign (items)                          ;
;       DX = Fog                                        ;
;                                                       ;
; Bonus: Is attack eligible for a random "luck" bonus?  ;
;       0 = no                                          ;
;       1 = yes                                         ;
;                                                       ;
; Power: Output level of the attack (used for damage    ;
;        calculations)                                  ;
;       000 = melee or item                             ;
;       001 = alpha                                     ;
;       010 = delta                                     ;
;       011 = beta                                      ;
;       100 = epsilon                                   ;
;       101 = gamma                                     ;
;       110 = ohmega                                    ;
;       111 = sigma                                     ;
;                                                       ;
;--------------------------------------------------------
;    D    |             |         |A:A|         |         |
;    A    |    HALF     | STATUS  |d:l|  MENU   |ANIMATE  |
;    M    |   SHOOTER   | flags   |v:g|  (2)    |  (4)    |
;    A    |     ATK     |         |a:o|ACCURACY | LUCK?   |
;    G    |             |         |n:r|  (2)    |  (1)    |
;    E    |             |         |t:y|TARGETING| POWER   |
;         |             |         |a:t|  (2)    |  (3)    |
;         |             |         |g:h|  COST   |         |
;         |             |         |e:m|  (2)    |         |
;         |             |         | : |         |         |
;         |             |         |   |         |         |
attack_data:
.dw 0007  \.db %00000000,%00000000,$F0,%00110100,%00000000;   Fist
.dw 0014  \.db %00000000,%00000000,$F0,%00110100,%00000000;   Blade
.dw 0021  \.db %00000000,%00000000,$F0,%00110100,%00000000;   Axe
.dw 0037  \.db %00000000,%00000000,$F0,%00110100,%00000000;   Iron Fist
.dw 0009  \.db %00000000,%00000000,$F0,%00010100,%00001000;   Vulcan
.dw 0020  \.db %00000000,%00000000,$F0,%00010100,%00001000;   Carbine
.dw 0028  \.db %00000000,%00000000,$F0,%00010100,%00001000;   Cannon
.dw 0045  \.db %00000000,%00000000,$F0,%00010100,%00001000;   Hellfire
.dw 0004  \.db %00000000,%00000000,$F0,%00110100,%00000000;   Fist 2
.dw 0000  \.db %00000000,%00000000,$F3,%00100100,%00000000;   Flares
.dw 0030/2\.db %00000000,%00010000,$30,%10000101,%00100000;   Fire x
.dw 0026/2\.db %00000000,%00000000,$10,%10000101,%01010000;   Ice x
.dw 0028/2\.db %00000000,%00000000,$40,%10000101,%00110000;   Bomb x
.dw 0024/2\.db %00000000,%00000000,$20,%10000101,%01000000;   Spark x
.dw 0022/2\.db %00000000,%00000000,$50,%10000101,%01100000;   Laser x
.dw $F90A \.db %00000000,%00000000,$04,%11000101,%01000000;   Gravity x
.dw 0000  \.db %00000000,%00000000,$05,%11000101,%01000000;   Pulse x
.dw 0000  \.db %00000000,%00000000,$0A,%11001001,%11000000;   HP+50% x
.dw 0000  \.db %00000000,%00000000,$07,%11001001,%11000000;   Renew x
.dw $0A09 \.db %00000000,%00000000,$04,%11001001,%10110000;   Defenend x
.dw 0000  \.db %00000000,%00000000,$06,%11001001,%10110000;   Shield x
.dw 0000  \.db %00000000,%00000000,$08,%00000110,%10010111;   Force
.dw 0000  \.db %00000000,%00000000,$0C,%00000010,%10100111;   Teleport
.dw 0000  \.db %00000000,%00000000,$0B,%00000110,%10010111;   Stasis
.dw 0000  \.db %00000000,%00000000,$09,%00010110,%10010111;   Flux
.dw 0100  \.db %00000000,%00000000,$F1,%00000000,%00000000;   Repair
.dw 0200  \.db %00000000,%00000000,$F1,%00000000,%00000000;   Repair DX
.dw $0602 \.db %00000000,%00000000,$04,%00000000,%11000000;   Energy
.dw $0C02 \.db %00000000,%00000000,$04,%00000000,%11000000;   Energy DX
.dw $0F08 \.db %00000000,%00000000,$04,%00000000,%11000000;   ATK surge
.dw $0F09 \.db %00000000,%00000000,$04,%00000000,%11000000;   DFN surge
.dw $0000 \.db %00000000,%00000000,$0F,%00000000,%11000000;   Rapid SPE
.dw 0000  \.db %00000000,%00000000,$07,%00000000,%11000000;   Renewal
.dw 0000  \.db %00000000,%00000000,$0E,%00001000,%11000000;   Clone
.dw 0000  \.db %00000000,%00000000,$F0,%00000000,%00110000;   Arson
.dw 0000  \.db %00000000,%00000000,$0D,%00001000,%00000000;   Rebuild
.dw 0000  \.db %00000000,%00000000,$02,%00001000,%00000000;   Add
.dw 0000  \.db %00000000,%00000000,$02,%00001000,%00000000;   Multiply
.dw 0000  \.db %00000000,%00000000,$02,%00001000,%00000000;   Negate


basix_shop:
;--------------------------------------;
.db library_repair      ;              ;
.db library_energy      ; ITEM SHOP 1  ;
.db library_renewal     ;______________;
.db library_rebuild     ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
;---------------------------------------

pro_shop:
;--------------------------------------;
.db library_repair_dx   ;              ;
.db library_energy_dx   ; ITEM SHOP 2  ;
.db library_renewal     ;______________;
.db library_rebuild     ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
.db 0                   ;              ;
;---------------------------------------

advanced_shop:
;--------------------------------------;
.db library_arson       ;              ; 
.db library_clone       ; EXPERT'S SHOP;
.db library_atk         ;______________;
.db library_dfn         ;              ;
.db library_spe         ;              ;
.db library_add         ;              ;
.db library_multiply    ;              ;
.db library_negate      ;              ;
;---------------------------------------

armory:
;--------------------------------------;
.db library_steel       ;              ;
.db library_silver      ; ARMORY       ;
.db library_platinum    ;______________;
.db 0                   ;              ;
.db library_jade        ;              ;
.db library_diamond     ;              ;
.db library_ruby        ;              ;
.db 0                   ;              ;
;---------------------------------------

weapons:
;--------------------------------------;
.db library_blade       ;              ;
.db library_carbine     ; ARMORY       ;
.db library_axe         ;______________;
.db library_cannon      ;              ;
.db library_iron_fist   ;              ;
.db library_hellfire    ;              ;
.db                     ;              ;
.db                     ;              ;
;---------------------------------------


;-----------------------------------------------\
;                                               |
; ITEM COST TABLE                               |
;                                               |
; The following table is used to itemize the    |
; cost in GOLD of items in the game.            |
;                                               |
; Items are sold for the amounts given below    |
;                                               |
; Items are refunded for the same amount        |
;                                               |
;-----------------------------------------------/
Item_costs:
.dw 2                   ; Repair
.dw 5                   ; Repair DX
.dw 2                   ; Energy
.dw 5                   ; Energy DX
.dw 3                   ; ATK surge
.dw 3                   ; DFN surge
.dw 3                   ; SPE surge
.dw 1                   ; Renewal
.dw 2                   ; Clone
.dw 2                   ; Arson
.dw 5                   ; Rebuild
.dw 2                   ; + Add
.dw 5                   ; + Multiply
.dw 5                   ; + Negate
.dw 400                 ; Level-1 anti-melee armor
.dw 100                 ; Level-1 anti-energy armor
.dw 800                 ; Level-2 anti-melee armor
.dw 200                 ; Level-2 anti-energy armor
.dw 1200                ; Level-3 anti-melee armor
.dw 300                 ; Level-3 anti-energy armor 

.dw 50                  ; Fist
weapon_costs:
.dw 100                 ; Blade
.dw 190                 ; Axe
.dw 400                 ; Iron Fist
.dw 58                  ; Vulcan
.dw 142                 ; Carbine
.dw 250                 ; Cannon
.dw 480                 ; Hellfire
