;################################################
;AFBCDEHL                                       #
;                       SPRITE TV               #
;                                               #
; Draw up to 6 icon sprites to screen. Selected #
; icons are shown as normal.  Unselected        #
; sprites are shown to look like TV monitors.   #
;                                               #
;------------------------------------------------
; IN:   HL = table of sprites to draw           #
;       DE = table of locations to draw to      #
;        A = number of sprites to show          #
;                                               #
; OUT:   A = G_2ND or G_ALPHA                   #
;        C = latest cursor position             #
; EXAMPLE:                                      #
;      ld hl,sprites_to_draw                    #
;      ld de,sprite_positions                   #
;      ld a,6                                   #
;      call SPRITE_TV                           #
;                                               #
; sprites_to_draw:                              #
; .dw ICON0                                     #
; .dw ICON3                                     #
; .dw ICON7                                     #
; .dw ICON4                                     #
; .dw ICON0                                     #
; .dw ICON2                                     #
;                                               #
; sprite_positions:                             #
; .db GRAPH_MEM                                 #
; .db GRAPH_MEM+2                               #
; .db GRAPH_MEM+4                               #
; .db GRAPH_MEM+6                               #
; .db GRAPH_MEM+120                             #
; .db GRAPH_MEM+122                             #
;                                               #
;################################################
SPRITE_TV:
 ld c,1                 ; C = cursor position (1 thru XX)
 ld (sprite_max),a
sprite_tv_loop2:
 push af                ; Save original pointer to table
 push de                ; Save original icon count
 push hl
sprite_tv_loop1:
 push af                ; Save current pointer to table
 push de                ; Save current icon number 

 ex de,hl               ; <-- ld de,(de)
 ROM_CALL($0033-$1A)    ; Loads HL <- (HL)
 ex de,hl               ;

 ld a,%11111111         ; Draw top row of "TV Monitor"
 ld (de),a              ;
 inc de                 ;
 ld (de),a              ;
 call ADD_DE_10         ; Next row
 inc de
 ld b,14                ; Middle portion is 14 pixels tall
sprite_tv_loop0:
 call RANDOM            ; "Static" effect. A= random number (0-255)
 or %10000000           ; Draw right side of TV
 ld (de),a
 inc de
 call RANDOM            ; "Static effect"
 or %00000001           ; Draw left side of TV
 ld (de),a
 call ADD_DE_10         ; Next row
 inc de
 djnz sprite_tv_loop0   ; Loop until 14 rows have been created
 ld a,%11111111         ; Draw bottom of TV Monitor
 ld (de),a              ;
 inc de                 ;
 ld (de),a              ; Done drawing TV Monitor!

 pop de                 ; Restore current icon number
 pop af                 ; Restore current pointer to table

 inc de                 ; Load next position
 inc de                 ;
 dec a                  ; Decrement icon number
 or a                   ; "Are there any monitors left to draw?"
 jr nz,sprite_tv_loop1  ; ...If so jump back
 call GET_KEY           ; All monitors have been drawn, scan for keys
 cp G_ALPHA
 jr z,sprite_tv_escape          ; 2ND exits
 cp G_2ND
 jr z,sprite_tv_escape          ; AlPHA exits too
 cp G_RIGHT
 jr z,sprite_tv_cursorinc       ; RIGHT increments cursor
 cp G_LEFT
 jr z,sprite_tv_cursordec       ; LEFT decrements cursor


sprite_tv_icon:                 ; Draw the selected 16x16 icon
 pop hl
 push hl
 ld b,c                         ; Icon table lookup
 dec hl
 dec hl
sprite_tv_loop3:                ;
 inc hl                         ;
 inc hl                         ;
 djnz sprite_tv_loop3           ;
 ROM_CALL($0033-$1A)            ; <-- ld HL,(HL)
 ld (sprite_current),hl
 pop hl
 pop de
 push de

 push hl
 ld b,c                         ; Positions table lookup
 dec de
 dec de
sprite_tv_loop4:                ;
 inc de                         ;
 inc de                         ;
 djnz sprite_tv_loop4
 ex de,hl                       ; <-- ld DE,(DE)
 ROM_CALL($0033-$1A)            ;
 ex de,hl                       ;
sprite_current = $+1
 ld hl,$0000
 call _16BY16                   ; Draw the appropriate icon
 call CR_GRBCopy        ; Show the finished image
 pop hl

 pop de
 pop af
 jp sprite_tv_loop2



sprite_tv_cursorinc:
 ld a,c
sprite_max = $+1
 cp 0                   ; Is cursor at highest position?
 jr z,sprite_tv_icon    ; Stop of so
 inc c                  ; Move to next icon
 jr sprite_tv_icon

sprite_tv_cursordec:
 ld a,c
 cp 1                   ; Is cursor at lower position?
 jr z,sprite_tv_icon    ; stop if so
 dec c                  ; Move to previous icon
 jr sprite_tv_icon

sprite_tv_escape:       ; Fix the stack before leaving SPRITE_TV
 pop hl                 ;
 pop hl                 ;
 pop hl
 ret                    ; C = last cursor position. A = G_2ND or G_ALPHA
