;----------------------------------------------------------------------------
;[[ render_map v2.0 ]] (c) 2000 CrASHware                      [341] (59240)
;----------------------------------------------------------------------------
; Draws a full screen of 8x8 tile map starting at (ssx:sox, ssy:soy)
;
; sox = screen x offset pixel offset (0-7)
; soy = screen y offset pixel offset (0-7)
; ssx = screen tile x
; ssy = screen tile y
;
; mapbuffer      = array of map data, with data being multiplied by 8
; graphic_set   -> array of NUM_TILES tiles, 8 bytes each
; graphics      -> free buffer of [256 + NUM_TILES * 8] bytes, page aligned
; graphics_page  = high byte of pointer to first page of graphics
;

;NUM_TILES = 32                         ; number of tiles (1-32)
;LCD_WIDTH = 12                         ; width of screen (12 or 16)
;MAP_WIDTH = 12                         ; x-width of map (16, 32, or 64)

render_map:

;=- graphic copy section -=

;- init -

   LD    A, (sox)                      ; BC = offset
   CPL                                 ;
   AND   $07                           ;
   LD    B, 0                          ;
   LD    C, A                          ;

   LD    HL, gfx_mask                  ; A = gfx mask
   ADD   HL, BC                        ;
   LD    A, (HL)                       ;
   LD    (gfx_modify1), A              ;
   CPL                                 ;
   LD    (gfx_modify2), A              ;

   LD    HL, snip_copy                 ; HL -> location of snip to copy
   ADD   HL, BC                        ;

   LD    DE, snip_dest                 ; DE -> destination of snip
   LD    C, 15                         ; BC = 15
   LDIR                                ;

;- copy -

   LD    HL, graphic_set               ; HL -> source
   LD    DE, graphics                  ; DE -> dest
   LD    B, NUM_TILES * 4              ; B = number of tiles

_mg_loop:
   LD    A, (HL)                       ; A = graphic
   INC   HL                            ; HL -> next byte of source

snip_dest:
   RLCA                                ; This part relocates
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   LD    C, A                          ; save data in C
   AND   %00001111                     ; mask
   LD    (DE), A                       ; (DE) = first part of data
   INC   D                             ; DE -> right half of byte
   XOR   C                             ; find other side of tile
   LD    (DE), A                       ; (DE) = second part of data
   LD    A, (HL)                       ; A = graphic
   RRCA                                ;
   RRCA                                ;
   RRCA                                ;

   RRCA                                ;

   INC   E                             ; DE -> next byte of dest
   INC   HL                            ; HL -> next byte of source

   LD    C, A                          ; ditto above
gfx_modify2 = $+1                      ;
   AND   %11110000                     ;
   LD    (DE), A                       ;
   DEC   D                             ;
   XOR   C                             ;
   LD    (DE), A                       ;
   INC   E                             ;

   DJNZ  _mg_loop                      ; loop

;=- map render section -=

;- init -

; *** must modify - change calculation to multiply by MAP_WIDTH ***
   ld a,(MAP_WIDTH)
   cp 32
   LD    A, (ssy)                      ; A = ssy
   jr c,rlca_skip
   RLCA                                ; Do extra RLCA if map is 32 wide
rlca_skip:
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   LD    L, A                          ;
   AND   %00001111                     ;
   LD    H, A                          ;
   XOR   L                             ;
   LD    L, A                          ; HL = ssy * 16
   LD    A, (ssx)                      ;
   LD    C, A                          ; BC = ssx
   ADD   HL, BC                        ;
   EX    DE, HL                        ; DE = ssy * 16 + ssx
; *** *** ***

;- render -

   LD    IX, mapbuffer                 ; IX -> mapbuffer + offset
   ADD   IX, DE                        ;

   LD    A, (soy)                      ; A = soy
   AND   $7                            ;
   LD    (soy_and_svn), A              ;
   LD    C, A                          ; C = soy & 7
   SUB   9                             ;
   CPL                                 ;
   LD    B, A                          ; B = 8 - (soy & 7)
   LD    A, C                          ; A = starting row

   LD    C, LCD_WIDTH                  ; C = width of LCD
   LD    D, graphics_page              ; DE -> graphics

   LD    HL, GRAPH_MEM                 ; HL -> GRAPH_MEM
   CALL  draw_map_row_var              ; Draw first row of data

   LD    BC, 1-LCD_WIDTH               ; HL -> next row in GRAPH_MEM
   ADD   HL, BC                        ;
;;-------
;;   LD    BC, MAP_WIDTH-LCD_WIDTH+1     ; IX -> next row in map
   ld b,0
   push af
   ld a,(MAP_WIDTH)
   sub LCD_WIDTH
   ld c,a
   inc bc
   pop af

;;-------
   ADD   IX, BC                        ;

   PUSH  HL                            ; EX HL,IX
   EX    (SP), IX                      ;
   POP   HL                            ;

   INC   D                             ;

   LD    B, 7                          ; B = loop count
_render_loop:
   PUSH  BC
   LD    B, LCD_WIDTH                  ; B = width of LCD

   LD    E, (HL)                       ; DE -> graphic of tile on left

;----------------------------------------------------------------------------
;[ draw_map_row_fix ] 0 lines = 900, 8 lines = 6764 clocks (733/line)
;----------------------------------------------------------------------------
; BC = width of screen
; DE -> graphic of tile on left edge
; HL -> map
; IX -> GRAPH_MEM

_dmrf_loop1:
   INC   HL                            ;
   INC   IX                            ;
   PUSH  HL                            ;

   LD    L, (HL)                       ; HL -> graphic of tile on right
   LD    H, graphics_page              ;   (which is the left half)

   LD    A, (DE)                       ; time critical loop (speed)
   OR    (HL)                          ; 41 clocks per loop
   LD    (IX-1), A                     ; join and save 2 graphics
   INC   E                             ; DE -> next row of graphic
   INC   L                             ; HL -> next row of graphic
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH-1)), A         ;
   INC   E                             ;
   INC   L                             ;
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*2-1)), A       ;
   INC   E                             ;
   INC   L                             ;
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*3-1)), A       ;
   INC   E                             ;
   INC   L                             ;
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*4-1)), A       ;
   INC   E                             ;
   INC   L                             ;
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*5-1)), A       ;
   INC   E                             ;
   INC   L                             ;
   LD    A, (DE)                       ;
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*6-1)), A       ;
   INC   E                             ;
   INC   L                             ;

   LD    A, (DE)                       ; final: 33 clocks
   OR    (HL)                          ;
   LD    (IX+(LCD_WIDTH*7-1)), A       ;

   POP   HL                            ;
   LD    E, (HL)                       ; DE -> graphic of tile on left

   DJNZ  _dmrf_loop1                   ;

   LD    C, LCD_WIDTH*7                ; IX -> next row in GRAPH_MEM
   ADD   IX, BC                        ;
;;-----
;;   LD    C, MAP_WIDTH-LCD_WIDTH        ; HL -> next row in map
   push af
   ld a,(MAP_WIDTH)
   sub LCD_WIDTH
   ld c,a
   pop af
;;-----
   ADD   HL, BC                        ;

   POP   BC                            ;
   DJNZ  _render_loop                  ;

   DEC   D                             ;

   PUSH  HL                            ; EX HL,IX
   EX    (SP), IX                      ;
   POP   HL                            ;

soy_and_svn = $+2
   LD    BC, LCD_WIDTH                 ; B = soy
   XOR   A                             ; A = 0

;----------------------------------------------------------------------------
;[ draw_map_row_var ] 0 lines = 1283, 8 lines = 11259 clocks (1247/line)
;----------------------------------------------------------------------------
; A = starting row
; B = number of rows
; C = width of screen
; DE -> graphics
; HL -> GRAPH_MEM
; IX -> map

draw_map_row_var:
   LD    (start_row), A                ; save starting row
   ADD   A, (IX+0)                     ;
   LD    E, A                          ;

   LD    A, B                          ; save number of rows
   OR    A                             ; but test it first
   JR    Z, _dmrv_delay                ;

   INC   D                             ; DE -> right half of graphic
   LD    (num_rows), A                 ;

   PUSH  BC                            ;

   PUSH  HL                            ;
_dmrv_loop1:
   LD    A, (DE)                       ; 54 clocks per loop
   LD    (HL), A                       ;
   INC   E                             ; copy graphic
   ADD   HL, BC                        ; HL -> next row of GRAPH_MEM
   LD    A, H                          ;
   SUB   B                             ;
   LD    H, A                          ;
   DJNZ  _dmrv_loop1                   ;
   POP   HL                            ;

   DEC   D                             ;
   DEC   C                             ; C = width of screen - 1

_dmrv_loop2:
   PUSH  HL                            ;
start_row = $+1
   LD    A, $00                        ; DE -> graphic
   ADD   A, (IX+1)                     ;
   LD    E, A                          ;
num_rows = $+1
   LD    B, $00                        ; B = number of rows
_dmrv_loop3:
   LD    A, (DE)                       ; 93 clocks per loop
   OR    (HL)                          ; overlay graphic
   LD    (HL), A                       ;
   INC   HL                            ;
   INC   D                             ;
   LD    A, (DE)                       ; copy graphic
   LD    (HL), A                       ;
   INC   E                             ;
   DEC   D                             ;

   LD    A, LCD_WIDTH-1                ; HL -> next row of GRAPH_MEM
   ADD   A, L                          ;
   LD    L, A                          ;
   ADC   A, H                          ;
   SUB   L                             ;
   LD    H, A                          ;
   DJNZ  _dmrv_loop3                   ;

   INC   IX                            ; point to next byte of map
   POP   HL                            ;
   INC   HL                            ; HL -> next column in GRAPH_MEM
   DEC   C                             ;
   JP    NZ, _dmrv_loop2               ;

   LD    A, (start_row)                ; DE -> graphic
   ADD   A, (IX+1)                     ;
   LD    E, A                          ;

   POP   BC                            ; B = rows, C = width of screen
_dmrv_loop4:
   LD    A, (DE)                       ; 61 clocks per loop
   OR    (HL)                          ; overlay graphic
   LD    (HL), A                       ;
   INC   E                             ;
   ADD   HL, BC                        ; HL -> next row of GRAPH_MEM
   LD    A, H                          ;
   SUB   B                             ;
   LD    H, A                          ;
   DJNZ  _dmrv_loop4                   ;
_dmrv_doneloop4:
   RET                                 ;

_dmrv_delay:                           ; delay 1067 clocks
   LD    B, 80
_dmrv_delay_loop:
   DJNZ  _dmrv_delay_loop
   RET

;- data -

gfx_mask:
   .DB   %01111111
   .DB   %00111111
   .DB   %00011111
   .DB   %00001111
   .DB   %00000111
   .DB   %00000011
   .DB   %00000001
   .DB   %00000000

snip_copy:
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   LD    C, A
gfx_modify1 = $+1
   AND   %00001111
   LD    (DE), A
   INC   D
   XOR   C
   LD    (DE), A
   LD    A, (HL)
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
