;################################################
;
;                     BATTLE
;
; The following file contains instructions and
; data for the un-linked battle segment of
; ROBOT WAR. 
;
; INFORMATION:
;
; (HOST)  calculators HOST/GUEST status.
;         HOST == 1...you are host
;         HOST == 4...you are guest
;
; (BOSS)  Boss status for determining rules
;         BOSS ==   0...normal battle
;         BOSS ==   1...boss battle
;         BOSS == 255...link battle
;         BOSS == 254...interrupted link battle
;
; ROBOT NUMBERS EXPLAINATION:
;
; An empty space           == 0
;
; Host's robots            -> Number Range = 1,2 and 3
;
; Guest's robots           -> Number Range = 4,5 and 6
;
; Host's robots(by FORCE)  -> Number Range = 7,8 and 9
;
; Guest's robots(by FORCE) -> Number Range = 10,11 and 12
;
;################################################
;AFBCDEHL            REINSTALL_INT
; Interrupt installer/reinstaller.
; Re-creates interrupt table and copies interrupt
; code to the APD buffer
; NEEDS: CURRENT_INT must hold address of interrupt
;        to install
;------------------------------------------------
BATTLE:
 xor a
 ld (turns),a                   ; Clear turns counter
 ld (BOSS),a                    ; This is a non-boss battle
BATTLE_000:
 ld hl,mapbuffer                ; Clear part of the general-use buffer and...
 ld de,mapbuffer+1              ; ...use it to store battle graphics and...
 ld bc,544+199                  ; ...temp data.
 ld (hl),0                      ;
 ldir                           ;

 ld a,1
 ld (HOST),a

 ld hl,HP                       ; Load the player's robots to the field
 ld de,_hp                      ;
 ld bc,21*max_team - 1          ;
 ldir                           ;
 ld hl,NAME0                    ; Load the player's robot names too
 ld de,_name0                   ;
 ld bc,11*max_team - 1          ;
 ldir                           ;
 ld hl,ITEM1                    ; Load player's items
 ld de,_item_temp               ;
 ld bc,8*3                      ;
 ldir                           ;

 ld hl,_stock_temp2            ; Go to enemy's item stock
 ld de,_stock_temp2+1
 ld bc,max_item_types-1
 ld (hl),1
 ldir                           ; Give enemy 8 random items of stock = 1
 ld hl,_item_temp2
 ld b,max_item_types            ; Length of a player's item list (8)
ai_gets_random_items:
 call RAND
 and %00001111
 add a,first_item
 ld (hl),a
 inc hl
 djnz ai_gets_random_items

 call Field_setup

BATTLE_NO_SETUP:

 call FADE

 DI
 call BG_restore        ; Decompress background image to GRAPH_MEM

 ld de,SPRITES_0        ; Load battle sprites and robot sprites
 ld hl,mapbuffer        ;
 call DecodeZCP         ;

 call RAND
 cp rain_chance
 ld hl,STORY0
 jr c,install_rain
install_sun:
 bit 3,(hl)                     ; "Are we SUPPOSED to do rain?"
 jr nz,install_rain2            ; "...then do rain."
install_sun2:
 xor a
 ld (weather),a
 ld hl,BATTLE_INTZ
 jr install_weather
install_rain:
 ld a,(CURRENT_TILESET)         ; Are you indoors?
 cp 1
 jr z,install_sun2              ; No rain indoors!
 bit 4,(hl)                     ; "Is rain allowed yet?"
 jr z,install_sun2              ; "If not, then do sun."
install_rain2:
 ld a,$FF
 ld (weather),a
 ld hl,BATTLE_INT1
install_weather:
 ld a,h
 ld (CURRENT_INT),a     ;
 ld a,l                 ;
 ld (CURRENT_INTL),a    ;
 call REINSTALL_INT     ; Install and activate graphics interrupt

 xor a                  ; 
 ld (SELECTOR),a        ; Start cursor at the left

;_2P_BATTLE:
; ld a,(BOSS)
; cp 254
; jr c,YOUR_TURN
; ld hl,_name0
; ld ix,_hp
; ld a,(HOST)
; cp 1
; jr z,opening_move
; ld hl,_name3
; ld ix,_hp3
;opening_move:
; IM 1
; call ROBOTS
; dec c                          ; C = Robot Choice  (0,1, or 2)
; ld a,(HOST)                    ; A = 1 or 4
; add a,c                        ; A = {1,2, or 3} or {4,5, or 6}
; dec a                          ; A = Robot being packed/unpacked
; ld (_source_temp),a
; ld a,254                       ; 254 = Code for "UNPACK"
; ld (_destination_temp),a
; ld (_serial_temp),a
; ld (_speed_temp),a
; call Attack_adder
; call BG_restore         ; Redraw background.  Activate IM 2.
; jp AI_TURN



YOUR_TURN:
 ld a,(HOST)
 call Select_leftmost   ; (You always start with the leftmost teammate)
 jp c,PASS              ; Pass if your only robot is FORCEd by the enemy
Your_turn:
 ld e,cursor_select     ; Activate blinked "box" cursor
 call Enable_cursor     ;
your_turn:
 xor a                  ; Reset power attack levels
 ld (power),a           
 ld a,(SELECTOR)
 ld (_source_temp),a    ; Pre-set attack SOURCE
 call Robot_lookup
 ld de,10
 add hl,de              ; Locate robot's SPEED byte
 ld a,(hl)
 ld (_speed_temp),a     ; Pre-set attack SPEED

 call Info_bar          ; Display HP:CP  SHIELDS  NAME
 call CLEAR_CENTER      ; Erase any old menus or text

 ld hl,GRAPH_MEM+446    ; Main ATTACK MENU
 ld de,$081A            ;
 call WINDOW            ;

 ld hl,menu_attack_assist_pass  ; Display: "Attack"
 ld de,$261C                    ;          "POWER ATTACK"
 ld a,4                         ;          "ASSIST"
 call MENU                      ;          "PASS"
 cp G_ALPHA
 jp z,STOP
 ld a,c
 cp 1
 jp z,MELEE                     ; Physical attacks menu
 cp 2
 jp z,POWER_ATTACK              ; Power attacks menu
 cp 3
 jp z,ASSIST                    ; Items and Robot
PASS:
 ld a,(HOST)
 ld (find_on_shift),a
 ld e,%11111110
 call Disable_cursor
 call Select_right
 jp c,AI_TURN
 ld e,%00000001
 call Enable_cursor
 jp your_turn



ASSIST:
 ld hl,GRAPH_MEM+508            ; Item/Robots window
 ld de,$080E                    ;
 call WINDOW                    ;

 ld hl,menu_assist              ; ITEM and ROBOT menu
 ld de,$2B28                    ; DE = position
 ld a,2                         ; A = number of choices
 call MENU                      ;
 cp G_ALPHA
 jp z,your_turn
 push bc                        ; Save latest menu choice
 IM 1                           ; Stop drawing the background and robots
 ld b,28                        ; B = distance to scroll screen down
assist_scroll_loop0:
 call Scroll_D                  ; (BC is unchanged) Scroll Down 1 pixel
 call CR_GRBCopy
 djnz assist_scroll_loop0
 pop bc                         ; Restore latest menu choice
 ld a,c                         ; A = menu choice (1=item 2=robot)
 cp 1
 jp z,assist_with_item          ; Jump over to run the item menu
assist_with_robot:
 call Team_lookup               ; Get shooter's team ; Out: Carry/Non-carry
 jr c,guest_robot_list          ; Jump if this is the guest's robot
host_robot_list:
 ld hl,_name0
 ld ix,_hp
 jr robot_list
guest_robot_list:
 ld hl,_name3
 ld ix,_hp3
robot_list:
 call ROBOTS                    ; Robot display/edit menu
; cp G_ALPHA
; jp z,assist_escape
 jp assist_escape

; dec c                          ; C = choice number (0-2 now)
; push bc                        ; Save choice number
; call Team_lookup               ; IN: SELECTOR
; pop bc                         ; Restore choice number ; C = 0-2
; ld a,c
; jr nc,rotate_host              ; Jump if this is a host robot
;rotate_guest:
; add a,max_team                 ; Make A = 3-5
;rotate_host:

; ld (_source_temp),a            ; A is (0-2) for a host...or (3-5) for guest
; ld hl,_hp - 21
; ld de,21
; ld b,a
; inc b
;robot_rot_valid:
; add hl,de                    ; Make HL -> Robot's HP
; djnz robot_rot_valid

; ld de,5                        ; HL -> Robot's STATUSL byte
; add hl,de                      ;
; bit 6,(hl)
; jr z,assist_with_robot         ; Can't unpack bots that are unpacked already

; push hl                        ; HL -> Robot data
; ld hl,GRAPH_MEM+361            ; Unpack window
; ld de,$0A08                    ;
; call WINDOW                    ;
; pop ix                         ; IX  -> Robot data

; ld hl,str_unpack               ; Write "UNPACK" in the tiny window
; ld de,$1F20                    ;
; call DMX2                      ;

;rotation_menu_now:
; ld hl,menu_null                ; Store/Withdraw menu
; ld de,$1F20                    ;
; ld a,1                         ;
; call MENU                      ;
; cp G_ALPHA
; jp z,assist_with_robot

; ld a,254                       ; A = Attack number for "UNPACK"
; ld (_destination_temp),a       ; 254 is Code for "UNPACK"
; ld (_serial_temp),a            ; Serieal is irrelevant

; call Attack_adder

; call BG_restore                ; ...since <Target> wasn't used

; ld a,(HOST)
; ld (find_on_shift),a
; ld e,%11111110
; call Disable_cursor            ; Erase cursor at old location
; call Select_right
; jp c,AI_TURN
; jp Your_turn


assist_with_item:
 call Team_lookup               ; Get team (Force compatible)
 jr nc,show_host_items          ; NoCarry?  Look at host's items
show_guest_items:
 ld ix,_item_temp2
 jr show_items
show_host_items:
 ld ix,_item_temp
show_items:
 ld hl,str_item                 ; Write: "ITEM"
 call ITEM                      ; Out: A=keypress C= item ###
assist_now:
 cp G_ALPHA
 jp z,assist_escape             ; ALPHA sends you back to main menu
 ld a,c
 or a
 jr z,assist_with_item          ; Jump back if you choose a blank
 ld (_serial_temp),a            ; Pre-set serieal
 cp last_battle_item+1          ; [see robot.h]
 jr c,no_item_error             ; Prevent use of In-game items
item_error:
 jp assist_with_item            ; Jump back to the item window
no_item_error:
 cp first_item                  ; [see robot.h]  
 jr c,item_error                ; Prevent use of Melee Weapons
 cp first_item_enhancer
 jr c,not_combo_item            ; Check for combo items (Add/Multi/Div/Neg)

combo_item:
 ld (combo_type),a              ; Forward the type for later use
 push ix                        ; Save pointer to items
 ld a,(_source_temp)            ; Send the combo item back to...
 ld (_destination_temp),a       ; ...the source (prevents random misses)
 call Attack_adder
 pop ix                         ; Restore pointer to items
combo_item0:
 ld hl,str_combine_with
 call ITEM
 cp G_ALPHA
 jr z,assist_with_item          ; If canceled, go back to item selection
 ld a,c
 cp first_item                  ; Reject non-items and blank spaces as choices
 jr c,combo_item0
 cp first_item_enhancer - 1     ; Reject more combo items, armor, and rebuild
 jr nc,combo_item0
 ld (_serial_temp),a
 ld (item_mode),a               ; Indicate that a combo needs to be made
 jr item_targeting

do_combo_item:
combo_type = $ + 1
 ld a,$00
 cp multiply                      ; Is this "multiply"?
 jr z,do_multiply
do_add:
do_negate:
finish_combo:
 call Attack_adder
 jr item_end
do_multiply:
 xor a                          ; Enable "auto-spread"
 ld (check_for_auto_spread),a   ; "Spreading" divides this item amoung 3!
 jr finish_combo

not_combo_item:
 cp rebuild                     ; "Is this REBUILD?"
 jr nz,not_rebuild
item_rebuild:
 ld ix,_hp
 ld hl,_name0
 ld a,(HOST)
 cp 1
 jr z,item_rebuild_host
item_rebuild_guest:
 ld ix,_hp3
 ld hl,_name3
item_rebuild_host:
 call ROBOTS                    ; Choose which robot to restore/revive
 cp G_ALPHA                     ; Exit the item system if canceled
 jp z,assist_escape
 ld a,(HOST)                    ; Determine the target based on HOST/GUEST
 add a,c                        ; ...status and menu cursor
 dec a
 ld (_destination_temp),a       ; DESTINATION= Bot to rebuild {1-3} or {4-6}
 call Attack_adder
 ld e,%11111110
 call Disable_cursor            ; Erase cursor at old location
 call BG_restore
 jr item_end

item_targeting:
not_rebuild:
 call BG_restore        ; BG restore replaces all the above commands
 ld a,(HOST)                    ; Get HOST/GUEST status for targeting
 ld c,a                         ; C = your team status
; ld a,(_serial_temp)            ; Restore attack number
 ld b,library_repair            ; ALL items target like the repair move
 call Target
 cp G_ALPHA
 jp z,Your_turn
item_mode = $ + 1
 ld a,$00
 or a
 jr z,do_normal_item
 jr do_combo_item
do_normal_item:
 call Attack_adder
item_end:
 xor a
 ld (item_mode),a
 ld a,(HOST)
 ld (find_on_shift),a
 call Select_right              ; Select next bot
 jp c,AI_TURN             ; End turn now if everyone has attacked
 jp Your_turn

assist_escape:
 call BG_restore         ; Redraw background.  Activate IM 2.
 jp your_turn



STOP:
 call CLEAR_CENTER
 ld hl,msg_undo_exit
 ld de,$2A29
 ld a,2
 call MENU
 cp G_ALPHA
 jp z,your_turn
 ld a,c
 cp 2
 jr z,exit_game
stop:
 call TEMPS_CLEAR
 jp YOUR_TURN
exit_game:
 call TEMPS_CLEAR2
 call BG_restore
 IM 1                   ; Restore normal interrupts
 pop hl                 ; Trash the call to BATTLE
 ld hl,STORY0
 bit 6,(hl)             ; "Have you saved at least once?"
 jp nz,Exit             ; If so, exit normally (without save)
 ld hl,Exit             ; <-- Used for an artificial jump after save is done
 push hl                ; 
 jp SAVE                ; If not, auto-save

POWER_ATTACK:
 ld hl,GRAPH_MEM+469
 ld de,$0714
 call WINDOW

 call Robot_lookup
 ld de,12
 add hl,de

 ld de,$2810                    ; Text coordinates
 ld b,3                         ; 3 = number of power attack you can get
power_att_loop1:
 push bc
 ld b,(hl)                      ; B = first robot power attack
 push hl
 call LIBRARY
 pop hl
 inc hl
 ld a,d
 add a,6
 ld d,a
 pop bc
 djnz power_att_loop1

 ld hl,menu_null
 ld de,$2810
 ld a,3
 call MENU
 cp G_ALPHA
 jp z,your_turn

 push bc                        ; Save menu choice
 call Robot_lookup              ; Point HL to robot's data block
 pop bc                         ; Restore menu choice
 ld de,12 - 1                   
 add hl,de                      ; HL points to byte *before* power attacks
 ld b,c
power_att_loop2:
 inc hl                         ; Lookup attack chosen
 djnz power_att_loop2
 ld a,(hl)                      ; A = Attack Chosen
 or a                           ; "Did you chose a blank?"
 jp z,POWER_ATTACK              ; "...if so, don't proceed."
 ld (_serial_temp),a
 ld b,a
 call Attack_lookup             ; IX -> attack data line

 bit 7,(IX+5)                   ; "Can you adjust the POWER of this attack?"
 jr nz,power_menu
host_add_attack:
 ld a,(HOST)                    ; Get HOST/GUEST status for targeting
 ld c,a                         ; C = your status
 ld a,(_serial_temp)            ; Restore attack number
 ld b,a
 call Target                    ; Clear Selection cursor, start targ
 cp G_ALPHA
 jp z,Your_turn
 call Attack_adder
 ld a,(HOST)
 ld (find_on_shift),a
 call Select_right                      ; Select next bot
 jp c,AI_TURN                     ; Go to turn ender if this is the last robot
 jp Your_turn

choice_limit:
 bit 6,(IX+5)
 ret z
 ld a,(RANK)
 or a
 jr nz,_2_choices
_1_choice:
 ld b,1
 ret
_2_choices:
 ld b,2
 ret

power_menu:
 ld hl,GRAPH_MEM+492            ; Power menu window
 ld de,$0B0E                    ;
 call WINDOW                    ;

 ld hl,str_power                ; Print the word: "POWER"
 ld de,$2A04                    ;
 call DMX2                      ;

 ld hl,tbl_blanks               ; Print six _ 's 
 ld de,tbl_greeks_positions     ;
 ld b,6                         ;

 call choice_limit              ; Look up player's attack privilages

 call TEXT_ARRAY                ; Print the attack levels he can use

 ld hl,tbl_greeks               ; Pointer to string pointer table
 ld de,tbl_greeks_positions     ; Pointer to string positions table
 ld a,(RANK)
 cp 6
 jr c,normal_menu
 ld a,6 - 1
normal_menu:
 inc a                          ; Rank alpha -> 1  delta -> 2 ...
 ld b,a                         ; B = Number of characters to show
 call choice_limit              ; Only display symbols you have earned
 call TEXT_ARRAY                ; Draw greek characters

power_menu_loop1:
 ld de,$2A2F                    ; Starting cursor position
 ld c,2                         ; C = 2 (minimum EP cost)
power_menu_loop0:
 ld l,c                         ; Load <C>, current EP cost
 push de                        ; Save cursor position
 ld de,$2A18
 call DMD3                      ; Display EP cost (will be 2,4,6,8,10 or 12)
 ld hl,str_close_blit
 call DMX                       ; Finish writing "< >" around the EP cost
 pop de                         ; Restore cursor position
 ld hl,arrow
 call DMX2
 call GET_KEY
 cp G_NONE
 jr z,power_menu_loop0
 ld hl,blit
 call DMX2
 cp G_UP
 jr z,top_power_row
 cp G_DOWN
 jr z,bottom_power_row
 cp G_RIGHT
 jr z,right_power_move
 cp G_LEFT
 jr z,left_power_move
 cp G_ALPHA
 jp z,POWER_ATTACK
 cp G_2ND
 jr z,power_menu_attack
 jr power_menu_loop0
right_power_move:
 bit 6,(IX+5)                   ; Class-based attack?
 jr nz,power_menu_loop0         ; Can only use alpha or delta
 ld a,e
 add a,10
 cp $43 + 1                     ; As far right as we can go?
 jr nc,power_menu_loop0
 ld e,a
 ld a,c
 add a,4
 ld c,a
 jr power_menu_loop0
left_power_move:
 ld a,e
 sub 10
 cp $2F                         ; As far left as we can go?
 jr c,power_menu_loop0
 ld e,a
 ld a,c
 sub 4
 ld c,a
 jr power_menu_loop0
top_power_row:
 ld a,d
 cp $2A
 jr z,power_menu_loop0
 ld d,$2A
 dec c
 dec c
 jr power_menu_loop0
bottom_power_row:
 ld a,d
 cp $30
 jr z,power_menu_loop0
 ld d,$30
 inc c
 inc c
 jr power_menu_loop0

power_menu_attack:
 srl c                  ; Get half the EP cost
 ld a,(RANK)
 inc a
 cp c                   ; "Have you even earned this attack yet?"
 jp c,power_menu_loop1  ; "...If not, go back to menu."
 ld a,c
 ld (power),a           ; (power): 1=alpha 2=delta 3=beta ...
 jp host_add_attack

tbl_blanks:
.dw str_blank
.dw str_blank
.dw str_blank
.dw str_blank
.dw str_blank
.dw str_blank

tbl_greeks:
.dw str_greeks          ; alpha
.dw str_greeks + 2      ; delta
.dw str_greeks + 4      ; beta
.dw str_greeks + 6      ; epsilon
.dw str_greeks + 8      ; gamma
.dw str_greeks + 10     ; ohmega

tbl_greeks_positions:
.dw $2A33               ; location of "a" (alpha)
.dw $3033               ; location of "^" (delta)
.dw $2A3D               ; location of "B" (beta)
.dw $303D               ; location of "E" (epsilon)
.dw $2A47               ; location of "y" (gamma)
.dw $3047               ; location of "O" (ohmega)


MELEE:
 call Robot_lookup
 ld de,11
 add hl,de
 ld a,(hl)
 ld (_serial_temp),a
 ld b,a                         ; B = Attack to be launched
 ld a,(HOST)
 ld c,a                         ; C = host/guest affiliation
 call Target                            ; Clear Selection cursor, start targ
 cp G_ALPHA                             ; If canceled, go back to main menu
 jp z,your_turn
 call Attack_adder              ; Launch the attack

 ld a,(HOST)
 ld (find_on_shift),a
 call Select_right                      ; Select next bot
 jp c,AI_TURN                     ; End your turn once everyone has moved
 jp Your_turn

ITEM:
; IN: IX -> item data
;     HL -> name data
 push ix
 push hl
 ld hl,GRAPH_MEM+25                     ; The main item window
 ld de,$0A3A                            ;
 call WINDOW                            ;

 ld hl,GRAPH_MEM+25                     ; The smaller window that holds names
 ld de,$0A08                            ;        
 call WINDOW                            ;

 pop hl
 ld de,$030A
 call DMX2                              ; Print robot's name

 push ix                                ; Save item pointer
 ld de,$0B10
 ld b,max_item_types                    ; B=Length of item list (see robot.h)
item_loop0:                             ; <-- Display item list
 push bc
 ld b,(ix)                              ; Load item pointer from index 1
 call LIBRARY                           ; Display item name
 inc ix
 pop bc
 ld a,d
 add a,6
 ld d,a
 djnz item_loop0

 pop ix                                 ; Restore item pointer
 push ix                                ; Save item pointer

item_no_qty = $ + 1
 ld a,$00
 or a
 jr nz,qty_done

 ld hl,table_item_brackets              ; Display brackets with 0's...
 ld de,table_item_backet_positions      ; ...in them.
 ld b,max_item_types                    ;
 call TEXT_ARRAY                        ;

 ld de,$0B4F
 ld b,max_item_types
item_loop1:                             ; <-- Display item quantities
 push de                                ; Save position 
 push bc                                ; Save loop counter
 ld l,(ix+max_item_types+max_item_types)
 call DMD3
 inc ix
 pop bc                                 ; Restore loop counter
 pop de                                 ; Restore position
 ld a,d
 add a,6
 ld d,a
 djnz item_loop1

qty_done:
 xor a                                  ; Re-enable item quantities
 ld (item_no_qty),a                     ;

 pop ix                                 ; Restore item pointer

 ld hl,menu_null                        ; Item selection menu
 ld de,$0B10                            ;
 ld a,max_item_types                    ; Each robot gets 8 items (A)
 call MENU
 ld b,c
 dec ix                                 ; Start *behind* item list
item_loop2:
 inc ix
 djnz item_loop2
 ld c,(ix)                              ; OUT: C=item choice A=latest key
 pop ix
 ret


table_item_brackets:
.dw str_bracket         ; Looks like " [0] "
.dw str_bracket
.dw str_bracket
.dw str_bracket
.dw str_bracket
.dw str_bracket
.dw str_bracket
.dw str_bracket


table_item_backet_positions:
.dw $0B4C
.dw $114C
.dw $174C
.dw $1D4C
.dw $234C
.dw $294C
.dw $2F4C
.dw $354C


ROBOTS:
; IN: IX = pointer to robot data block (top 4 or bottom 4 only)
;     HL = pointer to name data block
;  Carry = normal operations
;NoCarry = choice required

 push hl                                ; Save name pointer
 ld de,robot_display_lable0
 push ix
 pop hl
 ld bc,-21
 add hl,bc
.dw $DD7D
 ld (de),a
 inc de
.dw $DD7C
 ld (de),a

 call GRAPH_CLEAR                       ; Get rid of background junk
 ld hl,table_atk_dfn_spe                ; Table pointing to "ATK DFN SPE"
 ld de,table_atk_dfn_spe_positions      ; Table of positions for the strings
 ld b,3                                 ; Display 3 strings
 call TEXT_ARRAY                        ; Display multiple strings

 ld hl,GRAPH_MEM+72                     ; Robot name/status window
 ld de,$0C1A                            ; 
 call WINDOW                            ;

 pop hl                                 ; Restore name pointer
 ld de,$0707                        
 ld b,max_team
robots_name_loop0:
 push hl
 call DMX2                              ; Display a robot name
 pop hl
 push bc
 ld bc,11
 add hl,bc
 pop bc
 ld a,d
 add a,6
 ld d,a
 djnz robots_name_loop0

 push ix                                ; Save robot data pointer
; ld de,$0736
 ld d,$07                              ; First stat drawn on this row
 ld b,max_team
robots_stat_loop:
 push bc                                ; Save loop counter
 push de
 ld e,$36
 ld l,(IX+8)
 call DMD3                              ; Display ATK
 ld l,(IX+9)
 ld e,68
 call DMD3                              ; Display DFN
 ld l,(IX+10)
 ld e,82
 call DMD3                              ; Display SPE
 ld de,21                               ; 21 = Lenght of robot data
 add ix,de                              ; Move on to next robot
 pop de                                 ; Restore text position
 ld a,d                                 ; Goto next line
 add a,6
 ld d,a
 pop bc                                 ; Restore loop counter
 djnz robots_stat_loop
 pop ix                                 ; Restore robot data pointer

robot_menu:
 ld de,$0702                    ; Position of live menu
 ld c,1                         ; Initial cursor position
robot_menu_loop:
 ld a,max_team                         ; A = Number of choices
 call MENU_LIVE                 ; Free-running menu system
 cp G_ALPHA
 ret z
 cp G_2ND
 ret z
 push bc
 push de
 ld b,c

robot_display_lable0 = $ + 2
 ld ix,_hp - 21
 ld de,21
robot_menu_loop1:
 add ix,de 
 djnz robot_menu_loop1

robot_display:
 ld hl,GRAPH_MEM+408                    ; Refresh the blackspace for stats
 ld de,GRAPH_MEM+409
 ld (hl),%11111111
 ld bc,359
 ldir
 set 3,(IY+$05)                 ; White text
 ld hl,table_robot_stats
 ld de,table_robot_stats_positions
 ld b,8
 call TEXT_ARRAY
 ld h,(IX)
 ld l,(IX+1)
 ld de,$220C
 call DMD2                      ; Display your HP
 ld l,(IX+2)
 ld h,0
 ld de,$280C
 call DMD2                      ; Display your EP
 ld h,(IX+6)
 ld l,(IX+7)
 ld de,$2228
 call DMD2                      ; Display your HP(max)
 ld l,(IX+17)
 ld de,$2250
 call DMD3                      ; Display your LEVEL
 ld l,(IX+19)
 ld h,(IX+20)
 ld de,$284C
 call DMD2                      ; Display your EXP LEFT
 ld b,(IX+11)
 ld de,$2E1F
 call LIBRARY                   ; Display your WEAPON
 ld b,(IX+16)
 ld de,$341B
 call LIBRARY                   ; Display your ARMOR
 res 3,(IY+$05)                 ; Default text setting
 pop de
 pop bc
 jp robot_menu_loop

table_atk_dfn_spe:
.dw str_atk
.dw str_dfn
.dw str_spe

table_atk_dfn_spe_positions:
.dw $0036
.dw $0044
.dw $0052

table_robot_stats:
.dw str_hp
.dw str_cp
.dw str_level
.dw str_of
.dw str_next
.dw str_weapon
.dw str_armor
.dw str_robot

table_robot_stats_positions:
.dw $2201 ; Position of "HP"
.dw $2801 ; Position of "EP"
.dw $2239 ; Position of "LEVEL"
.dw $221E ; Position of "of:"
.dw $2839 ; Position of "NEXT"
.dw $2E01 ; Position of "WEAPON"
.dw $3401 ; Position of "ARMOR"
.dw $0000 ; Position of "ROBOT"

;------------------------------------------------
;------------------------------------------------
;------------------------------------------------
;------------------------------------------------
;------------------------------------------------

;------------------------------------------------
Win_or_lose:
;
; This routine checks the battlefield for robots
; that are still in play.  If a player has no
; robots in play, one must be withdraw from
; storage.  If this player has no robots in
; storage he automatically looses.
;
;------------------------------------------------
 ld a,1                 ; See if the HOST has robots lefts
 call Select_leftmost   ; OUT: C=no robots left   NC=normal
 jr c,guest_win         ; If carry is set, the GUEST wins
 ld a,4                 ; Now check the GUEST
 call Select_leftmost   ; OUT: C = no robots left   NC=normal
 jr c,host_win          ; If carry is set, the HOST wins
 ret                    ; Return now if both parties still occupy the field
host_win:
 call False_win         ; Never end the match if a robot has FORCE ailment
 ret nc                 ; NoCarry = "fake win, continue fighting"
; ld b,4                 ; B = Opposing team number
; call False_win2        ; Goes back to ENDER if a bot is still in storage

 call QUEUE_CLEAR       ; Battle over!  Erase any left-over attacks
 ld a,(HOST)
 cp 1
 jr z,you_win
 jr you_lose

guest_win:
 call False_win         ; Never end the match if a robot has FORCE ailment
 ret nc                 ; NoCarry = "fake win, continue fighting"
; ld b,1                 ; B = Opposing team number
; call False_win2        ; Never end if a robot is still in storage

 call QUEUE_CLEAR        ; Erase any left-over attacks
 ld a,(HOST)
 cp 4
 jr z,you_win
you_lose:
 pop hl                 ; Fix the stack and prepare to "ret" out of BATTLE

 IM 1
faint:
 call TEMPS_CLEAR2
 call GRAPH_CLEAR
 call GRAPH_INVERT      ; Black screen
 ld hl,msg_collaspe
 ld de,$1D16
 set 3,(IY+$05)                 ; White text
 call DMX2
 res 3,(IY+$05)                 ; Regular text
 call CR_GRBCopy
 call CONFIRM_ALPHA
 ld a,(BOSS)
 cp 2                   ; 0 or 1 == 1-player game ; 254 or 255 == 2-players
 jr c,_1player_loss
_2player_loss:
 ret                    ; <<< Exit Battle
_1player_loss:
 pop hl                 ; Trash the call to BATTLE

 ld hl,GOLD + 1         ; Lose half your gold
 sra (hl)               ;
 dec hl                 ;
 rr (hl)                ;

 ld a,(SAFEHOUSE)       ; Recall last "full repair safehouse" map
 ld (MAP_NUM),a         ; Send player back to the safehouse

 ld a,$FF
 ld (find_entry_point),a        ; Put player on map's entry title

 xor a
 ld (BOSS),a
 jp LOAD_MAP

win_lose_window:
 ld hl,GRAPH_MEM+444    ; "WINNER" window.  This screen shows how much...
 ld de,$0C1A            ; ...EXP, and GOLD you've won
 call WINDOW            ;
 ret

you_win:
 pop hl                 ; Trash the win_or_lose call ; Prepare to exit

 call win_lose_window   ; Draw a window at the bottom

 ld hl,msg_winner       ; Display "VICTORY!"/"exp+ gold+ "
 ld de,$2902            ;
 call PARAGRAPH         ;

 ld a,(BOSS)
 cp 2                   ; 0 or 1 == 1-player game ; 254 or 255 = 2-player 
 jr c,_1player_win
_2player_win:
 halt                   ; Wait for IM2 to refresh the screen
 IM 1
 call CONFIRM_ALPHA
 call TEMPS_CLEAR2
 ret                    ; <<< Exit battle
_1player_win:

;------------------------
; EXP GAIN
;
; Exp = MAP_DIFFICULTY * 4
;
exp_award:
 ld a,(MAP_DIFFICULTY)
 ld e,a
 call exp_multiply
 call exp_multiply
 jr exp_gain_done

exp_multiply:
 rlc e
 ld a,e
 ld (exp_gain),a
 ret nc
max_exp_gain:
 ld a,255
 ld (exp_gain),a
 ret
exp_gain_done:
 ld a,(gain_nullify)    ; Some battles do not allow you to gain gold...
 or a                   ; ...Set gain_nullify to a non-zero...
 jr z,gold_award       ; ...value prior to battle to invoke this.
 call exp_multiply
 call exp_multiply
 call exp_multiply
 jr end_gains


;------------------------
; GOLD GAIN
;
; Gold = Sum of Robots left + RANK
;
;       ...or 100G for bosses
;
gold_award:
 ld a,(BOSS)
 cp 1
 jr nz,normal_gold
 ld c,boss_reward
 jr gold_add
normal_gold:
 ld hl,_squares
 ld c,0                 ; C == is gold counter
 ld b,max_robots        ; B == loop counter (check all 6 spaces)
gold_loop:
 ld a,(hl)
 or a                   ; Is anyone on this space?
 jr z,gold_lable0
 inc c                  ; Inc gold award for each robot left
 inc c
gold_lable0:
 inc hl
 djnz gold_loop
gold_add:
 ld a,(RANK)
 add a,c
 ld (gold_gain),a
 ld c,a
 ld hl,(GOLD)
 add hl,bc              ; B is always zero from "djnz" above
 ld (GOLD),hl

end_gains:
 ld hl,tbl_winner_values        ; Display everything that was gained
 ld de,tbl_winner_positions     ;
 ld b,2                         ;
 call VALUE_ARRAY               ;

;----
gain_rank:
 ld a,(BOSS)
 cp 1                           ; Was this a special battle?
 jr nz,promoting
 ld a,(RANK)
 cp 5                           ; Are you already at top rank?
 jr nc,promoting
 inc a
 ld (RANK),a
 call CONFIRM_ALPHA
 call win_lose_window

 ld hl,msg_rank_up
 ld de,$2602
 call PARAGRAPH
;---
promoting:

 xor a
 ld (BOSS),a                    ; Reset Boss indicator

default_win:

;---Save HP,EP and STATUS
 ld hl,_hp                      ; Save HP, EP, and STATUS
 ld de,HP
 ld bc,6
 ldir                           ; Copy HP,HPl,EP

 ld hl,_hp+21                   ; Save HP, EP, and STATUS
 ld de,HP1
 ld bc,6
 ldir                           ; Copy HP,HPl,EP

 ld hl,_hp+21+21                ; Save HP, EP, and STATUS
 ld de,HP2
 ld bc,6
 ldir                           ; Copy HP,HPl,EP

 call Promotions                ; Do "Level Ups" if applicable

 ld hl,exp_gain                 ; Reset all the battle variables, except TURNS
 ld de,gold_gain                ;
 ld bc,4 - 1                    ;
 ld (hl),0                      ;
 ldir                           ;
;---Save Item Info-------
 ld hl,_item_temp
 ld de,ITEM1
 ld bc,max_item_types
 ldir
 ld hl,_stock_temp
 ld de,STOCK1
 ld bc,max_item_types
 ldir
;------------------------

 halt                           ; Hold for interrupt for display purposes
 halt
 IM 1
 call CONFIRM_ALPHA             ; Hold
 call TEMPS_CLEAR2
 ret                            ; << Go back to game loop

;----------BATTLE TRACKING : DO NOT MOVE OR REARRANGE
exp_gain:
.db 0
gold_gain:
.db 0
gain_nullify:
.db 0           ; Bit 0 = No exp ; Bit 1 = No gold ;
turn_limit:
.db 0           ; Exit automatically after this duration
                ; 
turns:
.db 0           ; Number of turns taken so far
;-------------------

tbl_winner_values:
.dw exp_gain
.dw gold_gain

tbl_winner_positions:
.dw $2F12               ; Position for EXP GAIN
.dw $3515               ; Position for GOLD GAIN

;------------------------
False_win:
;
; If a player seems to
; have lost, check all the
; robots for FORCE.
;
; The match will not end
; until all robots are
; cured of FORCE.
;
; IN: -nothing-
; OUT: Carry == "This is a true win.  End the match"
;    NoCarry == "This is a fake win.  Continue fighting"
 ld hl,_squares
 ld b,max_robots        ; B = Number of spaces to inspect (6 spaces)
false_win_loop:
 ld a,(hl)
 cp 7                   ; Any robots affect by FORCE?
 ret nc
 inc hl
 djnz false_win_loop
 ret

;------------------------
;False_win2:
;
; If a player seems to
; have lost, check all the
; robots for STORE status.
;
; The match will not end
; until all stored robots
; have 0=HP.
;
; Force a non-random bot out
; of storage using the
; WITHDRAW system.
;
; B = team to check (1 or 4)
;   = loosing team
;-------------------------
; push bc                        ; Save team number
; ld hl,_hp - 21                 ; Start at robot data spaces
; ld de,21                       ; Robot data is 21 bytes each
;false_win2_loop0:
; add hl,de
; djnz false_win2_loop0
; ld b,max_team
;search_for_playable_teammates:
; call ld_de_hlPTR               ; HL -> Robot data ; DE = Robot's HP
; ld a,d
; or e                           ; Is HP non-zero?
; jr nz,have_playable_robot      ; If so, this was a false win
; ld de,21
; add hl,de
; djnz search_for_playable_teammates
; pop bc                         ; Fix the stack ; Restore team number
; ret                            ; This was a true win ; End the match
;have_playable_robot:
; call QUEUE_CLEAR               ; Cancel all pending attacks
; pop bc                         ; Restore team number
; pop hl                         ; Trash False_win2 call
; pop hl                         ; Trash Win_or_lose call
; ld a,(HOST)                    ; Get your host/guest status ; A = 1 or 4 only
; cp b                           ; Did YOU lose the robot just now? B = 1 or 4 only
; jp nz,LINK_TURN                ; If not, let other player withdraw a replacement
; jp _2P_BATTLE                  ; If so, chose a replacement like it's...
;                                ; ...your opening move

;------------------------------------------------
BG_restore:
;
; Clear the screen. Redraw the background picture
; and sync with any weather animations.
;
;------------------------------------------------
 IM 1
 call GRAPH_CLEAR
 ld de,(BACKGROUND)     ; Backgroup image to decompress (Varies by map)
 ld hl,GRAPH_MEM+96     ;
 call DecodeZCP         ;
 xor a
 ld (_YL),a
 IM 2
 ret

;------------------------------------------------
Promotions:                                     ;
Promotion_system:                               ;
;                                               ;
; Boost Robot Levels and Experience while       ;
; reducing TO GO values.  To promote a robot,   ;
; a series of comparisons should take place;    ;
;                                               ;
; - Generate random EXP gain                    ;
; - Subtract EXP byte from TOGO word            ;
; IF: If difference is positive                 ;
;  - Load difference into TOGO word             ;
;  - Proceed to next robot                      ;
; IF: If difference is negative                 ;
;  - Save the difference                        ;
;  - Increment robot level                      ;
;  - Square the new level, store result in TOGO ;
;  - Subtract saved difference as EXP           ;
;  - Promote stats                              ;
;  - Add new attacks if appropriate             ;
;  ^- Go to top                                 ;
;                                               ;
;------------------------------------------------
 ld ix,TOGO             ; Load promotion tracker word
 ld b,max_team          ; B = Number of bots on one team (3)
Promoter_loop:
 push bc                ; Save loop counter
 ld a,b
 ld (promoting_target),a        ; Save current robot number (polymorph)
 ld a,(ix-$02)          ; Check level
 or a                   ; Check for level = 0
 jr z,promoter_next     ; Skip empty data spaces
 ld h,(ix+$01)          ; HL = TO GO
 ld l,(ix)              ; 
 ld b,0                 ; Clear upper byte
 ld a,(exp_gain)        ; Load exp gain from last battle
 ld c,a                 
 sbc hl,bc              ; HL = EXP Required - Earned EXP
 push hl                ; Save signed "left over" points
 push af                ; Save sign
 call m,promote_now     ; Call if the difference was negative
 pop af                 ; Restore sign
 pop bc                 ; Restore signed "left over" points
 jp p,promoter_save
 add hl,bc              ; HL = New EXP Required + (- Leftover EXP)
 bit 7,h                ; Check sign bit of MSB (M flag will false-trigger)
 call nz,promote_now     ; Call if the sum was negative
promoter_save:
 ld (ix+$01),h          ; Update new TO GO
 ld (ix),l              ;

promoter_next:
 ld de,21               ; DE = Lenght of robot data.  21 bytes
 add ix,de              ; Point to next robot
 pop bc                 ; Restore loop counter
 djnz Promoter_loop
 ret                    ; Return to battlent.asm or map.asm

promote_now:
 call CONFIRM_ALPHA             ; Hold for readability
 call win_lose_window           ; Draw a window at the bottom
promoting_target = $ + 1
 ld b,$00
 ld hl,NAME2 + 11
 ld de,-11
attack_o_loop0:
 add hl,de
 djnz attack_o_loop0
 ld de,$2602
 call DMX2
 ld hl,str_level_up
 call DMX
attack_offers:
;---BOOST ALL STATS; ENFORCE THE STAT MAXIMUM
 ld a,stat_maximum              ; <-- Maximum for any EP-SPE stat
 cp (ix-$0B)
 jr c,dfn_up
 inc (ix-$0B)                   ; Enhance ATTACK
dfn_up:
 cp (ix-$0A)
 jr c,spe_up
 inc (ix-$0A)                   ; Enhance DEFENCE
spe_up:
 inc (ix-$09)                   ; Enhance SPEED
ep_up
 cp (ix+$10)
 jr c,hp_up
 inc (ix-$10)                   ; Enhance EPmax
hp_up:
 ld h,(ix-$0D)                  ; Load current HPmax
 ld l,(ix-$0C)
 ld de,hp_promote_rate
 add hl,de
 ld (ix-$0D),h                  ; Promote HPmax
 ld (ix-$0C),l
 inc (ix-$02)                   ; Increment level
 ld a,(ix-$02)                  ; TO GO = (New Level)^2
 ld b,a                         ;
 ld l,a                         ;
 ld h,0                         ;
 push hl                        ; "LD DE,HL"
 pop de                         ;
 dec b                          ; HL = (HL)^2
promotion_benchmark_set:        ;
 add hl,de                      ;
 djnz promotion_benchmark_set   ;
new_moves:
 ld a,(ix-$0F)                    ; Load Prereq points
 ld e,a
 cp 10
 jr z,get_new_move
 ret nc
 cp 5
 jr nz,new_moves_escape
get_new_move:
 ld a,(ix-$04)                   ; Load Type/class
 and %00000011
 ret z                          ; Destroyers don't get extra abilities
 sla a
 add a,last_elemental_power - 1 ; A = 1st class move available
 ld c,a                         ; Save new class move into <C>
 ld a,e                         ; Re-Load Prereq points
 cp 5
 jr z,_1st_new
_2nd_new:
 inc c
_1st_new:
 ld a,(ix-$06)
 or a
 jr nz,next_slot
 ld (ix-$06),c
 jr new_moves_escape
next_slot:
 ld a,(ix-$05)
 or a
 jr nz,new_moves_escape
 ld (ix-$05),c
new_moves_escape:
 inc (ix-$0F)
 ret



;------------------------------------------------
Vulture:                                        ;
;                                               ;
; The vulture program scans for robots with 0   ;
; HP that are still in play.  If a "dead" bot is;
; found, the vulture erases it's placeholder    ;
; in the _squares table.                        ;
;                                               ;
;------------------------------------------------
 ld hl,_squares                 ; Start at place holder table
 ld c,0                         ; Start at position 0
vulture_scan:
 ld a,c
 push hl                        ; Save placeholder pointer
 call Robot_lookup_0            ; IN: A= position to scan
 jr z,vulture_skip
 ld a,(hl)                      ; HL -> Robot's HP<msb>
 inc hl                         ; HL -> Robot's HP<lsb>
 ld e,(hl)
 or e                           ; CP HL,$0000
 jr nz,vulture_skip             ; Don't bother with robots with HP
 pop hl                         ; Restore pointer to placeholder
 push hl
 ld (hl),0                      ; Robot is 'dead' ; Zero-out placeholder
vulture_skip:
 pop hl
 inc hl                         ; Next table byte
 inc c                          ; Increment position tracker
 ld a,c
 cp 6                           ; Stop at rightmost square
 jr nz,vulture_scan
 ret




;------------------------------------------------
Animate:                                        ;
;                                               ;
; Display a series of AND-XOR masked sprites    ;
; at set coordinates in a delayed loop.         ;
;                                               ;
; IN: HL -> Table of sprites to draw            ;
;     BC == Position of sprite                  ;
;      E == Number of frames                    ;
;                                               ;
;-----------------------------------------------
 push de                        ; Save loop counter
 push hl                        ; Save pointer
 push bc                        ; Save position
 ROM_CALL($0033-$1A)            ; "LD HL,(HL)"
 call PutSprClp_special         ; Draw one frame
 ld a,animate_delay
 call DELAY             ; Slow for visibility ; Comphensates for interrupt slowness
 pop bc                         ; Restore position
 pop hl                         ; Restore pointer
 pop de                         ; Save loop
 inc hl                         ; Increment pointer
 inc hl                         ;
 dec e                          ; Decrement 
 jr nz,Animate
 xor a
 ld (int_redraw_off),a          ; Re-enable interrupt-based robot drawing
 jp Animation_end

tbl_fire_animation:
.dw spr_fire_0
.dw spr_fire_1
.dw spr_fire_2
.dw spr_fire_0                  ; Repeat, since this animation is 
.dw spr_fire_1                  ; usually repeated.  (Saves ~4 byte)
.dw spr_fire_2
tbl_cyclone_animation:
.dw spr_rain
.dw spr_rain_0
.dw spr_rain_1
.dw spr_rain
.dw spr_rain_0
.dw spr_rain_1
.dw spr_rain
.dw spr_rain_0
.dw spr_rain_1
tbl_laser_animation:
.dw spr_laser_0
.dw spr_laser_1
tbl_explode_animation:
.dw spr_explode_0
.dw spr_explode_1
.dw spr_explode_2
tbl_shield_animation:
.dw spr_shield1
.dw spr_shield2
.dw spr_shield3
tbl_plus_animation:
.dw spr_plus
.dw spr_plus
tbl_shock_field:
.dw spr_lightning
.dw spr_lightning
.dw spr_lightning2
.dw spr_lightning2
.dw spr_lightning
.dw spr_lightning
.dw spr_lightning2
.dw spr_lightning2



;------------------------------------------------
Field_setup:
;
; Clears and sets up _sqares table placeholders
;------------------------------------------------
 ld hl,_squares         ; Erase _squares table
 ld de,_squares+1       ;
 ld (hl),0              ;
 ld bc,max_robots - 1   ; (BC = 5)
 ldir                   ; 
 ld b,1                 ; Lowest robot placeholder number (use 1-6)
 ld hl,_squares
field_set_loop0:
 push hl                ; Save table pointer
 ld hl,_hp - 21
 ld de,21
 push bc                ; Save robot number
field_set_loop1:
 add hl,de
 djnz field_set_loop1
 pop bc                 ; Restore robot number
 ld a,(hl)              ; Does robot have non-zero HP?
 inc hl                 ;
 ld c,(hl)              ;
 or c                   ; "Does this robot have non-zero HP?"
 pop hl                 ; Restore table pointer
 jr z,no_placeholder
 ld (hl),b              ; Put a placeholder (e) in the _squares table
no_placeholder:
 inc b                  ; Move to next robot data space
 inc hl                 ; Move to next _square byte
 ld a,b
 cp 7
 jr c,field_set_loop0
 ret

;------------------------------------------------
;
LINK_TURN:
;
; Swap fight data between both calculators
;
; If there is a data conflict involving
; attacks, robot data, items or positions,
; the HOST data will overwrite the GUEST's data
;
; In the event of a link failure, both
; players must have the option of
; continuing the match against an AI opponent.
; This is called "disconnecting" in the game.
;
; Disconnected games cannot be "re-connected."
; You have to exit the game first.
;
; PROCEDURE
;
; Data involved in transfer:
; - attack_queues       (11x24) ; 264 bytes
; - robot data          (21x08) ; 168 bytes
; - item lists          (7x8x2) ; 112 bytes
; - squares             (1x6)   ;   8 bytes
;
;--------------------------------------------------------------
; AFTER YOUR ATTACKS ARE SELECTED...
;
; What the HOST does:
;
; - Display "Waiting : MODE disconnects"
;       - Run "synchronise:" to wait for other calculator to catch up
;       - Receive attack_queue
;               - Sort the queue, with HOST attacks on top
;       - Send sorted queue
;       - ENDER code (Turn ender)
;
; What the GUEST does:
;
; - Display "Waiting : MODE disconnects"
; - Run "synchronise:" to wait for other calculator to catch up
;       - Send attack_queue
;       - Receive sorted attack_queue
;       - ENDER code (Turn ender)
;--------------------------------------------------------------
; AFTER EACH ATTACK HITS....
;
; What the HOST does:
;
;  - Send robot_data
;  - Send item lists (and quantities)
;  - Send _squares table
;   - If successful:
;   * If fail bring up "Failed Link" "Continue?"
;    * If yes, set (BOSS) == 254
;    * If no, exit battle
;   * If no fail:
;    - Try to get attack_queue
;     * If fail bring up "Failed Link" "Continue?"
;     * If pass, jump to <ender>
;
; What the GUEST does:
;
;  - Receive attack_queue to attack_queue+132
;  - Receive robot_data to robot_data
;  - Receive item_temp to item_temp
;  - Receive _squares table
;   * If fail bring up "Failed Link" "Continue?"
;    * If yes, set (BOSS) == 254
;    * If no, exit battle
;   *If no fail:
;    - Try to send attack_queue
;     * If fail, bring up "Failed Link"
;     * If pass, jump to <ender>
; 
;------------------------------------------------
; call win_lose_window   ; Draw a window at the bottom

; ld hl,str_link_waiting ; Display "Waiting..."
; ld de,$2602            ;
; call DMX2              ;

; ld hl,str_disconnect   ; Display "Mode Disconnects"
; ld de,$2C08            ;
; call DMX2              ;
; halt                   ; Hold until IM2 has re-freshed the screen
; DI                     ; Now turn off IM2 to stop the constant re-freshing

; ld a,(HOST)                    ; "Am I the host or the guest?"
; cp 4
; jr z,guest_mode_transfers
;host_mode_transfers:
; call synchronise
; cp G_MODE
; jr z,Disconnect

; ld hl,attack_queue+99          ; ** RECEIVE GUEST'S ATTACKS
; ld b,99                        ;
; call blockReceive              ; Put guest's attacks on the bottom

; call Bubble_sort               ; Sort out the queue

; call RAND                      ; ** TRANSMIT SHARED RANDOM NUMBER TO GUEST
; ld hl,shared_rand              ;
; ld (hl),a                      ;
; ld b,1                         ;
; call blockSend                 ;

; ld hl,attack_queue             ; ** TRANSMIT COMPLETED ATTACK QUEUE TO GUEST
; ld b,(11*3*6)+6+6              ; Also send cursors (6 bytes) and cell data (6 bytes)
; call blockSend                 ;

; ld hl,_hp                      ; ** TRANSMIT ROBOT STATS
; ld b,21*6                      ;
; call blockSend                 ;

; EI  
; jp ENDER_no_sort

;guest_mode_transfers:
; call synchronise 
; cp G_MODE
; jr z,Disconnect

; ld hl,attack_queue             ; ** TRANSMIT ATTACK QUEUE TO HOST
; ld b,99                        ; Only send half
; call blockSend                 ;

; ld hl,shared_rand              ; ** RECEIVE SHARED RANDOM NUMBER TO GUEST
; ld b,1                         ;
; call blockReceive              ;

; ld hl,attack_queue             ; ** RECEIVE COMPLETED ATTACK QUEUE FROM HOST
; ld b,(11*3*6)+6+6              ; Also get cursors (6 bytes) and cell data (6 bytes)
; call blockReceive              ; All old data is overwritten by the HOST

; ld hl,_hp                      ; ** RECEIVE ROBOT DATA CORRECTIONS FROM HOST
; ld b,21*6                      ;
; call blockReceive              ;

;---Transmit Item List
;---Receive Item List
;---Receive robot data
;---Receive battlefield positions

; EI  
; jp ENDER_no_sort

;------------------------------------------------
;
;Disconnect:
; ld a,254                       ; Indicate "Terminated link game"
; ld (BOSS),a
; IM 2
; jp AI_TURN

;------------------------------------------------
;
AI_TURN:
;
; - Locate the first valid shooter
; - Create a list of possible actions
; - Choose a move at random
; - Target at random
;
;------------------------------------------------
; ld a,(BOSS)                    ; Check Battle type
; cp 255                         ; Is this a linked battle
; jp z,LINK_TURN                 ; Use linking procedures instead

 ld a,4                         ; AI plays as the guest by default
 ld hl,HOST
 cp (hl)                        ; See if this is a disconnect match
 jr nz,ai_is_guest
ai_is_host:
 ld a,1                         ; 1 == Host Player
ai_is_guest:
 ld (ai_team),a
 call Select_leftmost           ; Start with the leftmost AI robot
 jp c,ENDER

Ai_turn:

ai_turn:
 ld a,(SELECTOR)
 ld (_source_temp),a            ; Pre-set attack SOURCE
 call Robot_lookup              ; HL -> Robot data
 ld de,10
 add hl,de                      ; Locate AI robot's SPEED byte
 ld a,(hl)
 ld (_speed_temp),a             ; Pre-set attack SPEED

 ld hl,AI_choices2              ; Load table of options
 ld e,4                         ; E = Number of options
attack_randomizer2:
 call RAND                      ; A = (0-127)
 cp 64-1
 jr c,attack_ran_lable1
 dec e                          ; Decrement counter
 jr z,attack_ran_lable1         ; Exit by default after 4 iterations
 inc hl
 inc hl
 jr attack_randomizer2
attack_ran_lable1:
 ld de,AI_action_address        ; "LD (DE),(HL)
 ldi                            ; * Polymorph
 ldi                            ;
 ld de,12                       ; <-- Required for use of power attacks
AI_action_address = $ + 1
 jp $0000                       ; * Polymorphing jump


AI_choices2:                    ; Event probability
.dw MELEE_AI                    ; P[.] = 50.00%
.dw POWER1_AI                   ; P[.] = 25.00%
.dw POWER2_AI                   ; P[.] = 12.50%
.dw POWER3_AI                   ; P[.] = 12.50%


MELEE_AI:
 ld a,(SELECTOR)
 push af

 call Robot_lookup
 ld de,11
 add hl,de                      ; Locate WEAPON byte
 ld a,(hl)                      ; Load Weapon
 or a
 jr z,PASS_AI                   ; Some AI players have no melees
 ld (_serial_temp),a            ; A = Robot's melee weapon
 call Target_randomizer         ; Make random _destination_temp
 ld a,(SELECTOR)                ; SELECTOR = random victim
 ld (_destination_temp),a
 call Attack_adder              ; Launch the attack


PASS_AI:
 ld a,(ai_team)                 ; A == 1 or 4 (host or guest)
 ld (find_on_shift),a           ; Search for allies
 pop af
 ld (SELECTOR),a
 ld a,(ai_team)                 ; A == 1 or 4 (host or guest)
 call Select_right              ; Select next 
 jp c,ENDER
 jp ai_turn


POWER3_AI:
 inc de                         ; **Special Note: DE -must- be set to 12
POWER2_AI:                      ; prior to jumping to POWER1 POWER2 or POWER3
 inc de
POWER1_AI:
 ld a,(SELECTOR)
 push af
 push de                        ; Save Power Attack Number
 call Robot_lookup              ; HL -> Robot data
 pop de                         ; Restore Power Attack Number
 add hl,de
 ld a,(hl)
 ld (ai_power_choice),a         ; * Polymorph.  [See about
 or a                           ; Is there any attack to actually use?
 pop bc                         ; Pop AF as "BC" to leave flags unchanged
 jp z,MELEE_AI                  ; If not, do a simple melee move instead
;------- CHECK FOR MINIMUM EP----
 push bc
 call Robot_lookup              ; Get robot stats
 inc hl                         
 inc hl                         ; HL -> EP byte
 ld a,(hl)
 cp ep_rate                     ; Can robot at least launch a weak attack?
 pop bc
 jp c,MELEE_AI                  ; If not, do a simple melee move instead
;--------------------------------

 push bc                        ; Re-save "BC" to be poped as "AF" later
ai_power_choice = $ + 1
 ld a,$00                       ; * Polymorph.  Load 
 ld (_serial_temp),a            ; A = Robot's power attack
 call Target_randomizer         ; Make random _destination_temp

 ld a,(SELECTOR)                ; SELECTOR = random victim
 ld (_destination_temp),a

random_power:
 call RAND
 and %00000011
 ld e,a
 ld a,(RANK)
 cp e
 jr c,random_power
 inc e
 ld a,e
 ld (power),a


 call Attack_adder              ; Launch this atacks
 ld a,(attack_queue)
 ld a,(ai_turn)                 ; A == 1 or 5 (host or guest)
 ld (find_on_shift),a
 pop af
 ld (SELECTOR),a
 ld a,(ai_turn)                 ; A == 1 or 5 (host or guest)
 call Select_right
 jp c,ENDER
 jp ai_turn


Target_randomizer:
 ld a,(_serial_temp)
 ld b,a
 call Attack_lookup
 bit 3,(ix+$05)                 ; Is this a benign attack?
 jr z,target_rand_opponent      ; If not, target an enemy
target_rand_ally:
 ld a,(ai_team)                 ; Target guests only ; A == 1 or 5
 jr target_rand_now
target_rand_opponent:
 ld a,(HOST)                    ; Target human only
target_rand_now:
 call Select_leftmost           ; Find primary target.  Setup (find_on_shift)
target_random_hosts:
 call RAND                      ; Put a random bit in carry. (A is random too)
 ret c                          ; 
 call Select_right              ; (find_on_shift) was already set --^
 jr target_random_hosts



Bubble_sort:
 ld b,18 - 1
bubble_sor_loop1:
 push bc
 ld hl,attack_queue+11
 ld de,attack_queue
 ld b,18 - 1                    ; B = Number of _attack_queue rows - 1
bubble_sor_loop0:
 ld c,(hl)
 ld a,(de)
 cp c                           ; Is the second attack faster than the first?
 call c,Swap                    ; Swap ONLY if (HL) > (DE). NOT if equal.
 push bc
 ld b,11                        ; B = Length of attack queue row
bubble_lable0:
 inc hl
 djnz bubble_lable0
 ld b,11
bubble_lable1
 inc de
 djnz bubble_lable1
 pop bc
 djnz bubble_sor_loop0
 pop bc
 djnz bubble_sor_loop1
 ret

Swap:
 push hl
 push de
 push bc
 ld b,11
swap_a_p_r_loop0:
 ld c,(hl)
 ld a,(de)
 ld (hl),a
 ld a,c
 ld (de),a
 inc hl
 inc de
 djnz swap_a_p_r_loop0
 pop bc
 pop de
 pop hl
 ret

;------------------------------------------------
ENDER:
;
; Ender should be Jumped to; not called
;
; - looks at attack queue
; - ignore attacks with SPEED = 0
; - throws out invalid attacks
; - redirects attacks if the target has burned out
; - applies damage points
; - applies shield logic
; - launches counter-attacks
; - subtracts your charge
; - run hit-or-miss code for inaccurate attacks
; - announce what attack was used
; - display damage report
; - do animations
;
;------------------------------------------------
 call Bubble_sort
ENDER_no_sort:
time_healer2:                   ; Time-healer2 undoes the effect of FORCE
 ld hl,_squares
 ld b,max_robots                ; B = Number of sqaures to mask (6 of them)
time_h2_loop:
 ld a,(hl)                      ; Load current placeholder value (0-16)
 or a                           ; Check for empty spaces
 jr z,time_h2_lable0            ; Skip over if a = 0
 cp 7
 jr c,heal_normal
heal_special:
 sub 6
 ld (hl),a                      ; Update
heal_normal:
time_h2_lable0:
 inc hl                         ; Next placeholder
 djnz time_h2_loop
;----
 call CLEAR_CENTER
;----
Ender:
;----

;----
ender:
 ld a,(attack_queue+3)   ; Get DESTINATION
 cp 254                 ; UNPACKING a BOT?
 jp z,WITHDRAW          ; Is someone new going to be in play?

 ld a,(attack_queue+2)   ; Check the next SERIAL byte
 or a                   ; 0-SERIAL indicates end-of-turns
 jp z,TERMINATOR        ; End the turn

 ld a,(attack_queue)    ; Check the SPEED byte
 or a                   ; Attacks with 0 speed are null
 jr z,null_attack

 ld a,(attack_queue+1)  ; Get SOURCE
 cp 255                 ; Omit Spread attacks
 jr z,not_null_attack
;---CHECK FOR NEWLY-APPLIED FORCE
 ld e,a
 ld d,0
 ld hl,_squares
 add hl,de
 ld a,(hl)
 cp 7                   ; Is robot's placeholder 7 or higher?
 jr nc,null_attack      ; If so, it is under FORCE and can't attack
;---CHECK IF SQUARE IS OCCUPIED
; ld a,(attack_queue+1)  ; Get Source
; call Robot_lookup_0    ; See if this square is still occupied [don't remove]
 call Attack_source_lookup
 jp nz,not_null_attack  ; Don't jump if the shooter left ;
;---NULLIFY ATTACK
null_attack:
next_attack:
 call Ender_append_lines        ; Erase this attack...
 ld a,(attack_queue+1)          ; Get next attack's source
 cp 255                         ; Part of null chain?
 jr nz,null2                    ; If not, don't skip next attack
 call Ender_append_lines        ; If so, skip
null2:
 ld a,(attack_queue+1)           
 cp 255                         ; Part of a large null chain?                                
 jr nz,null3                    ; If not, don't skip next attack
 call Ender_append_lines        ; If so, skip
null3:
 jr ender                       ; ...and move ahead
not_null_attack:
 ld a,(attack_queue+1)   ; Determine SOURCE byte
 cp 255                 ; Is this a spread attack?
 jr z,end_announcement  ; 255-SOURCE attacks are unannounced

 call Robot_lookup_0    ; See if this square is still occupied
 ld de,5                ; Use Robot_lookup (above) to check for stasis
 add hl,de
 ld a,(hl)
 and %00001100
 jr nz,null_attack      ; Robot's with stasis can't attack

;---------------ATTACK ANNOUNCEMENT
 call CLEAR_CENTER

 ld a,(attack_queue+1)
 call Name_lookup
 ld de,$2D03
 call DMX2              ; Display "[Name]"

 ld hl,str_used         ; Display the word " used " with spaces included
 call DMX               ; Display "used"

 ld a,(attack_queue+2)   ; Lookup attack serial number
 ld b,a
 ld de,(GRAF_CURS)
 call LIBRARY           ; Now show the name of the attack used

 ld a,(attack_queue+10) ; Get attack's power level
 and %00000111          ; Mask off animation and bonus bits
 jr z,normal_power      ; Melee attacks and items have no greek symbol
 ld hl,msg_space
 call DMX               ; Show dash
 ld hl,str_greeks - 2   ; HL -> Greek characters strings
 sla a                  ; x2 because the chars are seperated by zeros
 ld e,a
 ld d,0
 add hl,de              ; HL -> greek character
 call DMX               ; Display greek letter (alpha - ohmega)
normal_power:
end_announcement:
;---------------DAMAGE REPORT
 call Charge_check      ; - Does the shooter have enough charge for attack?
 call Item_deduct       ; - Did the shooter use an item?
 call Shooter_ailment   ; - Does shooter have an ailment that may interfere?
 call Do_Animation      ; - Draw animations, delays
 call Hit_or_miss       ; - Cancel some inaccurate attacks

 ld a,(attack_queue+2)   ; Check attack SERIAL
 cp teleport            
 jr z,occupied_check_skip ; Teleportation can hit empty squares
 cp rebuild
 jr z,occupied_check_skip ; ...so can "rebuild"
occupied_check:
; ld a,(attack_queue+3)   ; Check attack DESTINATION
; call Robot_lookup_0    ; Is this destination occoupied?  NZ=yes Z=no
 call Attack_dest_lookup
 ld a,(attack_queue+3)   ; Check attack DESTINATION
 jp z,miss0             ; Don't inflict damage to blank squares
occupied_check_skip:
 ld hl,attack_queue+8    ; Get ALGORITHM
 ld a,(hl)
 and %00001111          ; Mask off ADVANTAGE byte
 cp 1
 jp z,HEAL              ; ADVANTAGE-1 attacks will heal
 cp 2
 jp z,NULL              
 cp 3
 jp z,FLARES
 cp 4
 jp z,STAT_BOOST
 cp 5
 jp z,PULSE
 cp 6
 jp z,SHIELD
 cp 7
 jp z,RENEW
 cp 8
 jp z,FORCE
 cp 9
 jp z,FLUX
 cp 10
 jp z,HP_PLUS_50PERCENT
 cp 11
 jp z,STASIS
 cp 12
 jp z,TELEPORT
 cp 13
 jp z,REBUILD
 cp 14
 jp z,CLONE
 cp 15
 jp z,RAPID_SPE

HARM:
 call Target_ailment    ; Check for ailments that might affect attack
 call Damage_setup      ; Set IX (the damage counter) to attack's damage
;---Power Level Bonus---
 ld a,(attack_queue+10) ; Get attack's power level
 inc a                  ; Adjust (to get the right flags)
 push ix                ; LD DE,IX
 pop de                 ; Prepare to multiply damage
 bit 1,a                ; "Is this a Alpha/Beta or Epsilon/Ohmega attack?..."
 jr z,no_bit1_bonus
 add ix,de
no_bit1_bonus:
 bit 2,a                ; "Is this a Gamma/Delta or Epsilon/Ohmega attack?"
 jr z,no_bit2_bonus
 add ix,de
 add ix,de              ; Result:
no_bit2_bonus:          ; alpha = x2 beta = x3 gamma = x4
;---Check for Spreads----
 ld a,(attack_queue+1)  ; Get attack's source
 cp 5+1                 ; Is Source 255?  No ATK bonus for spread attacks!
 jr c,atk
 ld a,(attack_queue+6)  ; For wide attacks ,Get secondary shooter atk
 ld e,a
 ld d,0
 jr atk2
;---Attack Bonus---------
atk:
 call Robot_lookup_0    ; DAMAGE = ATTACK STRENGTH
 ld de,8                ; HL -> Shooter's HP
 add hl,de              ; HL -> Shooter's ATK
 ld e,(hl)              ;
atk2:
 add ix,de              ; D is still clear from above
;---Defense Penalty------
dfn::
; ld a,(attack_queue+3)  ; Get destination
; call Robot_lookup_0
 call Attack_dest_lookup
 ld de,9                ; HL -> Target's HP
 add hl,de              ; HL -> Target's DFN
 ld e,(hl)
 call NEG_DE            ; DE = -DE
 add ix,de
;---Advantage Bonus------
bonuses:
; ld a,(attack_queue+3)  ; Get Destination
; call Robot_lookup_0    ; HL -> Robot data
 call Attack_dest_lookup
 ld de,15
 add hl,de              ; HL -> TypeClass byte
 ld e,(hl)
 ld a,(attack_queue+8)  ; Get Advantage/Algorythm byte
 xor e                  ; Set mis-matched bits to "1"
 and %11110000          ; Mask lower
 jr nz,advantage_escape ; Victim's type didn't match attacks' advantage?  Jump
 ld hl,str_advantage
 call line2_message
 call line_message_pause
 call x2_damage2        ; Double IX
 jp armor_escape        ; An advantage bonus circuments any armor present
advantage_escape:
;---Luck Bonus-----------
 ld a,(attack_queue+10) ; Get Animation/Bonus/Spread byte
 bit 3,a                ; Check BONUS bit
 jr z,luck_bonus_escape ; No luck bonus allowed?  Jump.
; call RAND_SPECIAL      ; A = {0-127}
 call RAND
 cp luck_chance         ; Luck bonus probability (right now this is 15)
 jr nc,luck_bonus_escape; Not lucky?  Jump
 ld hl,str_luck
 call line2_message
 call line_message_pause
 call x2_damage2        ; Multiply IX and show "x2 Hit"
 jr armor_escape        ; Lucky shots circumvent any armor present
luck_bonus_escape:
;---Armor Penalty--------
; ld a,(attack_queue+3)  ; Get Target's info
; call Robot_lookup_0    ; HL -> Robot stats
 call Attack_dest_lookup
 ld de,16
 add hl,de              ; HL -> Armor byte
 ld a,(hl)              ; Get armor type
 sub first_armor        ; Normalize
 jp m,armor_escape      ; Ignore armor-free bots
 sra a                  ; Shift armor types in to carry
 ld b,a                 ; Save armor byte for later
 ld a,(attack_queue+2)  ; Get current attack type
 jr c,energy_armor      ; Shifted out a 1?  Jump!
melee_armor:
 cp last_melee+1
 jr nc,armor_escape
 jr do_armor
energy_armor:
 cp first_power
 jr c,armor_escape
do_armor:
 inc b                  ; Now B = (1-3) Number of times to divide damage
do_armor_loop:
 call half_damage2      ; Divide IX by two
 djnz do_armor_loop
 call line_message_pause
 ld hl,str_armor
 call line2_message
armor_escape:
;---Shield Check---------
 call Shield_check      ; Divide IX by two if victim has shield
;---Arson Ability--------
 ld a,(attack_queue+2)  ; Get attack serial
 cp library_arson       ; Arson?
 jr nz,arson_escape
 call check_negate_mode ; Should arson burn or freeze now?
 jp nz,freeze           ; Freeze if negated
 jp burn                ; Burn if normal
arson_escape:
;---Get Damage Sign-----
damage_ready:
 push ix
 pop hl                 ; HL = total damage
 ld de,0
 xor a                  ; Clear carry
 sbc hl,de              ; Update Flags
 jr z,minimum_hit       ; <-- Added jump here since zero is a positive number
 jp p,harm_non_zero     ;
minimum_hit:
 ld hl,minimum_damage   ; <-- Normally this is 1 to add difficulty
harm_non_zero:
;---Display Damage-------
 push hl
 pop ix                 
 call Damage_report     ; Display numerical damage
;---Apply Damage---------
; ld a,(attack_queue+3)  ; Get Victim
; call Robot_lookup_0    ; HL -> Robot's data space
 call Attack_dest_lookup
 push hl                ; Save pointer
 ROM_CALL($0033-$1A)    ; "LD HL,(HL)"
 call HL_SWAP
 push ix                ; "LD BC,IX"
 pop bc                 ; BC = HP Damage ; IX = HP Damage
apply_damage:
 xor a                  ; Clear carry flag
 sbc hl,bc              ; Subtract HP!
 jp p,harm_not_fatal    ; Skip if not negative
harm_fatal:
 ld hl,$0000            ; "You have 0 HP!"
harm_not_fatal:
remaining_hp:
 ex de,hl               ; DE = Remaining HP
 pop hl                 ; Restore pointer
 ld (hl),d
 inc hl
 ld (hl),e              ; Update HP ; HL -> HP byte 2
;---Special Abilities----
 ld a,(attack_queue+8)  ; Get advantage/algorhythm byte
 and %11110000          ; Mask off algorhythm
 cp $10                 ; << ICE ATTACK?
 jr z,ice_ability
 cp $20                 ; << SPARK ATTACK?
 jr z,spark_ability
 cp $30                 ; << FIRE ATTACK?
 jr z,fire_ability
 cp $40                 ; << BOMB ATTACK?
 jr z,bomb_ability
 cp $50                 ; << LASER ATTACK?
 jr nz,special_damage_end
laser_damage:
 ld de,5
 add hl,de              ; HL -> HPmax
 push hl                ; Save HPmax pointer
 ROM_CALL($0033-$1A)
 call HL_SWAP
 sbc hl,bc
 ld d,h
 ld e,l                 ; DE = Remaining HPmax
 pop hl
 ld (hl),d
 inc hl
 ld (hl),e
 jr special_damage_end

;---Defense Destruction-
bomb_ability:
 ld de,8                        ; Blasts will affect DFN
 add hl,de                      ; Offset target address to DFN byte
 ld a,(hl)                      ; Load victim's DFN
 sub blast_damage               ; Apply damage to DFN stat
 ld (hl),a                      ; Save new DFN
 jp p,special_damage_end        ; Still positive?  Jump.  No need to adjust.
 ld (hl),0                      ; Set to zero if sub result was negative
 jr special_damage_end

;---Spark Ability--------
spark_ability:
; call RAND_SPECIAL              ; Time to apply random freeze?
 call RAND
 cp poison_chance
 jr nc,special_damage_end       ; If not, jump
 ld a,(attack_queue+10)         ; Get current attack's power level
 and %00000111                  ; Mask of non-power level bits
 jr z,special_damage_end        ; No repeated aftershocks
 inc hl                         ; Now HL -> Victim's Energy
 ld a,(hl)
 ld de,7
 add hl,de                      ; Now HL _> Victim's DFN
 add a,(hl)                     ; A = Victim's EP + DFN
 ld (attack_queue+4),a          ; Overwrite lower damage byte
 ld a,255
 ld (attack_queue+1),a          ; Make it a spread attack so it's not reannounced
 inc a                          ; A = 0
 ld (attack_queue+6),a          ; Clear shooter's ATK byte
 ld a,(attack_queue+10)         ; Make the power level 0
 and %11110000                  ;
 ld (attack_queue+10),a         ;
 jp ender

;---Fire Ability---------
fire_ability:
; call RAND_SPECIAL              ; Time to apply random freeze?
 call RAND
 cp poison_chance
 jr nc,special_damage_end       ; If not, jump
burn:
; ld a,(attack_queue+3)  ; Get Target's info
; call Robot_lookup_0    ; HL -> Target's Data
; ld de,5
; add hl,de              ; HL -> _statush
; ld a,(hl)
 call Attack_dest_status_lookup
 or %00010000           ; Set BURN flag
 and %01111111          ; Clear ICE flag (Can't be burned AND frozen, duh!)
 ld (hl),a              ; Update robot's status
 call Burn_message
 jr special_damage_end

;---Freeze Ability-------
ice_ability
; call RAND_SPECIAL              ; Time to apply random freeze?
 call RAND
 cp poison_chance
 jr nc,special_damage_end       ; If not, jump
freeze:
; ld a,(attack_queue+3)          ; Get attack DESTINATION
; call Robot_lookup_0
; ld de,5
; add hl,de                      ; HL -> Target's status byte
; ld a,(hl)
 call Attack_dest_status_lookup
 and %11101111          ; Stop burns (Can't be burned and frozen at once!)
 or %10000000           ; Apply ice ailment
 ld (hl),a

special_damage_end:
 call Vulture           ; Remove 'dead' bots
 call Win_or_lose       ; Check for win/lose condition
 call Ender_append_lines
 jp ender

HEAL:
 call check_negate_mode
 jp nz,HARM
 call Damage_setup      ; Set IX (the damage counter) to attack's damage
 ld a,$FF                       ; "Show + sign too"
 ld (damage_report_sign),a      ;
 call Damage_report             ; Display numerical "damage"
; ld a,(attack_queue+3)          ; Check DESTINATION
; call Robot_lookup_0            ; HL -> Robot data space
 call Attack_dest_lookup
 push hl                        ; Save pointer
 ROM_CALL($0033-$1A)            ; Now HL = Current HP
 call HL_SWAP
 push ix                ; "LD BC,IX"
 pop bc                 ;
 add hl,bc              ; HL = Current HP + Heal Amount = New HP
 ex de,hl               ; DE = New HP
 pop hl                 ; Restore pointer
 push hl                ; Re-save pointer
 ld (hl),d              ; HL -> HP<msb>
 inc hl
 ld (hl),e              ; HL -> HP<lsb>
 ld bc,5                ; Save <DE> by using <BC> instead this time
 add hl,bc              ; HL -> HPmax<msb>
 ROM_CALL($0033-$1A)    ; "LD HL,(HL).  HL == maximum HP
 call HL_SWAP
 call CP_HL_DE          ; Is Heal > HPmax ?
 ex de,hl               ; DE = HPmax
 pop hl                 ; Restore pointer
 jr nc,no_hp_cap
hp_cap:
 ld (hl),d              ; Cap off HP
 inc hl
 ld (hl),e
no_hp_cap:
 call Ender_append_lines
 jp ender

NULL:
 ld a,(attack_queue+2)  ; Get attack serial
 cp add                 ; [see robot.h] ; Check for Add+... combo
 jr z,exempt_one_item
 cp negate              ; [see robot.h] ; Check for Negate+... combo
 jr z,negate_active
null_lable0:
 call damage_report_end ; Delay, clear cursors, etc.
attack_over:
 call Ender_append_lines
 jp ender

exempt_one_item:
 ld a,1
exempt_now:
 ld (item_deduct_exempt),a  ; Number of "free" items following this attack
 jr null_lable0
negate_active:
 ld (item_negate),a
 jr null_lable0

check_negate_mode:      ; OUT: NZ = negate current item  Z = don't negate
item_negate = $ + 1
 ld a,$00
 or a
 ld a,0
 ld (item_negate),a
 ld (attack_queue+6),a  ; Nullify shooters 1/2 ATK bonus
 ret


FLARES:
 ld hl,_hp3 + 18        ; See if bot dataspace is in use
 ld b,3                 ; Number of spaces to check before giving up
 ld c,3                 ; Use C to determine where new bot data will appear
beckoning_search:
 ld a,(hl)
 or a                   ; Have room to introduce a new robot?
 jr z,beckon_now
 ld de,21               ; 21 is length of whole robot data space
 add hl,de              ; Jump the next robot
 inc c                  ;
 djnz beckoning_search
 ld a,255               ; Change attack's SOURCE to 255 so it's not...
 ld (attack_queue+1),a  ; ...announced twice
 ld a,$F0
 ld (attack_queue+8),a  ; Change Flare's to "No Advantage(F), "HARM(0)"
 jp ender
beckon_now:
 call Hit_or_miss       ; Half accuracy
 ld b,flare_responder   ; Set B=7 to summon an N-FORCER bot [see enemies.asm]
 push bc                ; Save robot's data space location number {4 thru 6}
 call Opponent_loader   ; ( "C" has already been set ) ; New bot's data ready

 ld a,254               ; 254 = Code for "UNPACK this robot to the field"
 ld (attack_queue+3),a  ;
 pop bc                 ; Restore robot's data space location number
 ld a,c
 ld (attack_queue+1),a  ; This byte tells which robot is arriving
 jp ender               ; Redo attack, but this time, it's an UNPACK

CLONE:
 call check_negate_mode
clone0:
 ld a,(attack_queue+3)
 jp nz,NEGATED_CLONE
 call Robot_lookup_0    ; OUT: B == robot number
 ld a,b
 dec a
 ld (attack_queue+1),a          ; Copy for withdraw's code
 inc a
 jp withdraw0                   ; Do the "withdraw" code that doesn't...
                                ; ...after the victim's "in storage" flag.
NEGATED_CLONE:
 ld e,a
 ld d,0                 ; Clear upper byte
 ld hl,_squares         ; Go to placeholders table
 add hl,de              ; Move to approprite placeholder
 ld a,(hl)
 cp 7                   ; Has robot already been affected by FORCE?
 jr nc,negated_clo_escape
 add a,6                ; Indicate FORCE by raising the placeholder +6
 ld (hl),a
negated_clo_escape:
 xor a                  ; Set zero flag; avoid infinate loop at clone0 lable
 jr clone0              ; Do clone and force at the same time


STAT_BOOST:
 call Shield_check      ; Reflect harmful stat-changers (gravity, etc)
; ld a,(attack_queue+3)  ; Get destination
; call Robot_lookup_0
 call Attack_dest_lookup
 jr z,stat_boost_escape
 ld a,(attack_queue+4)  ; Get stat instruction (offset) byte from DAMAGE byte
 ld e,a
 ld d,0                 ; Clear upper ; Prepare to add
 add hl,de              ; HL -> Stat to boost
 ld b,(hl)              ; Save current stat 
 call check_negate_mode ; Negating this item?
 jr nz,stat_negate
 ld a,(attack_queue+5)   ; Get stat magnetude instruction byte
stat_adjust:
 add a,b                ; Boost stat
 jp m,stat_zero
 ld (hl),a              ; Update stat
stat_boost_escape:
 call damage_report_end
 call Ender_append_lines
 jp ender
stat_negate:
 ld a,(attack_queue+5)
 neg
 jr stat_adjust
stat_zero:
 ld (hl),0              ; Zero-out the stat
; ld a,(attack_queue+3)  ; Get attack destination
; call Robot_lookup_0
 call Attack_dest_lookup
 jr pulse_hp_down0      ; To price for have a 0-stat is half your HP

PULSE:
 call Shield_check      ; Pulses can be blocked by shields
; ld a,(attack_queue+3)  ; Get attack destination
; call Robot_lookup_0    ; HL -> Robot data space
 call Attack_dest_lookup
 inc hl
 inc hl                 ; HL -> Victim's current EP
.db $CB,$3E             ; srl (hl) ; Divide EP by two (Zero flag affected)
 jr z,pulse_hp_down
 call Ender_append_lines
 jp ender
pulse_hp_down:          ; PULSE does damage if victim had no EP
 dec hl
 dec hl
pulse_hp_down0:
 call ld_de_hlPTR       ; DE <- (HL) little endian
 srl e                  ; 16-bit divide HP by two now
 rr d                   ;
 call ld_hlPTR_de       ; (HL) <- DE  little endian
 ld a,d                 ; Swap high-low bytes
 ld d,e                 ;
 ld e,a                 ;
 push de                ; LD IX,DE
 pop ix                 ;
 call Damage_report     ; Show how much damage was caused by PULSE
 call Ender_append_lines
 jp ender

SHIELD:
; ld a,(attack_queue+3)  ; Get DESTINATION
; call Robot_lookup_0    ; HL -> Robot's data space
; ld de,5
; add hl,de              ; HL -> Robot status(2) byte
; ld a,(hl)
 call Attack_dest_status_lookup
 or %00000011           ; Add three shield points
 ld (hl),a              ; Add the shield
 call damage_report_end
 call Ender_append_lines
 jp ender

RENEW:
 call check_negate_mode
 jp nz,STASIS
; ld a,(attack_queue+3)  ; Get DESTINATION
; call Robot_lookup_0    ; HL -> Robot's data space
 call Attack_dest_lookup ; Out: Z = no one here  NZ = robot is here
 jr z,restore_escape
 ld de,5
 add hl,de              ; Now HL -> Robot's STATUS2 byte
 ld (hl),0              ; Clear STATUS byte to erase ailments and shields
 add hl,de              ; Got to SPEED byte
 set 0,(hl)             ; Cure paralysis by guaranteeing 1 speed point
restore_escape:
 call damage_report_end
 call Ender_append_lines
 jp ender

FORCE:
 call Shield_check      ; A shield will reflect this attack
 ld a,(attack_queue+3)  ; Get DESTINATION
 ld e,a
 ld d,0                 ; Clear upper byte
 ld hl,_squares         ; Go to placeholders table
 add hl,de              ; Move to approprite placeholder
 ld a,(hl)
 cp 7                   ; Has robot already been affected by FORCE?
 jr nc,force_escape
 add a,6                ; Indicate FORCE by raising the placeholder +6
 ld (hl),a
force_escape:
 call damage_report_end
 call Ender_append_lines
 jp ender

FLUX:
 call Shield_check      ; Flux will be reflected by shields
; ld a,(attack_queue+3)  ; Get DESTINATION
; call Robot_lookup_0    ; HL -> Target's HP
 call Attack_dest_lookup 
 ld (hl),0
 inc hl
 ld (hl),0
 call damage_report_end
 call Vulture
 call Win_or_lose
 call Ender_append_lines
 jp ender

HP_PLUS_50PERCENT:
; ld a,(attack_queue+3)  ; Get destinatino
; call Robot_lookup_0    ; HL -> Robot data space
 call Attack_dest_lookup
 ld de,6
 add hl,de              ; HL -> Max HP
 call ld_de_hlPTR       ; ld de,(hl) ; DE = HPmax
 srl e                  ;
 rr d                   ; DE = 50% HPmax
 ld a,d
 ld d,e
 ld e,a
 ld hl,attack_queue+4   ; HL -> Damage word for heal attack
 call ld_hlPTR_de       ; ld (hl),de
 jp HEAL                ; Redo attack as a healing move


STASIS:
 call Shield_check      ; Stasis will be blocked by shields
; ld a,(attack_queue+3)   ; Get DESTINATION
; call Robot_lookup_0    ; HL -> Robot's data space
; ld de,5
; add hl,de              ; HL -> Robot status(2) byte
; ld a,(hl)              ; Get current robot status
 call Attack_dest_status_lookup
 or %00001100           ; Add the stasis barrier to any existing ailments
 ld (hl),a              
 call damage_report_end
 call Ender_append_lines 
 jp ender


TELEPORT:
 call Shield_check      ; Teleport will be blocked by shields
 ld a,(attack_queue+1)  ; Get sources' location (0-5)
 inc a                  ; Prepare loop counter
 ld b,a                 ; B = (1-6)
 ld hl,_squares - 1
teleport_subject1:
 inc hl
 djnz teleport_subject1
 ld a,(attack_queue+3)   ; Get destinations' location
 ld (teleport_victim),a
 inc a
 ld b,a
 ld de,_squares - 1
teleport_subject2:
 inc de
 djnz teleport_subject2
 ld a,(de)              ; (HL)= teleporters marker   (DE)=teleportees marker
 ldi                    ; [Subject 1] ---> [Subject2]
 dec hl                 ; Undo the "inc hl" in LDI
 ld (hl),a              ; --TELEPORT COMPLETE

teleport_victim = $ + 1
 ld a,$00
 call Robot_lookup_0
 jr z,teleport_escape           ; No redirects for blank square exchanges
 ld hl,attack_queue+1+11        ; Check attack SOURCES
 ld b,18 - 1                    ; B = Number of attack queue rows - 1
teleport_victim_find:
 ld a,(teleport_victim)
 cp (hl)
 jr nz,teleport_no_redirect
 inc hl                         ; Get SOURCE
 ld a,(hl)
 or a
 jr z,teleport_escape   ; Exit at end-of-attack bank
 inc hl                 ; Get DESTINATION
 ld a,(hl)              ; Save
 dec hl
 dec hl
 ld (hl),a              ; NEW SOURCE <-- DESTINATION
 jr teleport_escape
teleport_no_redirect:
 ld de,11
 add hl,de
 djnz teleport_victim_find
teleport_escape:
 call damage_report_end
 call Ender_append_lines
 jp ender


REBUILD:
 ld a,(attack_queue+3)   ; Get DESTINATION (Robot to revive) (1-6)
 ld b,a                  ; Setup loop counter
 dec a                   ; <-- Reduce this for "withdraw" routine
 ld (attack_queue+1),a   ; <-- Prepare to withdraw robot "<A>" (0-5)
 ld hl,_battle_temp - 21
 ld de,21
rebuild_locating:
 add hl,de
 djnz rebuild_locating
 inc hl                 ; HL -> Revived robot's HP(low) byte
 ld a,(hl)
 or a                   ; Does robot have 0 HP?
 ld (hl),rebuild_heal
 jp z,withdraw0         ; Jump if robot had 0 HP
reinforce:
 dec hl
 call ld_de_hlPTR       ; If robot already has HP, reinforce it's HP
 ex de,hl               ; by 50 points (with no HP limit)
 ld b,a
 ld c,0
 add hl,bc
 ex de,hl
 call ld_hlPTR_de
 call damage_report_end ; Delay, clear cursors, etc.
 call Ender_append_lines
 jp ender



RAPID_SPE:
 call check_negate_mode
 jr nz,negate_rapid_spe
 ld hl,_hp+10           ; Lookup each robot's SPE
 ld de,21               ; Each robot's data is 21 bytes apart
 ld b,max_robots - 1
 ld c,(hl)
speed_search:
 add hl,de              ; Next robot
 ld a,(hl)
 cp c
 jr c,this_robot_not_faster
 ld c,a
this_robot_not_faster:
 djnz speed_search
change_spe:
 inc c
 push bc
; ld a,(attack_queue+3)
; call Robot_lookup_0
 call Attack_dest_lookup
 ld de,10
 add hl,de
 pop bc
 ld (hl),c
 call damage_report_end
 call Ender_append_lines
 jp ender
negate_rapid_spe:
 ld c,0
 jr change_spe

WITHDRAW:
 ld a,(attack_queue+1)  ; Get SOURCE (robot to be unpacked) 0 thru 5
 inc a
 ld b,a
 ld ix,_hp - 21
 ld de,21
withdraw_loop0:
 add ix,de
 djnz withdraw_loop0
 res 6,(IX+5)           ; reset "IN STORAGE" bit
withdraw0:              ; IN: A = Robot serial to withdraw (range 1-6)
 ld b,max_robots        ; Field width limit (6)
 cp 3+1                 ; Guest robot?
 jr nc,withdraw_to_left ; Make the guest's robots withdraw on the right
 ld hl,_squares
withdraw_to_right:
 ld a,(hl)              ; Look for a place to unpack to
 or a                   ; Unoccupied square?
 jr z,withdraw_now
 inc hl
 djnz withdraw_to_right
 jp miss0               ; If you run out of spaces, it's a "miss"
withdraw_to_left:
 ld hl,_squares+5
withdraw_to_left_loop:
 ld a,(hl)              ; Look for a place to unpack to
 or a                   ; Unoccupied square?
 jr z,withdraw_now
 dec hl
 djnz withdraw_to_left_loop
 jp miss0               ; If you run out of spaces, it's a "miss"


withdraw_now:
 ld a,(attack_queue+1)
 push af                ; Save source
 inc a                  ; Start at 1
 ld (hl),a

 call CLEAR_CENTER      ; Erase any old menus or text

 pop af                 ; Restore source
 inc a
 ld b,a
 call Name_lookup2
 ld de,$2D00
 call DMX2

 ld hl,str_unpacked     ; "_____ has arrived!"
 call DMX               ;
 call damage_report_end  ; Show the text, disable the cursor
 call Ender_append_lines
 jp ender



Chain_cancellation:
;
; Cancel up to 2 additional attacks
; that have the same SERIAL as the
; current attack.
;
; Don't cancel the current attack.
;
 ld hl,attack_queue+13  ; Point to next attacks' SERIAL byte
 ld b,2                 ; B = Maximum chain length
chain_cancellation_loop
 ld a,(attack_queue+2)   ; Get current SERIAL byte
 cp (hl)
 ret nz
 dec hl
 dec hl
 ld (hl),0
 ld de,13
 add hl,de
 djnz chain_cancellation_loop
 ret



Hit_or_miss:
 ld a,(attack_queue+9)  ; Get accuracy bits (bit 5,4)
 and %00110000          ; Mask off non-accuracy bits ; A {0,16,32,48}
 ret z                  ; "00" means "can't miss"
 ld b,a                 ; B = {16,32,48}
; call RAND_SPECIAL      ; A = (0-127)   B = {16,32,48}
 call RAND              ; A = (0-127)
 add a,b                ; A = {16-143} {32-159} {48-175}
 cp 63                  ; 
 ret nc                 ; Passed!  The attack is uneffected.
miss:
 pop hl                 ; Fix the stack!
miss0:
 call Report_positioner ; Position the next text string
 ld hl,str_miss
miss_lable0:
 call DMX2              ; Display "mis" where damage would normally go
 ld a,report_delay2     ; Pause for readability and...
 call DELAY     ; ...comphensate for interrupt slowness
 call Ender_append_lines; Cancel the attack
 jp ender


Ender_append_lines:
 ld hl,attack_queue+11
 ld de,attack_queue
 ld bc,11*6*3           ; BC = Attack queue length (11bytes*6robots*3lines)
 ldir                   ; Append 
 ret

Damage_setup:
 ld hl,attack_queue+4           ; Point to damage byte           
 ROM_CALL($0033-$1A)            ; "LD HL,(HL)
 push hl                        ; "LD IX,HL"
 pop ix                         ;
 ret


Damage_report:  ; IN: IX = damage  NEEDS: (DAMAGE) word pre-set
                ; NOT DESTROYED: IY
 call CLEAR_LINE1               ; Clear the announcement
 push ix                        ; "LD HL,IX"
 pop hl                         ;
 call Report_positioner         ; DE = position of damage count
 call DMD2                      ; Display HL (a number) at "DE"
damage_report_sign = $ + 1
 ld a,$00
 or a                           
 jr z,damage_lable              ; If A=0, don't draw the plus sign
 ld hl,msg_plus                 ; HL -> "+"
 call DMX                       ; Write plus sign
damage_lable:
 xor a                          ; Fix command above ^
 ld (damage_report_sign),a
damage_report_end:
 ld a,report_delay              ; Pause for readability and...
 call DELAY             ; ...comphensate for interrupt slowness
 ret



Report_positioner:
 ld a,(attack_queue+3)           ; Get destination
Report_positioner_0:
 ld e,$00-$04                   ; E = -6
 ld b,a
 inc b                          ; Start at 1, not zero
damage_rep_loop0:
 ld a,e                         ; E = -6
 add a,12
 ld e,a                         ; E = Position of damage report
 djnz damage_rep_loop0
 ld d,$24                       ; Always write text at this line
 ret



Do_Animation:
 call Launcher_animation
Do_animation_0:
 call Animation_init    ; Enable IM 1.  Pre-set sprite position.
 ld a,(attack_queue+10)  ; Load ANIMATION/CRITICAL/HAVOC/SPREAD/COMBO byte
 and %11110000           ; Mask off irrelevant nibble
  cp $20                 ; Flames animation?
  jr z,Animation_2
  cp $30                 ; Explosion animation?
  jr z,Animation_3
  cp $40                 ; Lightning animation?
  jr z,Animation_4
  cp $50                 ; Water animation? (Formerly cyclone)
  jp z,Animation_5
  cp $60                 ; Laser burst animation?
  jp z,Animation_6
  cp $90                 ; Flux Animation?
  jp z,Animation_9
  cp $A0                 ; Teleport animation?
  jp z,Animation_10
  cp $B0                 ; Shield animation?
  jp z,Animation_11
  cp $C0                 ; Item/Assist animation?
  jp z,Animation_12
Animation_end:
 ld hl,GRAPH_MEM+288    ; Erase hovering robot image...
 ld de,GRAPH_MEM+289    ; This is necessary for tall robot sprites
 ld bc,23               ;
 ld (hl),%00000000      ;
 ldir                   ;
 IM 1
 call CR_GRBCopy
 IM 2
 call Timed_attack_check
 ret

Animation_1:            ; Melee Animation <removed>

Animation_2:            ; Immolation Animation
 ld e,6                         ; E = Number of frames for fire
 ld hl,tbl_fire_animation       ; HL -> Animation frame order
 jp Animate                     ; Show Animation

Animation_3:            ; Explosion Animation
 ld e,3
 ld hl,tbl_explode_animation
 jp Animate

Animation_4:            ; Shock field animation
 ld e,8
 ld hl,tbl_shock_field
 jp Animate

Animation_5:            ; Cyclone Animation
 ld e,3                         ; E = Number of frames
 ld hl,tbl_cyclone_animation    ; HL -> Frame order
 jp Animate

Animation_6:            ; Laser Burst Animation
 ld e,2                         ; E = Number of frames
 ld hl,tbl_laser_animation      ; HL -> Frame order
 jp Animate

Animation_8:            ; Protons Animation
; <Removed>
Animation_9:            ; Electrons Animation
 ld e,20
 ld hl,$0000            ; HL can be *any* address,It's supposed to show trash
 jp Animate

Animation_10:           ; Teleport Animation
 ld e,9                         ; E = Number of frames
 ld hl,tbl_cyclone_animation    ; HL -> Frame order
 jp Animate


Animation_11:           ; Shield Animation
 ld e,3                         ; E = Number of frames
 ld hl,tbl_shield_animation
 jp Animate

Animation_12:           ; Good Energy Animation (Plus sign)
 ld hl,tbl_plus_animation
 ld e,2
 jp Animate


Launcher_animation:
 ld a,(attack_queue+1)  ; Get information on source
 cp 255                 ; Is this a side shot?
 jr z,hover_end         ; If so, don't do "jumping" animation
 call Animation_init_0  ; Use Animation placer, but with source instead
 push bc                ; BC = sprite position
 ld hl,spr_blank
 ld a,$FF               ; Disable interrupt-based robot redrawing
 call PutSprClp_special ; 

; ld a,(attack_queue+1)  ; Get information on source
; call Robot_lookup_0    ; Get robot's data
 call Attack_source_lookup
 ld de,18
 add hl,de              ; HL -> Robot image number
 ld b,(hl)              ; B = Robot image number
 ld hl,spr_robot0 - 16  ; HL -> Robot image #1
 ld de,16
launcher_loop0:
 add hl,de              ; Locate true robot image
 djnz launcher_loop0
 pop bc                 ; BC = Sprite Position  HL = Sprite Address
 ld c,$18               ; Move sprite up 2 pixels

 ld a,$FF               ; Disable interrupt-based robot redrawing
 call PutSprClp_special ;

hover_done:
 ld a,hover_delay       ; Pause for readability and...
 call DELAY     ; ...comphensate for interrupt slowness
hover_end:
 xor a                  ; Reverse affect of CR_GRBCopy_no_ints
 ld (int_redraw_off),a  ; (Re-enable interrupt redrawing)
 ret

PutSprClp_special:
 ld (int_redraw_off),a
 IM 1                   ; Disable interrupt redrawing
PutSprClp_special_0:
 call PutSprClp         ; Draw
 call CR_GRBCopy        ; 
 IM 2                   ; Re-enable interrupt redrawing
 ret

Timed_attack_check:
;---Check attacker's Class-------
 ld a,(attack_queue+2)          ; Get attack's serial
 cp last_melee + 1
 ret nc                         ; Timed attacks affect melee only
; ld a,(attack_queue+1)          ; Recover attack's source
; call Robot_lookup_0
 call Attack_source_lookup
 ld de,15
 add hl,de                      ; HL -> TYPECLASS byte for shooter
 ld a,(hl)                      ; 
 and %00001111                  ; Mask away "TYPE" nibble
 or a                           ; "0" means "destroyer class"
 ret nz                         ; Non-destroyers can't use this ability
;---Check attacker's team--------
 ld a,(attack_queue+1)          ; Check attack's source
 call On_my_team                ; Is this robot controlled by THIS player?
 jp nz,do_time_bonus            ; Allow the AI to mimmick your timed attacks
;---Check for held-down key------
 call GET_KEY                   ; No keys may be pressed at start of cycle
 cp G_NONE                      ;
 jp nz,bonus_failed             ;
 ld b,timed_attack_difficulty
timing_b_loop0:
 halt
 call GET_KEY                   ; What bonus did the player choose?
 cp G_DOWN                      ; Amplification?
 jr z,timing_b_window2
 cp G_UP                        ; Self-healing?
 jr z,timing_b_window2
 cp G_LEFT                      ; Redirect left?
 jr z,timing_b_window2
 cp G_RIGHT                     ; Redirect right?
 jr z,timing_b_window2
 djnz timing_b_loop0            ; Wait a little longer
bonus_failed:
 ld a,$FF                      ; Indicate timed attack failed
timing_b_window2:
 ld (timing_bonus),a

; ld a,(BOSS)
; cp 255                         ; Is this a link battle?
; jr nz,bonus_transmit_escape

; DI

; call synchronise               ; Wait until other calculator is ready
; ld hl,timing_bonus             ; Transmit time bonus type
; ld b,1                         ; Duration, 1 byte
; call blockSend                 ; Transmit
; EI
;bonus_transmit_escape:

do_time_bonus:
 ld a,(timing_bonus)
 cp 255                         ; 255 means the bonus failed ; sent via link
 ret z
 cp G_RIGHT
 jr z,RIGHT_award               ; A timed "RIGHT" redirects the attack
 cp G_LEFT
 jr z,LEFT_award                ; A timed "LEFT" redirects the attack
 cp G_DOWN
 jr z,DOWN_award                ; A timed "DOWN" amplifies your attack

UP_award:                       ; A timed "UP" self-heals
 ld a,(attack_queue+1)          ; Get attack's origin
 ld (attack_queue+3),a          ; Send attack to self
 call Robot_lookup_0            ; HL -> Robot's data space
 inc hl
 inc hl                         ; HL -> Robot's current EP
 ld a,(hl)
 ld (attack_queue+4),a          ; Update LSB of damage word
 xor a
 ld (attack_queue+5),a          ; Clear MSB of damage word
 ld a,1
 ld (attack_queue+8),a          ; Set attack's algorythm to 1 (Heal)
 ret

RIGHT_award:
 ld a,(attack_queue+3)          ; Get attack's current destination
 cp 5
 ret z                          ; Cancel if trying to redirect off-screen
 inc a
 jr timed_redirect

LEFT_award:
 ld a,(attack_queue+3)          ; Get attack's current destination
 or a
 ret z                          ; Cancel if trying to redirect off-screen
 dec a
timed_redirect:
 ld (attack_queue+3),a
 ret

DOWN_award:
; ld a,(attack_queue+1)          ; Get attack's source (0-5 position)
; call Robot_lookup_0            ; OUT: HL -> Shooter's data space
 call Attack_source_lookup
 inc hl                         ; Locate shooter's current EP level
 inc hl                         ; 
 ld a,(hl)                      ; Store ; A == Current energy
 or a
 ret z                          ; No energy...no bonus
 dec a                          ; Deduct 1 EP
 ld (hl),a
 ld hl,attack_queue+4           ; Locate attack's damage
 call ld_de_hlPTR               ; DE <- (HL) ; 16 - bit load
 ex de,hl                       ; HL = attack's initial damage
 ld e,a                         ; 
 ld d,0                         ; DE = shooter's EP
 inc de                         ; Replace the deducted EP
 add hl,de                      ; HL = damage + shooter's EP + 1
 ex de,hl                       ; DE = damage + shooter's EP + 1
 ld hl,attack_queue+4
 call ld_hlPTR_de               ; (HL) = damage + shooter's EP
 call Quake
 ret

;receive_bonus_by_link:
; ld a,(BOSS)
; cp 255                         ; Is this a link battle?
; ret nz                         ; Ret, if not
; DI
; call synchronise               ; Hold until other calcualtor is ready
; ld hl,timing_bonus             ; Send incoming byte to timing bonus variable
; ld b,1                         ; 1 incoming byte
; call blockReceive              ; Receive bonus type
; EI
; jp do_time_bonus

Quake:
 DI
 ld b,7
quake_loop_2:
 push bc
quake_loop_0:
 call Scroll_D
 djnz quake_loop_0
 ld b,9
quake_delay_1:
 call CR_GRBCopy
 djnz quake_delay_1
 pop bc
 push bc
quake_loop_1:
 call Scroll_U
 djnz quake_loop_1
 ld b,9
quake_delay_2:
 call CR_GRBCopy
 djnz quake_delay_2
 pop bc
 dec b
 jr nz,quake_loop_2
 EI
 ret

Animation_init:
 ld a,(attack_queue+3)   ; Locate destination
Animation_init_0:
 ld b,a
 ld hl,$0A1A            ; Sprite position for 1st robot
 or a                   ; "CP 0"
 jr z,animation_pla_escape
animation_placer:
 ld de,$0C00            ; The robots are 12 pixels apart
 add hl,de
 djnz animation_placer
animation_pla_escape:
 push hl                ; "LD BC,HL"
 pop bc                 ; The sprite drawer uses <BC> for positioning
 ret

;------------------------
Shooter_ailment:
;
; If BURN is detected
; - damage the shooter
;
; If ICE is detected
; - All non-damaging moves are wasted
; - All damaging moves are wasted, but shatter ice
;------------------------
 ld a,(attack_queue+1)  ; Check the shooter
 cp 255                 ; Ignore spread attacks
 ret z
 call Robot_lookup_0    ; Get robot status
 ret z                  ; Escape if no one is present
 ld de,5
 add hl,de              ; Point HL to lower STATUS byte
 ld a,(hl)
 and %10000000          ; Check for ice
 jr nz,ice_penalties
burn_penalties:
 ld a,(hl)
 and %00010000          ; Check for burn damage
 ret z
 ld a,(attack_queue+1)          ; Get SOURCE
 call Burn_message_0            ; Display "burn"
; ld a,(attack_queue+1)          ; Get the shooters's stats
; call Robot_lookup_0            ; HL -> Shooter's HP
 call Attack_source_lookup
 call ld_de_hlPTR               ; DE = HP (LOW:HIGH) ; HL -> Robot HP
 ex de,hl                       ; DE -> Robot HP ; HL = HP (LOW:HIGH)
 call HL_SWAP                   ; HL = HP (HIGH:LOW)
 xor a                          ; Clear carry flag
 ld bc,burn_damage              ; Amount of damage per burn
 sbc hl,bc                      ; HL = HP + (-10)
 jp p,burn_didnt_kill
 ld hl,0
burn_didnt_kill:
 call HL_SWAP                   ; HL = HP (LOW:HIGH)
 ex de,hl
 call ld_hlPTR_de                 
; ld a,report_delay3             ; Pause for readability and...
; call DELAY_special             ; ...comphensate for interrupt slowness
 ret
ice_penalties:
 call Chain_cancellation
 ld a,(attack_queue+8)          ; Get attack algorythm
 and %00001111                  ; Mask off advantage nibble ; Get algorythm
 jr nz,ice_cancels              ; All non-harming attacks are wasted
;---Shatter the ice--------------
 ld a,(attack_queue+1)          ; Get attack source
 call Robot_lookup_0
 ld de,5
 add hl,de
 res 7,(hl)                     ; Clear away ice effect
ice_cancels:
 ld a,(attack_queue+1)          ; Forward this value to <ailment_report>
 jr ailment_stasis2_lable1      ; Treat this as a "miss"

Burn_message:
 ld a,(attack_queue+3)  ; Get attack DESTINATION
Burn_message_0:
 call CLEAR_LINE1               ; Clear the announcement
 ld hl,str_burn
 call Report_positioner_0       ; Position the text cursor
 call DMX2
 ld a,report_delay3             ; Pause for readability and...
 call DELAY             ; ...comphensate for interrupt slowness
 ret



ailment_report:
 ld a,(attack_queue+1)
ailment_report_0:
 call Report_positioner_0       ; Position DE.  HL is uneffected.
 pop af                         ; Fix the stack
 jp miss_lable0                 ; Treat this attack as a "miss"

;------------------------
Target_ailment:
;
; Does the target have
; an ailment that may
; affect the next
; attack?
;
; If STASIS is detected:
; - Display "0" where
;   the shot went
; - Cancel the attack
;------------------------
; ld a,(attack_queue+3)  ; Get the TARGET's data
; call Robot_lookup_0
; ld de,5
; add hl,de              ; Check STATUS byte
; ld a,(hl)
 call Attack_dest_status_lookup
 and %00001100          ; Check for STASIS now
 ret z
ailment_stasis2_lable0:
 ld a,(attack_queue+3)  ; Forward this value to <ailment_report_0>
ailment_stasis2_lable1:
 ld hl,str_stasis       ; Display "0" for damage
 jr ailment_report_0    ; Treat this as a "miss"


;--------------------------------
half_damage2:                   ;
;                               ;
; IN: IX = current damage       ;
; OUT IX = 1/2 old damage       ;
; DESTROYED: HL                 ;
;--------------------------------
 ld hl,str_weak_hit
 call line3_message 
 push ix
 pop hl
 sra h
 rr l
LD_IX_HL:
 push hl
 pop ix
 ret
;--------------------------------
x2_damage2:                     ;
;                               ;
; IN: IX = current damage       ;
; OUT IX = 2 x old damage       ;
; DESTROYED: HL                 ;
;--------------------------------
 ld hl,str_x2_hit    
 call line3_message     ; Display "x2 Hit!"
 push ix
 pop hl
.db $CB,$35             ; SLL L ; Multiply lower damage byte
 rl h                   ; Multiply upper byte with carry
 jr LD_IX_HL
;--------------------------------
line3_message:                  ;
;                               ;
; For "x2 hit" and "weak" hit   :
;                               ;
; IN: HL -> String to show      ;
; OUT: -nothing-                ;
;--------------------------------
 ld de,$3937
 jp DMX2
;--------------------------------
line2_message:                  ;
;                               ;
; For "Armor" "Lucky Shot"      ;
; "Advantage" hits              ;
;                               ;
; IN: HL-> String to show       ;
; OUT: -nothing-                ;
;--------------------------------
 ld de,$3331
 jp DMX2
;--------------------------------
line_message_pause:             ;
;                               ;
; Very short delay for          ;
; readability                   ;
;--------------------------------
 ld a,report_delay4                   
 jp DELAY               ; IN: Use A register ; and Return to call


;------------------------
Time_healer:
;
; Cause stasis to disappear
; after 3 turns
;
;-------------------------
 ld a,max_robots - 1
time_healer_loop:
 push af                ; Save loop counter
 call Robot_lookup_0    ; Skip if no one present
 jr z,time_heal_next
 ld de,5
 add hl,de
 ld a,(hl)
 and %00001100          ; Check for - STASIS -
 jr z,time_heal_next
 sub 4                  ; Remove stasis point ; A = 0,4, or 8
 ld e,a                 ; Save new stasis point(s)
 ld a,(hl)
 and %11110011          ; Mask out old stasis timer
 or e                   ; Mask in new stasis timer
 ld (hl),a              ; Update STATUS-H
 jr time_heal_next
time_heal_next:
 pop af
 dec a
 jp p,time_healer_loop
 ret



;------------------------
Shield_check:
; 
; Check for shield and
; reduce any damage by
; 50%, if present.
; Deduct shield points.
;
; IN: -nothing-
; OUT: Z = have no shield
;     NZ = have shield
;
;------------------------
 call Attack_dest_status_lookup ; Get attack victim's stat bits
 and %00000011          ; Check for damage shield
 ret z
do_shield:
 dec a                  ; Deduct one shield point ;Other states not changed
 ld (hl),a              ; Update shield
 ld a,(attack_queue+8)  ; Check attack's algorhythm
 and %00001111          ; Mask of "advantage" property
 jp z,shield_basic      ; < Reduce HARMing attacks
shield_reflecting:      ; < Reflect special attacks
;---Check for teleport---
 cp $0C                         ; Is current attack "teleport?"
 jp z,ailment_stasis2_lable0    ; If so, treat it like a stasis hit
;---Ignore benign moves--
 ld a,(attack_queue+9)  ; Get attack's targeting
 and %00000100          ; Is this a benign attack?
 ret z
;---Cancel splash damage-
 ld a,(attack_queue+1)  ; Get attack's source
 cp 255                 ; Was this attack already reflected? (or splash hit?)
 jp z,ailment_stasis2_lable0 ; If so, treat it like a stasis hit
;---Reflect the move-----
 call Chain_cancellation; Cancel any side-attacks too
 call Report_positioner ; Position the next text string
 ld hl,str_block        ; Display "0" where true damage would normally go
 call DMX2              ;
 ld a,(attack_queue+1)  ; Now reflect the attack
 ld (attack_queue+3),a  ; Return to sender
 ld a,255
 ld (attack_queue+1),a  ; Set source to 255 so this attack will not return
 pop hl                 ; Trash call to Shield_check!
 jp ender
shield_basic:
 call half_damage2
 ret                    ; Go back to HARM lable


Charge_check:
 ld a,(attack_queue+1)
 cp 255                 ; Ignore spread attacks
 ret z
 call Robot_lookup_0    ; Get the shooter's address. HL -> Robot's HP
 inc hl
 inc hl                 ; HL -> Shooter's EP

 ld a,(attack_queue+10)
 and %00000111
 cp 7                   ; Was it a sigma move?
 jr nz,charge_normal
 ld a,sigma_cost / 2
charge_normal:
 sla a
 ld c,a
 ld a,(hl)
 sub c
 jr c,no_charge         ; Cancel this attack if there's no charge
 ld (hl),a              ; Subtract the robot's EP

 ret
no_charge:
 ld a,report_delay              ; Pause for readability and...
 call DELAY             ; ...comphensate for interrupt slowness
 call Chain_cancellation        ; Cancel all side attacks
 ld hl,str_no_charge
 call line2_message             ; Display "Low EP!"
 call line_message_pause
 call Ender_append_lines
 pop hl                 ; Fix the stack again because this was CALLed
 jp ender

Item_deduct:
 ld a,(attack_queue+2)   ; Get attack SERIAL
 cp first_item          ;[see robot.h] Filter out non-items 
 jr c,item_failure      ;
 cp 64                  ; Filter out A,B,Y attacks
 jr nc,item_failure     ; 
 ld a,(attack_queue+1)   ; Get attack SOURCE
 cp 255                 ; Ignore side attacks
 jr z,item_failure      ;
item_deduct_exempt = $ + 1
 ld a,$00               ; <-- Make this value non-zero to exempt this item
 or a
 jr nz,item_success

; ld a,(attack_queue+1)
; call Robot_lookup_0
 call Attack_source_lookup
 ld a,b
 cp 4                   ; A Guest's item?
 jr c,take_host_item
take_guest_item:
 ld hl,_item_temp2
 jr take_item
take_host_item:
 ld hl,_item_temp
take_item:
 ld a,(attack_queue+2)   ; Get attack SERIAL number.  HL -> First of 7 items
 ld bc,max_item_types   ; BC = lenght of item search
 cpir                   ; See if player still has this item
 dec hl                 ; Move back on byte
 cp (hl)                ; Match ; If not, it was not found
 jr nz,item_failure
item_found:
 ld de,16
 add hl,de              ; Jump forward to item quantities
 dec (hl)               ; Take away this item
 jr nz,item_success
 ld de,-16
 add hl,de              ; 
 ld (hl),0              ; Erase this item from the list
item_success:
item_failure:
 ld a,(item_deduct_exempt)
 or a
 ret z
 dec a
 ld (item_deduct_exempt),a
 ret

;Random_items:
; ld a,(BOSS)                    
; cp 254                         ; Linked (255) or Disconnected (254) battle?
; ret c                          ; If no return
; ld hl,_item_temp
; ld a,(HOST)
; cp 1
; jr z,random_i_lable0
; ld hl,_item_temp2
;random_i_lable0:
; ld bc,max_item_types           ; Scan to see if you have room for items
; xor a                          ;
; cpir                           ;
; dec hl
; ret nz                         ; Return if there is no room for new items
; call RAND                      ; Pick a random item
; and %00001111                  ;

; cp 14                          ; If the RAND was 14 or 15...(which are not items)
; jr c,random_i_lable1           ; ...give the player an Energy item
; ld a,2                         ; 2 + first_item = Energy
;random_i_lable1:
; add a,first_item
; ld (hl),a                      ; Add the item
; ld de,2*max_item_types
; add hl,de                      ; HL -> Item Quantities
; ld (hl),1                      ; Max Quantity is always 1 in linkmode
; ret
;-----------------------------------------------------------
;
;       ITEM DEDUCT (Outside of battle)
;___________________________________________________________
;
; Item_deduct2: Deducts one item (C) that is at address
;               (IX).  Also decrements the item's stock
;               number and stops at zero.  Checks for
;               null items.
;
;               IN: IX->Item in itemlist
;                    C= Item's serial
;
; Item_deduct3: Uses (HL) as an input pointer instead
;               of IX
;
;               IN: HL->Item in itemlist
;                    C= Item's serial
;
; Item_deduct4: Searches for the item for you and deducts
;               IN:  C= Item's serial
;-----------------------------------------------------------
Item_deduct4:           ;IN:
 ld hl,ITEM1
 ld a,c
 ld bc,max_item_types + 1
 cpir
 dec hl
 cp (hl)                ; Null item choice?  (Or not found)
 ret nz
Item_deduct3:           ; IN: HL=item pointer  C=item serial
 push hl
 pop ix
Item_deduct2:           ; IN: IX=item pointer  C=item serial
 ld a,c
 or a                   ; Check for a "null" (a blank selection)
 jr z,item_deduct_error
 dec (ix+16)            ; Decrement item quantity
 ld a,(ix+16)           ; Check remaining items of this type
 or a
 jr nz,item_ded_lable0
 ld (ix),0              ; Zero out the item
item_ded_lable0:
 xor a                  ; Clear carry flag
 ret
item_deduct_error:
 scf                    ; Carry = No such item
 ret


TERMINATOR:
 call Vulture           ; Clear out robots with 0 HP
 call Win_or_lose       ; Check once more for win-loss condition
; call Random_items
 call Time_healer       ; Remove -STASIS- gradually
 call QUEUE_CLEAR
Terminator:
terminator:
 ld hl,turns            ; Check number of turns taken
 inc (hl)               ; Increment turn count
 ld a,(turn_limit)      ; Check turn limit 
 cp (hl)                ; Are you out of turns?  If so, you "win" by default.
 jp z,default_win       ; (The maximum number of turns is 255)
 jp YOUR_TURN

QUEUE_CLEAR:
 ld hl,attack_queue
 ld de,attack_queue+1
 ld (hl),0
 ld bc,11*6*3           ; 11byte*6robots*3lines each.  *Do not subtract 1!
 ldir
 ret

;------------------------------------------------
Attack_adder:
;
; Loads (SPEED) (SOURCE) (SERIAL) (DESTINATION)
; data into attack queue.  Also adds 6 additional
; bytes of attack data from table (damage, etc.)
;
; The attack adder can spread attacks if
; indicated byte the table.  Spreading means that
; an attack can be sent to multiple spaces.
; (Attacks like SCORCH and FLASH do this)
;
; IN: -nothing- 
;
; NEEDS: (_speed_temp) pre-set
;        (_source_temp) pre-set
;        (_serial_temp) pre-set
;        (_destination_temp) pre-set
;
;        (auto_spread) == 0 : for forced spread
;
; OUT: Nothing
;------------------------------------------------
 ld a,(_source_temp)    ; Use known source to look up attacker's data
 call Robot_lookup_0    
 ld de,8
 add hl,de              ; HL -> Shooter's Attack
 ld a,(hl)
 ld (shooter_atk),a     ; * POLYMORPH to address below

 ld hl,attack_queue
 ld b,18                ; 18 rows in the attack queue
attack_add_loop1:
 push bc                ; Save outer loop count
 ld b,11                ; Each row is 11 bytes long
 xor a
attack_add_loop0:
 ld c,(hl)              ; Search this row for data
 or c                   ; queue rows which already contain data can't be used
 inc hl
 djnz attack_add_loop0
 pop bc                 ; Restore outer loop count
 or a                   ; "Does the last line hold any data?"
 jr z,add_attack        ; "If not, let's use it!"
 djnz attack_add_loop1
add_attack:
 ld de,-11              ; Move back one row
 add hl,de              ; "sub hl,11"
 ex de,hl               ; Destination = empty attack queue line
 ld hl,_speed_temp      ; Source = speed, source, attack #, and destination
 ld bc,4
 ldir                   ; Copy 
 ld a,(_serial_temp)
 ld b,a
 push de                ; Save destination
 call Attack_lookup
 pop de                 ; Restore destination
 push ix                ; "LD HL,IX"
 pop hl                 ;
 ldi                    ; Copy damage word:H
 ldi                    ; Copy damage word:L
shooter_atk = $ + 1     ; * POLYMORPH from Attack adder code above
 ld a,$00
 sra a                  ; Divide by two
 ld (de),a              ; Load shooters ATK to table
 inc hl                 ; Skip LDI
 inc de                 ;
 ld bc,4                ; Copying 4 more bytes of table from table
 ldir                   ; Copy status, advantage, menu and animation

 dec de                 ; DE -> Last byte written + 1
 ex de,hl               ; DE -> Attack Table  HL -> queue
 ld c,(hl)              ; Prepare to mask data (anim, bonus + power)
 ld a,(power)           ;
 or a
 jr z,check_for_auto_spread
 or c                   ;
 ld (hl),a              ;
 xor a
 ld (power),a
 bit 0,(hl)             ; 
 jr z,spread_damage     ; Zero = spread ; NonZero = don't spread

check_for_auto_spread = $
 ret                    ; * Morphic Instruction
spread_damage:
 ex de,hl               ; HL -> Last byte of table  DE -> attack queue
 inc de                 ; DE -> First unwritten byte of attack queue
 ld h,d
 ld l,e
 ld bc,-11
 add hl,bc              ; HL -> Original (center) attack  DE -> next line
 ld bc,11 * 2           ; Copy and create two side-attacks
 ldir
 push de
 pop ix                 ; IX -> First unwritten byte

 ld (ix-11+1),255       ; (Source=255 indicates it's a side-attack)
 ld (ix-22+1),255       ; (Source=255 indicates it's a side-attack)

right_side:
 ld a,(ix-11+3)                 ; Restrict side-shots to the playfeild...
 cp max_robots - 1              ; ...to avoid serious data corruption
 jr nc,cancel_right_side        ;
 inc (ix-11+3)                  ;
 jr left_side
cancel_right_side:              ;
 ld (ix-11),0                   ; Cancel by making speed = 0
left_side:
 ld a,(ix-22+3)                 ;
 or a
 jr z,cancel_left_side
 dec (ix-22+3)                  ;
 jr spread_done
cancel_left_side:
 ld (ix-22),0                   ;
spread_done:
 ld a,$c9               ; <-- Opcode for "ret"
 ld (check_for_auto_spread),a     ; <Morphic> Disable the autospread function above
 ret


;------------------------------------------------
Opponent_loader:
;
; Load opponent stats from data tables and into
; _temp robot data.  This is usually done
; *before* a battle begins.
;
; IN: B = opponent number (never use 0!)
;     C = opponent position on battle field (0-5)
;
;------------------------------------------------
 push bc                ; Save opponent number for later
 push bc                ;
 ld hl,e1 - 11          ; Point to space *behind* the data
 ld de,11               ; Prepare to add
opponent_loa_loop0:
 add hl,de              ; Locate opponent
 djnz opponent_loa_loop0
 push hl                ; Save source robot
 ld b,c                 ; Now load location data
 inc b                  ; Avoid starting at 0
;----Copy Basic Opponent Stats
 ld hl,_attack - 21
 ld de,21               ; Prepare to add
opponent_loa_loop1:
 add hl,de              ; Locate destination data space
 djnz opponent_loa_loop1
 ex de,hl               ; DE -> Bot data destination
 push de                ; Save Bot data destination
;--- Pre-clear Robot's data space
 ld h,d
 ld l,e
 inc de
 ld bc,11 - 1
 ld (hl),0
 ldir
;---
 pop de                 ; DE -> Bot data destinatino
 pop hl                 ; HL -> Bot data source
 ld bc,11               ; Always load 11 bytes of stats
 ldir                   ; Copy data! ; DE -> byte after _imageX
;----
 push de                ; Save _imageX address
 pop ix                 ; Restore _imageX address
 ld c,(ix-2)            ; C = RELATIVE LEVEL
 ld a,(MAP_DIFFICULTY)
 add a,c
 ld c,a                 ; C = MAP_DIFFICULTY + Robot's RELATIVE LEVEL
 ld e,a
 ld (ix-17),c           ; EP = MAP_DIFFICULTY + Robot's RELATIVE LEVEL + 3
 inc (ix-17)            ;
 inc (ix-17)            ;
 inc (ix-17)            ;
 ld b,3                 ; Adjust 3 stats total (ATK, DFN, and SPE)
stats_adjuster:
 ld a,(ix-11)           ; Stat = MAP_DIFFICULTY + RELATIVE LEVEL + Adj
 add a,c                ; 
 ld (ix-11),a           ;
 inc ix                 ; Move to next stat
 djnz stats_adjuster
 dec ix
 dec ix
 dec ix                 ; IX -> bytes after _imageX again

 ld (ix-12),c

.db $DD,$CB,-12,$36
; sll (ix-12)
 rl (ix-13)             ; HPmax = 2 x (MAP_DIFFICULTY + RELATIVE LEVEL)
 ld hl,STORY1
 bit 1,(hl)
 jr z,no_expert_difficulty
.db $DD,$CB,-12,$36     ; Double enemy HP again for expert players
; sll (ix-12)
 rl (ix-13)             ; HPmax = 4 x (MAP_DIFFICULTY + RELATIVE LEVEL)
no_expert_difficulty:
 ld h,(ix-13)           ; Add minimum_enemy_hp points to HPmax
 ld l,(ix-12)           ;
 push de                ; Save Robot's Difficulty
 ld de,minimum_enemy_hp ; 
 add hl,de              ;
 pop de                 ; Restore Robot's Difficulty
 ld (ix-13),h           ;
 ld (ix-12),l           ;

 ld a,(ix-12)           ; Set HP = HPmax
 ld (ix-18),a           ;
 ld a,(ix-13)           ;
 ld (ix-19),a           ;
 ld a,(ix-8)            ; Check weapon type
 or a                   ; No weapon allowed?
 jr z,weapon_done
 cp 9                   ; Non-standard weapon?
 jr nc,weapon_done
 ld c,a
 ld a,e
 srl a
 srl a
 srl a
 srl a
 and %00000011
 add a,c
weapon_done:
 ld (ix-8),a            ; Adjust weapon level up 0, 1, 2, or 3
;---
 pop bc                 ; Restore opponent number
 ld hl,e_names
opponent_loa_loop3:
 ld c,$FF               ; Maximum search range
 xor a                  ; Search 0's (string terminators)
 cpir
 djnz opponent_loa_loop3
 pop bc                 ; Restore opponent number
 push hl                ; Save name source
 ld b,c
 inc b
 ld hl,_name0 - 11      ; Point *behind* the data space
 ld de,11               ; Prepare to add
opponent_loa_loop2:
 add hl,de              ; Locate destination space
 djnz opponent_loa_loop2
 ex de,hl
 pop hl                 ; Restore name source
 ld bc,11               ; 11 bytes in a name
 ldir                   ; Copy name!
 ret

;------------------------------------------------
Target:
; Valid target's designator.  Searches for a
; valid target based on squares data and table
; information.  Adjusts cursors left or right
; based on users input
;
; IN: C = HOST/GUEST status
;     B = Attack used
;
; OUT: Carry = user cancelation or targeting error
;    NoCarry = success!
;------------------------------------------------
 ld a,(SELECTOR)
 push af
 call Attack_lookup             ; IX now points to attack data in table.
 bit 3,(IX+5)           ; If this bit is set, you must target your allys
 jr nz,target_allies
 bit 2,(IX+5)           ; If this bit is set, you must target your enemies
 jr nz,target_enemies
target_anywhere_loop0:
 ld (SELECTOR),a
 ld e,%00000010
 call Enable_cursor
target_anywhere:        ; If no bit is set, you can target any space
 call GET_KEY
 cp G_LEFT
 jr z,target_anywhere_left
 cp G_RIGHT
 jr z,target_anywhere_right
 cp G_2ND
 jp z,targeter_escape
 cp G_ALPHA
 jp z,targeter_escape
 jr target_anywhere
target_anywhere_left:
 ld e,%11111101                 ; Disable old targeting cursor
 call Disable_cursor
 ld a,(SELECTOR)
 or a
 jr z,target_anywhere_loop0
 dec a                          ; A = New cursor position
 jr target_anywhere_loop0
target_anywhere_right:
 ld e,%11111101                 ; Disable old targeting cursor
 call Disable_cursor
 ld a,(SELECTOR)
 cp max_robots - 1
 jr z,target_anywhere_loop0
 inc a                          ; A = New cursor position
 jr target_anywhere_loop0

target_allies:
 ld a,c
 ld (find_on_shift),a     ; Morphic code for <Select_left:>
 call Select_leftmost
 jr target_now                  ; Start targeting
target_enemies:
 ld a,c                         ; Change 1 to 5, and 5 to 1
 cp 1                           ;        
 jr z,target_enemies_L0         ;
 sub max_team*2                 ;
target_enemies_L0:              ;
 add a,max_team                 ;
 ld (find_on_shift),a    ; Morphic code for <Select_right:>
 call Select_leftmost
target_now:
 ld e,%11111101
 call Disable_cursor            ; Remove any targeting cursor
 ld e,%00000010                 ; Activate targeting cursor (crosshairs)
 call Enable_cursor
target_loop1:
 ld e,%00000010                 ; Activate Targeting cursor
 call Enable_cursor
 call Info_bar
target_loop0:
 call GET_KEY
 cp G_LEFT
 jr z,targeter_left
 cp G_RIGHT
 jr z,targeter_right
 cp G_2ND
 jr z,targeter_escape
 cp G_ALPHA
 jr z,targeter_escape
 jr target_loop0

targeter_left:
 ld e,%11111101                 ; Disable old cursor
 call Disable_cursor
 call Select_left
 jr target_loop1

targeter_right:
 ld e,%11111101                 ; Disable old targeting cursor
 call Disable_cursor
 call Select_right
 jr target_loop1

targeter_escape:
 ld d,a                         ; Save exiting keystroke
 ld e,%11111101
 call Disable_cursor            ; Remove the targeting cursors
 ld a,(SELECTOR)
 ld (_destination_temp),a       ; The user has chosen a destination
 pop af                         ; Restore original cursor
 ld (SELECTOR),a
 ld a,d                         ; Restore exiting keystroke
 cp G_ALPHA                     ; Leave selection cursor on if canceled
 ret z
 ld e,%11111110
 call Disable_cursor            ; Remove any selecting cursors
 ld a,d                         ; Restore exiting keystroke
 ret


;------------------------------------------------

;------------------------------------------------
Select_right:
 ld a,(SELECTOR)
 inc a                          ; We're checking the square to the right
Select_right_0:
 ld e,a                         ; E = Position tracker
select_inc:
 ld a,e                         ; Check (SELECTOR) boundaries
 cp max_robots                  ; "Are we out of bounds?"
 jr nc,Select_shift_fail
 ld b,e                         ; B = Square under scrutiny
 call Ally_search               ; "Does a host robot occupy this square?"
 jr nc,Select_shift_done        ; (SELECTOR) will be updated
 inc e                          ; Increment Position tracker
 jr select_inc

Select_left:
 ld a,(SELECTOR)
 dec a
 ld e,a
select_dec:
 ld a,e                         
 cp max_robots                  ; This will prevent (SELECTOR) from...
 jr nc,Select_shift_fail        ; going to non-(0-5) values.
 ld b,e
 call Ally_search
 jr nc,Select_shift_done
 dec e
 jr select_dec


Select_shift_done:
 ld a,e
 ld (SELECTOR),a        ; OUT ON PASS: NoCarry (from "cp")
 scf
 ccf                    ; Carry Clear (signals 'pass' condition)
 ret
Select_shift_fail:
 scf                    ; OUT ON FAIL: Carry set
 ret                    ; Carry Set (signals 'fail' condition)

Ally_search: ; IN: B = square #(0-5) to check  (find_on_shift) MUST BE preset
find_on_shift = $ + 1
 ld a,$00
 ld hl,_squares - 1             ; Prepare to lookup host/guest status
 inc b                          ; Avoid starting at zero
ally_sea_loop0:
 inc hl                         ; Lookup.  HL points to square to check
 djnz ally_sea_loop0
 ld b,max_team                  ; B = Number of robots you controll naturally
ally_sea_loop1:
 cp (hl)                        ; Is this an ally?
 ret z                          ; Return (Carry will be clear)
 call search_special
 inc a                          ; Move on to next placeholder value
 djnz ally_sea_loop1
 scf                            ; Set carry flag
 ret ; OUT: Carry= No ally found  NoCarry= Ally found!

search_special:
 push af
 cp max_team + 1                ; Is this a guest?
 jr c,search_special_host       
search_special_guest:
 add a,max_team
 cp (hl)
 jr z,special_success
 jr special_fail
search_special_host:
 add a,max_team + max_team + max_team
 cp (hl)
 jr z,special_success

special_fail:
 scf
 pop af
 ret

special_success:
 pop af
 pop hl                         ; Reduce the stack,prepare to exit loop
 scf
 ccf
 ret

;------------------------------------------------
Name_lookup:
;
; Load pointer to a robot's name into hl
;
; IN: A = robot position (0-5)
; OUT:  HL = robot name address (if found)
;
; Name_lookup2:
; IN: B = Robot's number (1-5)
; OUT:  HL = robot name address (if found)
;------------------------------------------------
 call Robot_lookup_0    ; B = Robot number (1-8)
 ret z                  ; Don't do empty spaces
Name_lookup2:
 ld hl,_name0 - 11      ; Point to name cache
 ld de,11               ; 11 = maximum name lenght (with ending "0")
name_search:
 add hl,de
 djnz name_search
 ret                    ; HL -> Robot Name


;------------------------------------------------
Attack_lookup2:
 ld a,(attack_queue+2)   ; Get SERIAL
 ld b,a
Attack_lookup:
;                                               ;
; Load pointer to attack table into IX          ;
; IN:   B = attack number (1-64)                ;
;                                               ;
; OUT: IX = pointer to table line               ;
;
; NOT EFFECTED: A C HL
;
;------------------------------------------------
 ld ix,attack_data - 7
 ld de,7
attack_loo_loop0:
 add ix,de
 djnz attack_loo_loop0
 ret

;------------------------------------------------
Robot_lookup:
;                                               ;
; Get info on whatever robot is occoupying the  ;
; position indicated by (SELECTOR).  Robot      ;
; lookup will point to this robot's HP byte     ;
;                                               ;
; NEEDS: (SELECTOR)                             ;
; OUT:  B = robot number (1-6) only             ;
;      HL = robot data space address            ;
;       Z = empty space detected!               ;
;      NZ = normal                              ;
;                                               ;
; NOT EFFECTED: IX C                            ;
;                                               ;
;Robot_lookup_0:                                ;
; Same as Robot_lookup, but takes in <A>        ;
; instead of (SELECTOR).  Also very useful.     ;
; IN: A = Position to check (0-5)               ;
; OUT: B = Robot number (1-6) only              ;
;     HL = Robot data space address             ;
;------------------------------------------------
 ld a,(SELECTOR)
Robot_lookup_0:
 ld hl,_squares                 ; Load robot placeholders table
 ld d,0                         ; Clear upper byte.  Prepare for ADD.
 ld e,a                         ; A = E = table offset (0-5)
 add hl,de                      ; HL -> Robot place holder
 ld a,(hl)                      ; A now holds number (0) or (1-6) or (7-12)

 ld (space_occupation),a        ; (Morphic) Forward occupied/unoccupied state
 cp 7
 jr c,lookup_normal
lookup_special:
 sub 6
lookup_normal:
 ld b,a                         ; Setup loop counter
 push bc                        ; Save robot number B = (1-6)  C = unknown
 ld hl,_hp - 21                 ; HL -> Robot data tables
 ld de,21                       ; DE = Step length
robot_loo_loop0:
 add hl,de
 djnz robot_loo_loop0
 pop bc                         ; HL -> Robot's data  B = Robot number (1-6)
space_occupation = $ + 1
 ld a,$00                       ; A = (0) or (1-6) or (7-12)
 or a                           ; Z = empty space  NZ = robot detected
 ret

;------------------------------------------------
Attack_source_lookup:                           ;
;                                               ;
; Get info on the current attacker              ;
; IN: -nothing-                                 ;
; OUT: HL -> Robot Data Space                   ;
;       B == Robot number (1-6) only            ;
;------------------------------------------------
 ld a,(attack_queue+1)
 jp Robot_lookup_0
;------------------------------------------------
Attack_dest_lookup:                             ;
;                                               ;
; Get info on the current attack victim         ;
; IN: -nothing-                                 ;
; OUT: HL -> Robot Data Space                   ;
;       B == Robot number (1-6) only            ;
;------------------------------------------------
 ld a,(attack_queue+3)          ; Get attack's destination
 jp Robot_lookup_0
;------------------------------------------------
Attack_dest_status_lookup:                      ;
;                                               ;
; Get status of the current attack victim       ;
; IN: -nothing-                                 ;
; OUT: HL -> Robot Data Space                   ;
;       B == Robot number (1-6) only            ;
;       A == Robot's status byte                ;
;------------------------------------------------
 call Attack_dest_lookup
 ld de,5
 add hl,de
 ld a,(hl)
 ret


;------------------------------------------------
Team_lookup:
;
; Takes in the SELECTOR value.  Looks up who
; controls the robot based it's placeholder.
; Also detects FORCE effect.
;
; IN:  (SELECTOR)
; OUT: NoCarry = belongs to host
;        Carry = belongs to guest
;
; Team_lookup_0:
;
; Does the same as Team_lookup, but takes
; in <A> instead of (SELECTOR)
;
; OUT: NoCarry = belongs to host
;        Carry = belongs to guest
;
;------------------------------------------------
 ld a,(SELECTOR)
Team_lookup_0:
 ld hl,_squares                 ; Look at robot placeholders table
 ld d,0
 ld e,a                         ; Set up offset (DE is 0-5)
 add hl,de                      ; Apply offset
 ld a,(hl)                      ; Get selected robot's placeholder
 cp max_team + 1
 jr c,looked_up_host
 cp 2*max_team + 1
 ret c
 cp 3*max_team + 1
 jr c,looked_up_host            ; FORCEd host?
 scf                            ; FORCEd guest?
 ret
looked_up_host:
 ccf
 ret

;------------------------------------------------
On_my_team:
; Takes in the SELECTOR value.  Looks to see
; if the selected fighter is on THIS player's team
;
; IN:  A == Robot to check
; OUT: Z == Same team
;     NZ == different team
;
;------------------------------------------------
 call Team_lookup_0
 ld a,(HOST)
 jr nc,this_is_host
this_is_guest:
 cp 4
 ret
this_is_host:
 cp 1
 ret


;------------------------------------------------
Info_bar:
;
; Displays:  HP:EP   Shield Points   (Name)
;
; NEEDS: (SELECTOR) must indicate position of robot
;------------------------------------------------
 call Sync_with_rain            ; Hold until the rain pattern dissapears
 ld hl,GRAPH_MEM+12             ; Clear 5 verticle lines
 ld de,GRAPH_MEM+12+1           ;
 ld (hl),%00000000              ;
 ld bc,71                       ;
 ldir                           ;

 call Robot_lookup              ; HL -> Robot HP
 ld a,(SELECTOR)
 ld b,a
 push bc                        ; Save SELECTOR position for later
 ld d,(hl)                      ; Load robot HP
 inc hl
 ld e,(hl)                      
 push hl                        ; Save robot data address
 ex de,hl                       ; HL = robot HP

 push hl                        ; Save robot HP pointer
 call On_my_team                ; Is THIS bot on YOUR team?
 pop hl                         ; Restore robot HP pointer
 jr nz,info_stats_done          ; If on different team, jump to hide stats

 ld de,$0000                    ; Move text cursor to upper-left
 call DMD2
 ld hl,msg_colon                ; Display a colon to seperate HP and CP
 call DMX                       ; Display colon to right of HP
 pop hl                         ; Restore robot data address
 inc hl                         ; Move to ENERGY bytes
 push hl
 ld l,(hl)
 ld h,0
 call DMD                       ; Show Energy
info_stats_done:
 pop hl
 inc hl
 inc hl                         ; Proceed to shield data
 inc hl
 ld a,(hl)                      ; HL -> status 2 ; E = status (upper)
 and %00000011                  ; Mask off non-shield-related flags
 jr z,info_bar_lable0           ; Skip if there you have no shield points
 ld bc,$2201                    ; Coordinates of leftmost shield icon
 ld e,a                         ; A = E = number of shield icons draw (1-3)
info_bar_loop2:
 push de                        ; Save shield icon count
 ld hl,spr_shieldicon
 push bc                        ; Save icon position
 call PutSprClp                 ; Draw a shield icon
 pop bc                         ; Restore icon position
 ld a,b                         ; Move 6 pixels to the right...
 add a,6                        ;
 ld b,a                         ;
 pop de                         ; Restore icon count
 dec e                          ; Decrement icon count
 jr nz,info_bar_loop2
info_bar_lable0:
 pop bc                         ; Restore SELECTOR value (0-5)

 ld a,b
 call Name_lookup               ; HL -> Robot name string
 ld de,$0034                    ; Text position
 call DMX2                      ; Display name-of-robot
 IM 2
 ret

;------------------------------------------------
Enable_cursor:
;
; Enable a cursor at the position indicated by
; (SELECTOR).  Existing cursor are not effected.
;
; IN:
;     E = cursor type
;
; INFO:
;     0000 0001 = selection box
;     0000 0010 = targeting 'sight'
;     0000 0100 = highligher box
;
cursor_select    = %00000001
cursor_target    = %00000010
;------------------------------------------------
 ld a,(SELECTOR)                ; Load target position
Enable_cursor_0:
 ld hl,_battle_cursors - 1      ; Subtract 1 to counter the "inc" below
 inc a
 ld b,a
enable_cur_loop0:
 inc hl                  
 djnz enable_cur_loop0
 ld a,(hl)                      ; Load present cursor
 or e                           ; Set new cursor flag
 ld (hl),a                      ; Update cursor's table
 ret

Disable_cursor:
 ld a,(SELECTOR)
Disable_cursor_0:
 ld hl,_battle_cursors - 1      ; Substract 1 to over starting at 0
 inc a
 ld b,a
disable_cur_loop0
 inc hl
 djnz disable_cur_loop0
 ld a,(hl)
 and e
 ld (hl),a
 ret


Sync_with_rain:
 ld hl,_YL
 bit 0,(hl)
 jr nz,Sync_with_rain
 IM 1
 ret

;------------------------------------------------
Select_leftmost:
; Search for leftmost ally or opponent.  Updates
; SELECTOR so the cursors can be updated to
; the correct positions
;
; IN:  A = 1 (search for HOST-controlled robot)
;      A = 4 (search for GUEST-controlled robot)
;
; EFFECT: Selector will hold the position number
;         (0-5) of the leftmost robot.
;
; OUT: C  = targeting error. No target found
;      NC = success
; out on success:
;      HL = points to robot's number
;      A  = E = SELECTOR (position) number
;
;------------------------------------------------
 ld (find_on_shift),a
 xor a
 call Select_right_0
 ret

;
;             Scrolling Routines Library v1.0 by CrASH_Man          02/14/98
;
;
;   This is a collection of highly optimized scrolling routines for the
; TI-82.  Calling the routine will scroll the entire screen by one pixel in
; the direction specified.  No registers are destroyed, so you may modify
; the routine if your code does not need to save them.
;
;   For an example on how to use this library, please take a look at
; SCRTEST.ASM.
;
;   This code is fairly simple to port so anyone can port it to the TI-83,
; 85, or 86 with only a couple changes.
;
;   You may use these routines in your programs, but please give me credit
; for my work by including my name in the documentation and source.
;
;                                                               by CrASH_Man
;
;
; Usage:
;
;  CALL Scroll_U   Scrolls the screen up by one pixel
;  CALL Scroll_D   Scrolls the screen down by one pixel
;  CALL Scroll_L   Scrolls the screen left by one pixel
;  CALL Scroll_R   Scrolls the screen right by one pixel
;
; No extra memory addresses are used/needed.
;
;
; Revision History:
;
;  1.0 02/14/98 - Scrolling Routines completed                     148 bytes

Scroll_U:                       ; [16228 Clock Cycles]
;;  PUSH HL                       ; Save Registers
;;  PUSH DE
  PUSH BC
  LD  HL, GRAPH_MEM+12          ; Copy from one row below top
  LD  DE, GRAPH_MEM             ;   to top row
  LD  BC, 756                   ;   756 bytes
  LDIR
  LD  H, D                      ; Fill Blanks
  LD  L, E
  INC E
  LD  (HL), B
  LD  C, 11
  LDIR
  POP BC                        ; Restore Registers
;;  POP DE
;;  POP HL
  RET

Scroll_D:                       ; [16228 Clock Cycles]
;;  PUSH HL                       ; Save Registers
;;  PUSH DE
  PUSH BC
  LD  HL, GRAPH_MEM+755         ; Copy from one row above bottom
  LD  DE, GRAPH_MEM+767         ;   to bottom row
  LD  BC, 756                   ;   756 bytes
  LDDR
  LD  H, D                      ; Fill Blanks
  LD  L, E
  DEC E
  LD  (HL), B
  LD  C, 11
  LDDR
  POP BC                        ; Restore Registers
;; POP DE
;;  POP HL
  RET

CLEAR_CENTER:
 ld hl,GRAPH_MEM+516            ; Get area that CLEAR_LINE1 won't cover
 ld de,GRAPH_MEM+517
 ld (hl),%00000000
 ld bc,252-1
 ldir
CLEAR_LINE1:                    ; Erase damage numbers
 ld hl,GRAPH_MEM+432
 ld de,GRAPH_MEM+433
 ld (hl),%00000000
 ld bc,84-1
 ldir
 ret

;RAND_SPECIAL:
; ld a,(BOSS)
; cp 255                         ; Linked gameplay? 255 = yes , else = no
; jr z,use_shared_rand
; call RAND                      ; If not linked, use a normal random number
; ret
;use_shared_rand:
; ld a,(shared_rand)             ; Recall the host-generated number
; ret

REINSTALL_INT:
 ld hl,$8400            ; APD_BUF address is $8228
 ld de,$8401            ;
 ld (hl),$82            ; The vector table will point to $8282 (within APD)
 ld bc,256              ; 256 = required size of all int vectors
 ldir                   ; Create table!

 ld hl,BATTLE_INTX      ; HL -> Default interrupt sequence
 ld de,$8282            ; DE -> Interrupt code destination (within APD BUF)
 ld bc,382              ; BC = maximum possible Interrupt size
 ldir                   ; Copy

 ld a,$84               ; High byte of int table's address (address is $8400)
 ld i,a                 ; Update Interrupt register
 IM 2                   ; Active maskable interrupt #2
 EI                     ; Enable!
 ret

;========================================================================
;========================================================================
;========================================================================
;========================================================================
;========================================================================
;========================================================================
;========================================================================

BATTLE_INTX:            ; BATTLE INTERRUPT X (used for all weather types)
 push ix                ; Save the index register too!
 exx                    ; Swap registers HL DE and BC
 ex af,af'              ; Swap register AF

 ld hl,_Y               ; Load the general-purpose pointer/register
 inc (hl)               ; Use _Y as an interrupt counter
 bit 0,(hl)             ; Check for even numbers
 jp nz,_BATTLE_INTZ     ; Exit if the counter holds an odd number

 DI                     ; Disallow a nested interrupt
 ld hl,GRAPH_MEM+420    ; Draw halfway line that divides the screen
 ld de,GRAPH_MEM+421    ;
 ld (hl),%11111111      ;
 ld bc,11               ;
 ldir                   ;

 ld a,(int_redraw_off)
 or a
 jr nz,skip_robots

 call Robot_draw        ; Re-draw the robots every other time the int runs

skip_robots:
 ld hl,_Y               ; _Y is my interrupt counter
 bit blink_rate,(hl)             
 jr z,int_cursors_skip
 call Battle_cursors
int_cursors_skip:
 ld hl,_YL
 inc (hl)
 ld a,(CURRENT_INT)     ; Prepare to jump to another interrupt segment.
 ld d,a
 ld a,(CURRENT_INTL)
 ld e,a
 push de                ; " JP (DE)
 ret                    ; 


BATTLE_INT1:            ; *** Rainy weather background effects
 bit 0,(hl)             ; See if the raindrops are still on-screen
 jp z,battle_int1_noshift       ; Raindrops still present, don't re-position
battle_int1_lable0 = $
 ld hl,spr_rain+15
 call Sprite_scroll
battle_int1_noshift:
 ld bc,$0000
 ld e,3                 ; Number of raindrop rows
battle_int1_loop1:
 push de                ; Save outter loop counter
 ld e,12                ; 12 raindrops in a row
battle_int1_loop0:
 push de                ; Save inner loop coutner
 push bc                ; Save current X-position
 ld hl,spr_rain
 call PutSprClp
 pop bc                 ; Restore latest X-position
 pop de                 ; Restore inner loop counter
 ld a,b                 ; Raindrops are 8 pixels apart, side-by-side
 add a,8                ;
 ld b,a                 ;
 dec e                  ; Decrement loop counter
 jr nz,battle_int1_loop0
 pop de                 ; Restore outter loop counter
 ld b,0                 ; Start over at the left side of the screen
 ld a,c                 ; Raindrops are 8 pixels apart, up-and-down
 add a,8                ;
 ld c,a                 ;
 dec e                  ; Decrement loop counter
 jr nz,battle_int1_loop1

BATTLE_INTZ:            ; BATTLE INTERRUPT CLOSER
 call CR_GRBCopy
_BATTLE_INTZ:
 exx
 ex af,af'
 pop ix                 ; Restore the index register too
 jp $0038

Robot_draw:
 ld ix,_squares         ; _squares indicates which robots occupy each square
 ld b,max_robots        ; B = maximum number of robots to draw
robot_int_loop0:
 push bc                ; Save loop counter
 ld hl,_hp - 21         ; Point to temporary robot data
 ld b,(ix)              ; Use the _squares contents to fetch the image number
 ld a,b
 or a
 jr z,robot_null_image
 cp 7                   ; Robot affected by FORCE?
 jr c,draw_normal
draw_special:
 sub 6
draw_normal:
 ld b,a

 ld de,21               ; Each robot 'file' is 21 bytes
robot_int_lookup:
 add hl,de              ; Fetch the robot 'file'...
 djnz robot_int_lookup
;---
 ld de,5
 add hl,de
 ld a,(hl)
 and %00001100
 jr nz,robot_stasis_image
 ld a,(hl)
 and %10000000
 jr nz,robot_ice_image
 ld de,13
 add hl,de
;---
 ld b,(hl)              ; Now use the image number to get the sprite addr.
 ld hl,spr_robot0 - 16
 ld de,16               ; Each sprite is 16 bytes
robot_int_lookup1:
 add hl,de              ; Fetch the sprite address...
 djnz robot_int_lookup1
robot_draw_now:
robot_int_lable0 = $ + 2
 ld bc,$0A1A
 push ix
 call PutSprClp
 pop ix
robot_int_nodraw:
 inc ix
 ld a,(robot_int_lable0)
 add a,12               ; Draw next robot 12 pixels to the left
 ld (robot_int_lable0),a
 pop bc                 ; Restore loop counter
 djnz robot_int_loop0
 ld a,$0A
 ld (robot_int_lable0),a
 ret
robot_null_image:
 ld hl,spr_blank
 jr robot_draw_now
robot_stasis_image:
 ld hl,spr_stasis
 jr robot_draw_now
robot_ice_image:
 ld hl,spr_ice
 jr robot_draw_now

;DELAY_special:
; push bc
; ld b,a
; ld a,(weather)         ; Get weather type ; 0 = normal  non-zero = rain
; or a
; ld a,b
; pop bc
; jp z,DELAY             ; Do regular delay in normal weather
; srl a                  ; For rain, the delays need to be half the size
; sub 3
; or %00000001           ; Avoid having a zero
; jp DELAY               ; Do "half" delay

;------------------------------------------------------------------------
Battle_cursors:
; Draw selection cursors like the square box and the targeting cursor
; This system looks at individual bits in the "_battle_cursors"
; table and treats them like flags according to...
;
; bit0  =  selection box 
; bit1  =  targeting box
;
; All cursors blink.  This is controlled by the interrupt (see above)
;
; The targeting cursor has the highest priority.  This means that if
; both cursor occupy the same square, only the targeting cursor will
; be drawn.
;------------------------------------------------------------------------
 ld de,_battle_cursors  ; Cursor reference table
 ld l,max_robots        ; Check 6 squares total
 ld bc,$0A1A            ; BC = leftmost cursor's position
battle_cur_loop0:
 push hl
 push bc                ; Save loop counter
 push de                ; Save table ofset
 ld a,(de)
 or a
 jr z,_cursor_end
;---
_cursor1:               ; ***Some code below was recently swapped.  This is
 bit 1,a                ; ...why I am starting at "cursor 1" instead of "0"
 jr z,_cursor0          ; Skip if clear
;---
 ld hl,spr_targeter
 call PutSprClp
 jr _cursor_end
_cursor0:
 bit 0,a
 jr z,_cursor_end          ; Skip if clear
;---
 ld hl,spr_selector
 call PutSprClp
_cursor_end:
 pop de
 pop bc
 inc de
 ld a,b
 add a,12
 ld b,a
 pop hl
 dec l
 jr nz,battle_cur_loop0
 ret

Sprite_scroll:
 ld a,(hl)
 ld (sprite_scr_lable0),a
 push hl
 pop de
 dec hl
 ld bc,7
 lddr                   ; <--- Don't use "LDIR"
sprite_scr_lable0 = $ + 1
 ld a,$00
 ld (de),a
 ld hl,spr_rain + 8
 ld b,8
sprite_s_loop0:
 ld a,(hl)
 rlc a
 ld (hl),a
 inc hl
 djnz sprite_s_loop0
 ret

