;################################################
;                                               #
;   -= ROBOT WAR MAPPING AND LEVEL CONTROL =-   #
;                                               #
; Supporting software:                          #
;                                               #
; render_map v2.0 [size = 341 bytes             #
;                                               #
;################################################
;                                               #
; TILESET LOADING                               #
;                                               #
; The tilemap v2.0 system allows for a maximum  #
; number of tiles of 32.  As of this moment,    #
; the number of tiles is Robot War numbers 64.  #
;                                               #
; Robot War will use "tile page" loading make   #
; all 64 tiles usable.  The first page is a     #
; fixed, "master set" of 16 the most used       #
; tiles.  The remaining three pages comprise    #
; 16 compressed sprites each.                   #
;                                               #
; NAME          RANGE           LABLE           #
; Indoor set    tiles (16-31)   gs001           #
; Outdoor set   tiles (32-47)   gs002           #
; Outdoor2 set  tiles (48-63)   gs003           #
;                                               #
;################################################
;                                               ;
;       STORY FLAGGING                          ;
;                                               ;
; The Story flags let the game keep track of    ;
; the player's progress. The flags are stored   ;
; in TEXT_MEM.  All flags are clear at the      ;
; start of the game.                            ;
;                                               ;
; STORY 0                                       ;
; Bit 0 -- "You won the tournament"             ;
; Bit 1 -- "You can use the builder"            ;
; Bit 2 -- "You can use the portable builder"   ;
; Bit 3 -- "Rain Only" (overrides Bit4 effect)  ;
; Bit 4 -- "Allow rain weather"                 ;
; Bit 5 -- "You are allowed to TRAVEL"          ;
; Bit 6 -- "Game has been saved at least once"  ;
; Bit 7 -- "Do not go to Edmunds' jail"         ;
;                                               ;
; STORY1                                        ;
; Bit 0 -- "Enemies in large numbers possible"  ;
; Bit 1 -- Elite Mode                           ;
; Bit 2 -- "Defeated REED"                      ;
; Bit 3 -- Beat mini-boss 1 in last level       ;
; Bit 4 -- Beat mini-boss 2 in last level       ;
; Bit 5 -- Beat mini-boss 3 in last level       ;
; Bit 6 -- Defeated Madison boss                ;
; Bit 7 -- BETH has joined your time            ;
;                                               ;
; STORY2                                        ;
; Bit 0 -- Beat DESOTO boss                     ;
; Bit 1 -- Mastered CAPE CITY target game       ;
; Bit 2 -- Mastered MADISON target game         ;
; Bit 3 -- Mastered DESOTO target game          ;
; Bit 4 -- Mastered LAKESHORE target game       ;
; Bit 5 -- Beat LAKESHORE bosses                ;
; Bit 6 -- Won the rooftop battle against REED  ;
; Bit 7 -- You defeated Lurid                   ;
;                                               ;
;################################################
; MAP DATA FORMAT                               #
;                                               #
; 1st = map width                               #
; 2nd = map height                              #
; 3rd = tileset to use (1, 2, or 3)             #
; 4th = special flags [infestation,save point]  #
; 5th = Connected map (word)                    #
; 5th = event location (word)                   #
; 6th = event pointer (word)                    #
; 8th = ...                                     #
; 9th = ...                                     #
; ...                                           #
; Nth = 0000                                    #
;                                               #
;################################################
graphics        =       $8300
graphics_page   =       $83

;------------------------------------------------
FIRST_RUN:
; Do the following whenever the game is played for the first time:
;
; STEP:
; - Set the CITY variable
; - Set a starting position
; - Declare map-to-load
; - Load CRONUS'S data to the savegame space
; - Load CRONUS's name to the savegame space

;---Set up starting variables
 ld a,first_city        ; A = "CAPE CITY"
 ld (CITY),a            ;
 ld a,starting_position
 ld (X_POS),a
 ld (Y_POS),a
 ld a,starting_map_position
 ld (COL_MAP),a
 ld (ROW_MAP),a
;---Put CRONUS in the gamer's file
 ld hl,e1               ; e1 = Enemy robot #1 (This is CRONUS)
 ld de,ATTACK           ; DE -> Robot data space
 ld bc,11
 ldir                   ; Load CRONUS'S data
 ld a,starting_HP       ; Setup CRONUS'S HitPoints
 ld (HPL),a             ;
 ld (HPmaxL),a          ;
 ld a,starting_EP
 ld (CPmax),a
;---Copy CRONUS's name
 ld hl,e_names+1        ; HL -> Enemy name strings
 ld de,NAME0            
 ld bc,7                
 ldir                   ; Copy
;---Set up a safe checkpoint map
 xor a
 ld (MAP_NUM),a
 ld (SAFEHOUSE),a
 inc a
 ld (MAP_DIFFICULTY),a  ; (Difficulty of enemies)
;---Get some starting Items
 ld a,library_repair    ; A = 26
 call Item_pickup       ; Get 1 "Repair"
 ld a,library_add       ; A = 37
 call Item_pickup       ; Get 1 "Add"
 ld a,library_negate    ; A = 39
 call Item_pickup       ; Get 1 "Negate"

LOAD_MAP:
MAP:
 ld hl,mapbuffer
 ld de,mapbuffer+1
 ld bc,1024-1
 ld (hl),0
 ldir

;---Get current map address from MAP_NUM
 ld hl,map_load_table   ; Get table of compressed map addresses
 ld a,(MAP_NUM)         ; This version contains maps 0 thru 21
 cp total_maps          ; Is this an unknown map (from a future revision?)
 jr c,load_known_map
 ld (find_entry_point),a; Put Edgar on-screen
 xor a                  ; Trying to load unknown maps send you to map #0
load_known_map:
 ld e,a                 ; Prepare for 16-bit add
 sla e                  ; x2 (because of words)
 ld d,0                 ; Clear upper
                        
 add hl,de              ; HL -> Current map (this program's addressing)
 ld de,CURRENT_MAP      ; Set current map
 ldi                    ;
 ldi                    ; <--Now CURRENT_MAP = address of map to load
;---Decompress map to mapbuffer
 ld de,(CURRENT_MAP)
 ld hl,mapbuffer        ; Destination = temporary buffer
 call DecodeZCP         ; Decompress the next level
;---Get map's properties
 ld hl,(CURRENT_MAP)
 ld bc,1024             ; Maximum data space (compressed)
 xor a                  ; Search for terminating zeros
 ld (auto_gateswap_time),a      ; Also, reset the gate time (prevent cheats)
 cpir
 ld a,(hl)              ; HL -> map width
 ld (MAP_WIDTH),a
 inc hl                 ; HL -> map height
 ld a,(hl)
 ld (MAP_HEIGHT),a
 inc hl                 ; HL -> map graphics set
 ld a,(hl)
 ld (CURRENT_TILESET),a
 push hl                ; Save map data pointer
 cp 2
 jr z,load_gs002
 cp 3
 jr z,load_gs003
;------------------------
;
; Tile Set Loaders
;
;------------------------
load_gs001:
 ld hl,gs001            ; INDOOR SET
 jr ts_load_now
load_gs002:
 ld hl,gs002            ; OUTDOOR SET
 jr ts_load_now
load_gs003:
 ld hl,gs003            ; OUTDOOR2 (bird-eye) SET
ts_load_now:
 ld de,gsxxx            ; Destination is temporary set
 ld bc,128              ; Length of tileset page
 ldir                   ; Copy tiles
 pop hl                 ; Restore map data pointer
 inc hl
 ld de,BACKGROUND
 ldi
 ldi
 inc hl                 ; HL -> Enemy Difficulty level on this map
 ld a,(hl)
 ld (MAP_DIFFICULTY),a

 ld a,(find_entry_point)        ; Should Edgar be placed on the special..
 or a                           ; .."map entry" tile?
 call nz,Entry_point            ;
 xor a
 ld (find_entry_point),a

game_loop_entry:        
 call Keystop           ; Hold until key activity stops
;------------------------
game_loop:
gameloop:

 halt
 halt
 halt
 halt

 call render_map        ; Draw the level
;--- ANIMATE WATER
 ld hl,water_tile
 ld b,8
water_anim:
 rrc (hl)
 inc hl
 djnz water_anim

;--- DRAW CHARACTER
edgar_draw:
 ld hl,edgar_animtable - 2
 ld a,(DIRECTION)
 inc a
 ld b,a
sprite_search:
 inc hl
 inc hl
 djnz sprite_search
 ROM_CALL($0033-$1A)
 ld a,(CURRENT_TILESET)
 cp 3                   ; Outdoor 2 graphics?
 jr nz,edgar_normal
edgar_small:
 ld a,(DIRECTION)
 bit 1,a
 jr z,small_1
 ld hl,short_edgar
 jr edgar_normal
small_1:
 ld hl,short_edgar+16
edgar_normal:
 ld a,(soy)             ; Comphensate for shifting map when drawing Edgar
 ld e,a                 ; E = A = Y Pixel Offset
 ld a,(ssy)             ; A = Y Tile Offset
 sla a
 sla a
 sla a                  ; A = 8 * (Tile Offset)
 ld c,a                 ; C = 8 * (Tile Offset)
 ld a,(Y_POS)           
 sub c                  ; A = (Y_POS) - 8 * (Tile Offset)
 sub e                  ; A = (Y_POS) - 8 * (Tile Offset) - Pixel Offset
 ld c,a                 ; C = (Y_POS) - 8 * (Tile Offset) - Pixel Offset

 ld a,(sox)             ; Comphensate for shifting map when drawing Edgar
 ld e,a                 ; E = A = X Pixel Offset
 ld a,(ssx)             ; A = X Tile Offset
 sla a
 sla a
 sla a                  ; A = 8 * (Tile Offset)
 ld b,a                 ; B = 8 * (Tile Offset)
 ld a,(X_POS)
 sub b                  ; A = (X_POS) - 8 * (Tile Offset)
 sub e                  ; A = (X_POS) - 8 * (Tile Offset) - Pixel Offset
 ld b,a                 ; B = (X_POS) - 8 * (Tile Offset) - Pixel Offset
 call PutSprClp         ; Draw sprite (HL) at B = verticle, C = horizontal



;--- GAME CLOCK
 ld hl,FRAME
 inc (hl)
 ld a,(hl)
 and clock_mask
 or a
 jr nz,clock_escape
seconds_inc:
 push hl
;--- ANIMATE REPAIR TILE
 ld hl,temporary_set+16
 ld de,temporary_set+16+8
 ld b,8
repair_anim:
 ld a,(de)
 ld c,(hl)
 ld (hl),a
 ld a,c
 ld (de),a
 inc hl
 inc de
 djnz repair_anim
 pop hl

 dec hl                 ; HL -> SECONDS
 inc (hl)
 ld a,(hl)
 cp 60
 jr nz,clock_escape
minutes_inc:
 ld (hl),0              ; Reset seconds count
 dec hl
 inc (hl)
 ld a,(hl)
 cp 60
 jr nz,clock_escape
hours_inc:
 ld (hl),0
 dec hl
 inc (hl)
clock_escape:

; ld hl,0                       ; STACK DISPLAY (for debugging)
; add hl,sp
; ld de,30
; call DMD2

 set 3,(IY+$05)                 ; White text

 ld hl,str_gold_spaced          ; Display Player's Gold here
 ld de,$0000                    
 call DMX2                      ; Write "gold:"

 ld hl,GOLD
 ROM_CALL($0033-$1A)
 ld de,$0012
 call DMD2

 res 3,(IY+$05)                 ; Normal text


;--- TILE CHECKING
auto_gateswap:
auto_gateswap_time = $ + 1
 ld a,$00
 or a
 jr z,auto_gateswap_escape
 ld hl,FRAME
 cp (hl)
 jr nz,auto_gateswap_escape
 call z,gateswap
 ld hl,auto_gateswap_time
 ld (hl),0
auto_gateswap_escape:

 call Underfoot         ; Check current floor tiles

;--- REFRESH
 call CR_GRBCopy        ; Display
 ld a,%11111111         ; Reset KEY port
 out (1),a              ;
 ld a,%11011111         ; Mask out irrelevant keys.
 out (1),a              ; 
 in a,(1)               ; Scan for 2ND and ALPHA
 bit 7,a                ; ALPHA ?
 jp z,Game_options

 ld a,%11111111         ; Reset KEY port
 out (1),a              ;
 ld a,%10111111         ; Mask out irrelevant keys.
 out (1),a              ; 
 in a,(1)               ; Scan for 2ND and ALPHA
 bit 5,a                ; 2ND?
 jp z,Game_click

; --- MOVEMENT
momentum:                       ; Cause EDGAR to move 8 pixels at a time
 ld a,(Y_POS)
 and %00000111                  ; Is EDGAR between tiles?
 jr z,no_verticle_moment        ; If not, do not adjust Y position

 ld a,(DIRECTION)               ; Is EDGAR traveling up or down
 cp 16                          ; 16 = downward momement
 jr z,step_down_8
step_up_8:
 call up_scroll_eligible
 ld a,(Y_POS)
 dec a                          ; DEC does not affect carry flag
 ld (Y_POS),a
 jp c,gameloop
 ld hl,soy
 call scroll_dec
 jp gameloop
step_down_8:
 call down_scroll_eligible
 ld a,(Y_POS)
 inc a                          ; INC does not affect carry flag
 ld (Y_POS),a
 jp c,gameloop
 ld hl,soy
 call scroll_inc
 jp gameloop

no_verticle_moment:
 ld a,(X_POS)
 and %00000111                  ; Is EDGAR between tiles?
 jr z,no_horizontal_moment      ; If not, do not adjust X position

 ld a,(DIRECTION)               ; Is EDGAR traveling up or down
 cp 8                           ; 0-7 = right momement
 jr c,step_right_8
step_left_8:
 call Direction_inc             ; Do EDGAR's walking frames
 call left_scroll_eligible      ; Don't scroll if close to map edge
 ld a,(X_POS)                   ; C= Can't Scroll  NC= You can Scroll
 dec a                          ; (DEC does not affect carry
 ld (X_POS),a
 jp c,gameloop
 ld hl,sox
 call scroll_dec
 jp gameloop
step_right_8:
 call Direction_inc             ; Do EDGAR's walking frames
 call right_scroll_eligible
 ld a,(X_POS)
 inc a
 ld (X_POS),a
 jp c,gameloop
 ld hl,sox
 call scroll_inc
 jp gameloop
no_horizontal_moment:

 xor a
 ld (character_motion),a

 ld a,%11111111                 ; Clear KEYS port
 out (1),a                      ; 
 ld a,%11111110                 ; Mask out irrelevant keys.  Ignore all...
 out (1),a                      ; ...except UP, DOWN, LEFT, RIGHT
 in a,(1)                       ; Get keys
 bit 2,a                        ; < Check for RIGHT
 jp z,Trywalk_right
 bit 1,a                        ; < Check for LEFT
 jp z,Trywalk_left
 bit 0,a                        ; < Check for DOWN
 jp z,Trywalk_down
 bit 3,a                        ; < Check for UP
 jp z,Trywalk_up

 ld a,$c9
 ld (character_motion),a

 jp game_loop








Game_options:
 ld hl,STORY0
 bit 6,(hl)
 call z,SAVE            ; Quick-save all new games!

 call Keystop           ; Hold until key activity stops

 ld hl,GRAPH_MEM+147
 ld de,$0626
 call WINDOW

 ld hl,menu_game_options
 ld de,$0D20
 ld a,6
 call MENU
 cp G_ALPHA
 jp z,game_loop_entry
 ld a,c
 cp 6
 jp z,Exit
 cp 5
 jp z,BUILDER
 cp 4
 jp z,Travel
 cp 3
 jr z,Robot_option
 cp 2
 jp z,Save_option
 jp game_loop_entry

Robot_option:
 ld hl,NAME0
 ld ix,HP
 call ROBOTS
 cp G_ALPHA
 jp z,game_loop_entry

 call Robot_lookup2             ; Reject blank selections ; Check robot data
 xor a
 cp (ix+17)                     ; Robots with no "level" are considered blank
 jp z,Robot_option              ; 

 ld a,c                         ; Save robot choice in short-term 'register'
 ld (_X),a                      ;

 ld hl,GRAPH_MEM+102
 ld de,$050E
 call WINDOW

 ld hl,menu_robot_options
 ld de,$0938
 ld a,2
 call MENU
 cp G_ALPHA
 jp z,Robot_option
 ld a,c
 cp 1
 jp z,Item_option
Recycle:
 ld hl,msg_recycle_notice
 call TALKBOX
 call no_yes_menu
 jr z,Robot_option
 ld a,c
 cp 1
 jr z,Robot_option
;---Able to Recycle?
 ld a,(BOTS)                    ; Get number of bots ; 0 means you have one,..
                                ; ...1 means you have two
 or a                           ; Check for zero/non-zero
 jp nz,recycle_now              ; Have more than one?  You can recycle!
 ld hl,msg_cant_use_that
 call TALKBOX
 jr Robot_option
recycle_now:
 ld a,(_X)                      ; Who's being recycled?
 ld c,a
 push af                        ; Save selected robot
 call Robot_lookup2             ; IN: C = Robot ; Out: IX->Robot's data
 ld d,0
 ld e,(ix+17)                   ; Award = 4 * (Robot's Level)
.db $CB,$33                     ; SLL E
 rl d
.db $CB,$33                     ; SLL E
 rl d
 ld hl,(GOLD)
 add hl,de
 ld (GOLD),hl
 push ix                        ; "LD HL,IX"
 pop hl                         ;
 ld d,h                         ; Erase robot!
 ld e,l                         ;
 inc de                         ;
 ld bc,21 - 1                   ;
 ld (hl),0                      ;
 ldir                           ;
 pop af                         ; Restore selected robot
 ld hl,NAME0 - 11
 ld de,11                       ; DE = Size of name strings
 ld b,a
name_erase:
 add hl,de
 djnz name_erase
 ld d,h                         ; HL -> Deleted Robot's name
 ld e,l
 inc de                         ; DE -> Deleted Robot's name + 1
 ld bc,11 - 1
 ld (hl),0
 ldir                           ; Erase name!

 ld hl,BOTS                     ; Decrement BOT counter
 dec (hl)
 jp Robot_option




Travel:
 ld a,(CURRENT_TILESET)
 cp 3
 jp z,game_loop_entry   ; Don't allow traveling when already on world map
 cp 1
 jp z,event051          ; Don't allow traveling when indoors

 ld hl,STORY0                   ; Is the player allowed to leave yet?
 bit 5,(hl)                     ;
 jr nz,leave_map_now
 call _event012_001
 jp game_loop_entry

leave_map_now:
 ld a,2
 ld (MAP_NUM),a

 ld a,32
 ld (MAP_HEIGHT),a
 ld (MAP_WIDTH),a

 ld a,(ROW_MAP)
 ld c,a
 ld a,(COL_MAP)
 ld b,a

 ld hl,LOAD_MAP
 push hl                        ; Prep the stack for an artificial jump
 jp entry_point_found


Item_option:
 ld a,(_X)
 ld b,a
 ld hl,NAME1 - 11 - 11          ; (Double-offset)
 ld de,11
Item_option_111:
 add hl,de
 djnz Item_option_111
Item_option_lable0:
 ld ix,ITEM1
 call ITEM                      ; Out: A=keypress C= item ###
 cp G_ALPHA
 jp z,game_loop_entry
 ld a,c
 or a
 jp z,Item_option

 push bc                        ; Save Item chioice
 ld hl,GRAPH_MEM+112            ; "Use/Drop" window
 ld de,$0410                    ;
 call WINDOW                    ;

 ld hl,menu_use_give            ; "Use/Drop" menu
 ld de,$0B2A                    ;
 ld a,2                         ;
 call MENU                      ;
 pop de                         ; Restore Item choice
 cp G_ALPHA
 jr z,Item_option
 ld a,c
 cp 2
 jp z,Drop
Use:
 ld a,(_X)              ; Restore selected robot from short-term 'register'
 ld c,a
 ld a,e                         ; A = E = Item choice
 cp last_melee + 1      ; Are you equipping a melee weapon?
 jp c,Equip
 cp last_armor + 1      ; Is this a non-usable item (Special item like KEY?)
 jp nc,undroppable              ; Show: "Can't do that"
 cp first_armor         ; Are you equipping armor?
 jp nc,Equip
 push de                        ; Save item choice
 cp library_renewal
 jp z,use_renewal
 cp library_energy
 jp z,use_energy
 cp library_energy_dx
 jp z,use_energy_dx
 cp library_repair
 jp z,use_repair
 cp library_repair_dx
 jp z,use_repair_dx
 cp library_rebuild
 jp z,use_rebuild
 pop de                 ; Fix the stack
 jp undroppable

hp_check:
 call Robot_lookup2
 ld a,(ix)
 ld c,(ix+1)
 or c
 ret

use_rebuild:
 ld hl,use_escape
 push hl
 call hp_check
 ret nz
 jr use_repair_no_hp_check

use_repair:
 ld hl,use_escape
 push hl                ; <-- Use this to "fake jump" below
use_repair_lable0:
 call hp_check          ; Have HP?
 ret z                  ; Do artificial jump, if not
use_repair_no_hp_check:
 ld h,(ix)
 ld l,(ix+1)                    ; HL = Current HP
 ld de,repair_heal_rate
 add hl,de                      ; HL = Current HP + 50
 ld (ix),h
 ld (ix+1),l                    ; Update HP
 ld d,(ix+6)
 ld e,(ix+7)    
 call CP_HL_DE                  ; "Is HP > HP max?"
 ret c; <-- Actually jumps to use_escape.  Useless use_repair_dx was used                          ; "If not, return"
 ld (ix),d
 ld (ix+1),e                    ; "If so, then HP = HPmax"
 ret ; <-- Actually jumps to use_escape.  Useless use_repair_dx was used

use_repair_dx:
 call use_repair_lable0
 call use_repair_lable0
 jp use_escape

use_energy:
 ld hl,use_escape
 push hl                ; <-- Use this to "fake jump" below
use_energy_lable0:
 call Robot_lookup2     ; IN: C=Robot to affect  OUT: IX->Bot's HP byte
 ld a,(ix+2)
 add a,energy_heal_rate         ; Restore 8 energy points
 ld (ix+2),a
 cp (ix+3)                      ; Full EP yet?
 ret c; <-- Actually jumps to use_escape.  Useless use_energy_dx was used                 
 ld a,(ix+3)
 ld (ix+2),a            
 ret ; <-- Actually jumps to use_escape.  Useless use_energy_dx was used

use_energy_dx:
 call use_energy_lable0
 call use_energy_lable0
 jp use_escape

use_renewal:
 call Robot_lookup2
 ld (ix+$05),0          ; Clear burn, force, ice, and shields
use_escape:
 pop bc                 ; Restore previous "push de" ; C = Item serial
 call Item_deduct4      ; IN: C = item
 ; jp game_loop_entry
 jp Robot_option

Robot_lookup2:           ; IN: C = Robot number  OUT: IX -> Robot Data Space
 ld b,c
 ld ix,HP - 21
 ld de,21
robot_search_loop:
 add ix,de
 djnz robot_search_loop
 ret



Drop:
 ld a,e
 ld (drop_this),a
 call Sure
 call no_yes_menu
 jp z,Item_option
 ld a,c
 cp 1
 jp z,Item_option
 ld hl,ITEM1
 ld bc,max_item_types + 1
drop_this = $ + 1
 ld a,$00
 cpir
 dec hl
 call Item_deduct3
 jp Item_option
undroppable:
 ld hl,str_cant_use_that
 jp Item_option_lable0

Drop_all:                       ; A = item to drop
 ld hl,ITEM1
 ld bc,max_item_types
 cpir
 dec hl                         ; Go back 1 bytes
 cp (hl)
 ret nz
 ld (hl),0
 ld de,2 * max_item_types
 add hl,de
 ld (hl),0
 ret

Save_option:
 call LOAD_SAVE_WINDOW
 ld hl,str_save
 ld de,$1407
 call DMX2
 call CR_GRBCopy
 call CONFIRM_ALPHA
 cp G_ALPHA
 jp z,Game_options

SAVE2:
Save_now:
;                                               ;
; Save and return to titlescreen                ;
;                                               ;
; Using "Save_now" will return you to the       ;
; homescreen                                    ;
;-----------------------------------------------;
 ld hl,save_and_clear
 push hl                        ; Place the lable on the stack for later

SAVE:
;                                               ;
; Basic save.  Call this to save *without*      ;
; returning to the mainscreen                   ;
;-----------------------------------------------;
 call verification      ; Generate save game "checksum"
 ld (hl),a              ; Update "checksum"

 ld hl,STORY0
 set 6,(hl)             ; Disable first-time auto-save function

 call RELOC_INSTALL             ; Reinstall Relocation Code

 ld hl,NewVarName               ; HL -> name of samegame variable
 rst 20h                        ; Copies (HL) -> (OP1)
 call _CHKFINDSYM               ; See if the variable exists already...
 inc de                         ; Load Savegame data into text_memory...
 inc de                         ; ...and text_memory2 for later use
                                ; Now DE=address of savegame
 ld hl,TEXT_MEM
 ld bc,SAVE_SIZE1
 ldir
 ld hl,TEXT_MEM2
 ld bc,SAVE_SIZE2
 ldir

 ret                    ; <-- IMPORTANT: This "ret" will send the PC...
                        ; to "save_and_clear" if "SAVE2" was called...
                        ; OR it will return as normal.
save_and_clear:

 ld hl,TEXT_MEM         ; Erase TEXT_MEM's to prevent data from this...
 ld de,TEXT_MEM+1       ; ...file from "escaping" into other files.
 ld bc,SAVE_SIZE1 - 1   ;
 ld (hl),0              ;
 ldir                   ;

 ld hl,TEXT_MEM2+7              ; Clear TEXT_MEM2, but *DO NOT!* overwrite..
 ld de,TEXT_MEM2+8+1            ; ...the savegames name string or...
 ld bc,SAVE_SIZE2 - 7 - 1       ; ...its terminating zero.
 ld (hl),0                      ;
 ldir                           ;

 jp Init_lable1            ; Go *all the way* to the start of Robot War!

;------------------------
Keystop:
;
; Wait for key activity
; to stop.  Prevent one
; keypress from being
; read as 2.
;
;------------------------
 ld b,quieting_duration
keystop_loop:
 halt
 call GET_KEY
 cp G_NONE
 jr nz,Keystop
 djnz keystop_loop
 ret

Trywalk_up:
 ld a,17                ; A = Edgar's back
 ld (DIRECTION),a
 call north_edge        ; Is Edgar on the upper edge of the map?
 jp z,gameloop          ;
 call Solid             ; Is the next tile passable?
 call up_scroll_eligible
 jr c,freewalk_up       ;
 ld hl,soy
 call scroll_dec
freewalk_up:
 ld hl,Y_POS
 dec (hl)               ; Trigger upward movement
 jp gameloop

Trywalk_down:
 ld a,16                ; A = Edgar's front
 ld (DIRECTION),a
 call south_edge        ; Is Edgar on the lower edge of the map?
 jp z,gameloop          ;
 call Solid             ; Is the next tile passable?
 call down_scroll_eligible
 jr c,freewalk_down
 ld hl,soy
 call scroll_inc
freewalk_down:
 ld hl,Y_POS
 inc (hl)               ; Trigger download movement
 jp gameloop

Trywalk_left:
 ld a,8                 ; A = Edgar facing the right
 ld (DIRECTION),a
 call west_edge         ; Is Edgar on the left edge of the map?
 jp z,gameloop          ;
 call Solid             ; Is the next tile passable?
 call left_scroll_eligible
 jr c,freewalk_left     ;
 ld hl,sox
 call scroll_dec
freewalk_left:
 ld hl,X_POS
 dec (hl)
 jp gameloop

Trywalk_right:
 xor a                  ; A = Edgar facing the left
 ld (DIRECTION),a
 call east_edge         ; Is Edgar on the right edge of the map?
 jp z,gameloop          ;
 call Solid             ; Is the next tile passable?
 call right_scroll_eligible
 jr c,freewalk_right
 ld hl,sox
 call scroll_inc
freewalk_right:
 ld hl,X_POS
 inc (hl) 
 jp gameloop

;------------------------
;
Scroll_eligibility:
;
; Don't allow scrolling
; is close to map edge.
;
; Out: NC = scroll (fail)
;       C = no scroll (pass)
;
;------------------------
down_scroll_eligible:
 ld a,(MAP_HEIGHT)
 sla a
 sla a
 sla a
 sub 40               
 ld e,a               
 ld a,(Y_POS)
 cp 24
 ret c
 cp e
 jr nc,Scroll_e_fail
 jr Scroll_e_pass

up_scroll_eligible:
 ld a,(MAP_HEIGHT)
 sla a
 sla a
 sla a
 sub 40-1               
 ld e,a               
 ld a,(Y_POS)
 cp 24+1
 ret c
 cp e
 jr nc,Scroll_e_fail
 jr Scroll_e_pass

left_scroll_eligible:
 ld a,(MAP_WIDTH)
 sla a
 sla a
 sla a
 sub 56-1
 ld e,a
 ld a,(X_POS)
 cp 40+1
 ret c
 cp e
 jr nc,Scroll_e_fail
 jr Scroll_e_pass

right_scroll_eligible:
 ld a,(MAP_WIDTH)
 sla a
 sla a
 sla a
 sub 56
 ld e,a
 ld a,(X_POS)
 cp 40
 ret c
 cp e
 jr nc,Scroll_e_fail

Scroll_e_pass:
 xor a                          ; Clear Carry flag
 ret
Scroll_e_fail:
 scf
 ret

;------------------------
Edge_check:             ;
;                       ;
; Determine if the      ;
; character is          ;
; attempting to walk    ;
; out of a map.         ;
;                       ;
; If not true, return   ;
; to the main loop      ;
;                       ;
; If true, load the     ;
; next appropriate map  ;
;                       ;
; OUT:                  ;
;NotZero == not an edge ;
;   Zero == edge        ;
;------------------------
north_edge:
 ld a,(Y_POS)
low_edge_check:
 or a
 ret nz
 jr edge

south_edge:
 ld a,(Y_POS)
 srl a
 srl a
 srl a
 ld e,a
 ld a,(MAP_HEIGHT)
 dec a
 cp e
 ret nz
 jr edge

west_edge:
 ld a,(X_POS)
 jr low_edge_check

east_edge:
 ld a,(X_POS)
 srl a
 srl a
 srl a
 ld e,a
 ld a,(MAP_WIDTH)
 dec a
 cp e
 ret nz
edge:
 ld hl,(CURRENT_MAP)    ; Get inforamtion on current map
 ld bc,1024             ; 1024 = Maximum map size
 xor a                  ; Search for terminating zero
 cpir                   ; HL -> First byte of special map constants
 inc hl
 inc hl
 inc hl
 inc hl
 inc hl                 ; HL -> Connected Map
 ld a,(hl)
 or a
 jr z,edge_escape       ; Return to gameloop.  Treat edge as solid tile

 ld (MAP_NUM),a
 ld a,$FF                ; Place Edgar on the special "entry point" tile..
 ld (find_entry_point),a ; ..on the next map.
 pop hl                 ; Trash the xxxx_edge call
 jp LOAD_MAP

edge_escape:
 pop hl                 ; Trash the call
 jp gameloop


Entry_point:
 ld hl,mapbuffer        ; Load tempoary map data
 ld c,0 - 1             ; C = Row counter (Y)
 ld a,(MAP_WIDTH)
 ld e,a
entry_p_newrow:
 ld d,0                 ; Loop counter
 ld b,0                 ; B = Column counter    ; Reset column counter
 inc c                  ; Indicate "next row"
entry_p_search:
 ld a,(hl)              ; Search for the "entry point tile"
 cp 4*8                 ;
 jr z,entry_point_found ;
 ld a,d                 ; Move to newrow every [MAP_WIDTH] cycles
 cp e                   ;
 jr z,entry_p_newrow    ;
 inc d                  ; Increment loop count
 inc b                  ; Increment column count
 inc hl                 ; Move to next tile
 jr entry_p_search

entry_point_found:      ; IN: Edgar's.. B = column C = row
 ld a,(MAP_WIDTH)
 cp LCD_WIDTH + 1
 jr c,no_horiz_offset
 ld a,b
 cp 7
 jr c,no_horiz_offset
 sub 5
 ld (ssx),a
 add a,5
 jr make_horiz_offset
no_horiz_offset:
 xor a
 ld (ssx),a
make_horiz_offset:
 ld a,b
 sla a
 sla a
 sla a
 ld (X_POS),a

;---
 ld a,(MAP_HEIGHT)
 cp LCD_HEIGHT + 1
 jr c,no_vert_offset
 ld a,c
 cp 5
 jr c,no_vert_offset
 sub 3
 ld (ssy),a
 jr make_vert_offset
no_vert_offset:
 xor a
 ld (ssy),a
make_vert_offset:
;---
 ld a,c
 sla a
 sla a
 sla a
 ld (Y_POS),a
 ret


;------------------------
Game_click:             ;
;                       ;
; Game click checks the ;
; tile you're facing    ;
; when you press the    ;
; 2nd key.              ;
;-----------------------;
 call Tilechecks        ; Update DIRECTION and G_POS
 ld hl,(CURRENT_MAP)    ;
 ld bc,1024             ; Maximum map data size
 xor a                  ; Search for terminating zero
 cpir                   ; Search!  HL -> Data after map bytes
 ld de,10-3             ; Move HL past other map data (width,height,set,etc.)
 add hl,de              ; HL -> first event location

click_event_search:
 inc hl                 ; Locate start of event data
 inc hl                 ;
 inc hl                 ; HL -> event location (WORD)
 ld e,(hl)              ; Get event location byte
 inc hl                 ;
 ld d,(hl)              ; DE == 0 - 65535

 xor a                  ; Check for *End of line* ($0000)
 or d                   ;
 or e                   ;
 jr z,no_event_found    ;

 push hl                ; Save event data pointer
 push de                ; Save event tile location
 ld bc,$0000
 ld ix,G_POS
 ld h,(ix)
 ld l,(ix+1)
 ld a,(DIRECTION)
 cp 8
 jr c,click_right
 cp 16
 jr c,click_left
 jr z,click_down
click_up:
 ld a,(MAP_WIDTH)
 ld c,a
 sbc hl,bc
 jr click_now
click_down:
 ld a,(MAP_WIDTH)
 ld c,a
 add hl,bc
 jr click_now
click_left:
 dec hl
 jr click_now
click_right:
 inc hl
click_now:

 pop de                 ; DE == Event Coordinates
 call CP_HL_DE          ; HL == Coordinates clicked
 pop hl                 ; Restore event data pointer.  Fix the stack
 jp nz,click_event_search
 inc hl                 ; HL -> Event address
 ld de,game_click_dest
 ldi
 ldi
 call Keystop           ; Halt key scanning
game_click_dest = $ + 1
 call $0000
 jp game_loop_entry     ; Reenter the system.  (Reset key ports)

no_event_found:
 ld hl,G_AHEAD
 ROM_CALL($0033-$1A)
 ld a,(hl)

 call do_tile
 jp game_loop_entry

;------------------------
Underfoot:              ;
;                       ;
; Underfoot             ;
; checks what tile you  ;
; are standing on.      ;
;                       ;
; Underfoot only scans  ;
; while you are moving  ;
;                       ;

 ld a,(Y_POS)           ; Return unless Edgar is centered on the tile
 ld e,a                 ;
 ld a,(X_POS)           ;
 or e                   ;
 and %00000111          ;
 ret nz                 ;

character_motion = $            ; * Polymorph avoids getting "stuck" on 
 .db 0                          ; these tiles....
                                
 ld a,$c9                       
 ld (character_motion),a        

 call Random_battles            ; Try a random battle, if allowed

 call Tilechecks                ; Update position data
 ld a,(G_NUM)           ; Get #value of tile being stood on
;---Anytime Events-------
events:
 cp 1*8                 ; 1 = DOOR
 jp z,event011          ; "You opened a door"

 cp 2*8                 ; 2 = WATER TILE
 jp z,event001          ; "You loose"

 cp 6*8                 ; 6 = ON Button
 jp z,event002          ; "Swap buttons and gates" "Start Gate Timer"

 cp 7*8                 ; 7 = OFF BUTTON
 jp z,event003          ; "Reset Game Timer"

 cp 5*8                 ; 8 = EVENT 054
 jp z,event054          ; Edgar gets ambushed (Used only once)
;---Anytime Events activated by 2nd key
do_tile:
 cp 9*8                 ; 5 = KEY
 jp z,event017          ; "You picked up a key!"

 cp 15*8                ; 15 = Locked Block
 jp z,event027          ; Try to open

 ld a,(CURRENT_TILESET)
 cp 3
 jr z,outdoor2_only
 cp 2
 jr z,outdoor_only
;---Indoor only----------
indoor_only:

 ld a,(G_NUM)

 cp 17*8                ; 16 = Alarm Button
 jp z,event029          ; "You triggered an alarm"

; cp 18*8                ; 18 = SCARE TACTICS
; jp z,event051          ; "Edmund tries to pshyce you out"

 cp 19*8                ; 20 = REPAIR TILE
 jp z,event004          ; "Repair all robots"

 cp 21*8                ; 21 = Deadly Spikes
 jp z,event001          ; "You loose automatically"

 ret
;---Outdoor only---------
outdoor_only:
 ld a,(G_NUM)
 cp 20 * 8              ; 20 = CHAIN FENCE
 jp z,event036          ; "Walk 'down"
 ret
;---Outdoor2 only--------
outdoor2_only:
 ld a,(G_NUM)
 cp 17 * 8              ; 49 = CITY'S ENTERANCE
 jp z,event034

 cp 21 * 8
 jp z,event037
 ret

Tilechecks:
 ld hl,$0000            ; HL tracks Grid Position
 ld a,(Y_POS)
 srl a                  ; Get verticle tile number by dividing by 8
 srl a                  ;
 srl a                  ;
 or a
 jr z,tilechecks_lable0
 ld b,a
 ld a,(MAP_WIDTH)
 ld e,a
 ld d,0
tilechecks_loop0:
 add hl,de
 djnz tilechecks_loop0
tilechecks_lable0:      ; HL = current Grid Row
 ld d,0                 ; Clear upper byte
 ld a,(X_POS)
 srl a
 srl a
 srl a
 ld e,a
 add hl,de              ; HL = Current Grid Number
 ld ix,G_POS
 ld (ix),h
 ld (ix+$01),l
 ex de,hl               ; DE = Current Grid Number
 ld hl,mapbuffer
 add hl,de              ; HL -> Tile you're standing on
 ld a,(hl)
 ld (G_NUM),a           ; Update 

 ld a,(DIRECTION)
 cp 8
 jr c,look_right
 cp 16
 jr c,look_left
 jr z,look_down

look_up:
 ld a,(MAP_WIDTH)
 ld e,a
 ld d,0
 call NEG_DE
 jr look_now
look_down:
 ld a,(MAP_WIDTH)
 ld e,a
 ld d,0
 jr look_now
look_left:
 ld de,-1
 jr look_now
look_right:
 ld de,1
look_now:
look:
 add hl,de              ; HL == Grid number you're *facing*

 ld ix,G_AHEAD
 ld (ix),l
 ld (ix+$01),h
 ret

Direction_inc:
 ld a,(DIRECTION)
 inc a
 ld (DIRECTION),a
 ret

;------------------------
Solid:
;
; Check current tile for
; clear (passable) or
; solid (non-passable)
; status.
;
; Out:
;
;   Carry = clear tile
; NoCarry = solid tile
;
; Actions:
;
; Returns if clear
;
; Jumps back to loop if
; solid.
;
;------------------------
 call Tilechecks        ; Update position variables
 ld ix,G_AHEAD
 ld l,(ix)
 ld h,(ix+$01)
 ld a,(hl)
 and %01111111
 cp 64
 ret c
 pop hl                 ; Trash 1 call
 ld hl,REPEL_COUNT
 set 0,(hl)             ; 
 jp gameloop

;------------------------
;
; SCROLL
;
; IN: HL = <sox> or <soy>
;
;------------------------
scroll_inc:
 ld a,(hl)              ; HL -> {sox or soy}
 inc a                  ; Pixel scroll right or down
 and %00000111          ; Range= (0-7)
 ld (hl),a              ; Update pixel offset
 ret nz
 inc hl                 ; HL -> {ssx or ssy}
 inc (hl)               ; Tile Scroll right or down
 ret

scroll_dec:
 ld a,(hl)              ; HL -> {sox or soy}
 and %00000111          ; Range= (0-7)
 call z,scroll_dec_lable0
 dec a                  ; Pixel scroll left or up
 and %00000111
 ld (hl),a              ; Update pixel offset
 ret
scroll_dec_lable0
 ld b,a
 inc hl                 ; HL -> {ssx or ssy}
 ld a,(hl)
 or a
 jr nz,no_scroll_block
 pop hl
 ret
no_scroll_block:
 ld a,b
 dec (hl)               ; Tile Scroll left or up
 dec hl
 ret

;------------------------
; RANDOM BATTLES
;
; Enter a robot battle
; at random intervals
;;------------------------
Random_battles:
 ld a,(REPEL_COUNT)     ; Use repel counter so that you don't have a...
 or a                   ; ...battle every square (I hate that!)
 jr z,repel_done        ;
 dec a                  ;
 ld (REPEL_COUNT),a     ;
 ret
repel_done:
 call RAND
 cp battle_chance
 ret nc
 ld a,(auto_gateswap_time)      ; No battles if the gates blocks are open
 or a                           ;
 ret nz                         ;

 ld hl,(CURRENT_MAP)
 ld bc,1024
 xor a
 cpir
 ld de,7 - 1
 add hl,de                      ; HL -> <Enemy 1> - 1
 ld a,(hl)                      ; Are enemies allowed for this map?
 or a
 ret z                          ; If no enemies on map, exit

 push hl                        ; Save enemies list pointer for map
 call TEMPS_CLEAR2              ; Erase any possible residual data
 pop hl                         ; Restore enemies list pointer for map

 ld b,max_team                  ; 3 = Max Enemy team size
 ld c,max_team                  ; 3 = Leftmost enemies' position
 ld e,0
enemy_placement_loop0:
 push hl
 push bc
enemy_placement_loop1:
 call RAND
 and %00000011
 jr z,enemies_done              ; Only 3 enemies are listed for each map
 ld d,0
 ld e,a
 add hl,de                      ; HL -> Random Enemy
 ld b,(hl)                      ; A = Enemy number
 call Opponent_loader           ; Load enemy into battle data
 ld hl,STORY1
 bit 0,(hl)                     ; "Are multiple enemies allowed?"
 jr z,enemies_done2             ; If not, stop at 1 enemy
 pop bc
 pop hl
 inc c
 djnz enemy_placement_loop0
 push hl                ; Crash prevention
 push hl                ;
enemies_done:
 ld a,e
 or a
 jr z,enemy_placement_loop1
enemies_done2:
 pop hl                 ; Trash the Random_battles call
 pop hl                 ; Trash the Underfoot call
 pop hl
 pop hl

 call BATTLE            ; Battle!

 ld a,immunity_distance ; [see robot.h]
 ld (REPEL_COUNT),a     ; Disable next random batt until player moves 5 steps
 jp LOAD_MAP            ; Reload map.  Ignore entry points
enemies_escape:
 pop hl
 pop hl
 ret

;------------------------
;
; ITEM PICKUP
;
; This runs whenever you
; aquire a new item
;
; IN: A = Item Choosen (see Library.asm)
; OUT: Carry = Item Not picked up
;    NoCarry = Item pickup successful
;------------------------
Item_pickup:
 push af
 call Item_room
 jr nz,pickup_fail
 pop af
 ld hl,ITEM1            ; HL Points to Player's Item data
 ld bc,max_item_types   ; Search range (You can hold 8 item types)
 cpir                   ; Search for existing items
 jr z,item_pickup_not_new       ; Jump if you already have 1 of that item
item_pickup_new:
 ld hl,ITEM1            ; HL Points to Player's Item data
 ld bc,max_item_types   ; Search range (You can hold 8 item types)
 push af
 xor a
 cpir
 pop af
 dec hl                 ; Go back one byte
 ld (hl),a              ; Add item
 jr item_inc
item_pickup_not_new:
 dec hl                 ; Go backwards one byte
item_inc:
 ld de,2 * max_item_types
 add hl,de              ; HL -> Item's STOCK byte
 ld a,(hl)
 cp 9
 jr z,no_room
 inc (hl)
 xor a                  ; Clear carry flag
 ret
pickup_fail:
 pop hl                 ; Fix the stack
 scf                    ; Set carry flag
 ret



Item_room:
 ld e,a                 ; Save current item in E register
 ld hl,ITEM1            ; New Search
 ld bc,max_item_types   ; Search range (You can hold 8 item types)
 xor a                  ; Search for zeros now
 cpir                   ; Search!
 ret z                  ; If there's room, Carry=clear  Zero=Set

 ld hl,ITEM1            ; No empty spaces?  See if you already have that item type
 ld bc,max_item_types   ; Check entire list again
 ld a,e                 ; Search for current item
 cpir                   ; Search!
 ret z                  ; If there's room. Carry=clear  Zero=Set
no_room:
 ld hl,msg_no_room
 call TALKBOX
 ld a,1
 or a                   ; Set the NZ flag
 scf                    ; Set the carry flag
 ret



;------------------------
;
; ITEM TAKEAWAY
;
; Checks to see if you
; have an item, then
; it deducts one
;
; IN: A = item number (no zeros!)
; OUT Zero = you have it
;     NotZero = don't have
;------------------------
item_takeaway:
 ld hl,ITEM1            ; Item pointer
 ld bc,max_item_types   ; Search range
 cpir
 dec hl
 cp (hl)                ; Is this actually the item? Or did the search fail?
 ret nz                 ; Return if the search failed
 push hl
 pop ix
 call Item_deduct2      ; [See battleNT.asm or battle.asm]
 xor a                  ; Clear "zero" flag
 ret
;------------------------
;
; ITEM CHECK
;
; Checks to see if you
; have an item
;
; IN: A = item number (no zeros!)
; OUT Zero = you have it
;     NotZero = don't have
;------------------------
Item_check:
 ld hl,ITEM1
 ld bc,max_item_types   ; Search range
 cpir
 ret

;------------------------
;
; ITEM EQUIP
;
; Equip a valid item
;
; De-equip an old item if
; necessary, buy placing
; it back on the item list
;
; E = item to equip
;------------------------
Equip:
 ld a,e
 ld (equip_this),a
;---Test Item------------
 cp last_melee+1
 jr c,equip_test_000    ; Can equip melee weapons
 cp first_armor
 jp c,game_loop_entry   ; Can't equip power attacks or battle items
 cp last_armor + 1
 jp nc,game_loop_entry  ; Can't equip in-game items like KEYs
equip_test_000:
;---Do Equip Dialog------
 ld hl,msg_equip_whom   ; Display "Equip?"
 call TALKBOX
 call no_yes_menu
 jp z,game_loop_entry   ; Alpha cancels the equip
 ld a,c
 cp 1
 jp z,game_loop_entry   ; "No" cancels the equip
;---Take equipment from list
equip_this = $ + 1
 ld c,$00               ; * Polymorph
 call Item_deduct4      ; Immidiately remove this item from the list
;---Lookup old equipment-
 ld a,(_X)              ; Load selected robot from short-term 'register'
 ld b,a
 ld hl,HP - 21
 ld de,21
find_robot:
 add hl,de              ; Search for robot
 djnz find_robot
 ld a,(equip_this)
 cp last_melee+1        ; Equiping a melee weapon?
 jr c,equip_weapon
equip_armor:
 ld de,16               ; Armor bytes' offset
 jr equip_now
equip_weapon:
 ld de,11               ; Weapon bytes' offset
equip_now:
 add hl,de              ; HL -> Armor or Weapon bytes
 push hl                ; Save Armor/Weapon pointer
 ld a,(hl)
 or a                   ; "No previous armor/weapon?"
 jr z,no_de_equip
 ld (de_equip_this),a   ; A = item being de-equipped

 call Item_room         ; Room for the old armor on list?
 jr nc,equip_ready      ; 
equip_reverse:          ; No room?
 pop hl                 ; Fix stack
 ld a,(equip_this)      ; Undo everything!  Return new armor to list
 call Item_pickup       ; New armor is returned
 ld hl,game_loop_entry  ; Prepare for an artifical jump
 push hl                ; Place target address on stack
 jp no_room             ; Display "no room" ; Then artifical-jump to...
                        ; ...game_loop_entry
equip_ready:
de_equip_this = $ + 1
 ld a,$00               ; * Polymorph
 call Item_pickup       ; Place the old armor/weapon back in your item list
no_de_equip:
 pop hl                 ; Restore armor/weapon pointer
 ld a,(equip_this)
 ld (hl),a              ; Equip the item!
 jp Robot_option

;------------------------
;
; TRANSACTIONS
;
; IN:  HL = text directions
;      (must be $FF-terminated)
;      DE = cost amount
;
; OUT: NoCarry == accept
;        Carry == decline
;------------------------
Transaction_toll:
 push de                ; Save cost amount
 call TALKBOX
 pop hl                 ; Restore cost amount
 jp amount_window

;------------------------
;
; PURCHASING TRANSACTION
;
; Display a list of
; purchasable items
;
; Add the appropriate
; item to the stock
;
; Subtract the proper
; amont of GOLD using
; the "toll" method
;
; IN: IX -> Item stock
;     HL -> Shop Name
;------------------------
Transaction_purchase:
purchase:
 push hl
 push ix
 ld hl,(GOLD)           ; Save current GOLD in our general-purpose address...
 ld (_X),hl             ; ...in case there is an error purchasing the...
                        ; ...item and we need to do a refund
 call APD_CLEAR         ; Use APD buffer to hold the Shop's item costs
 pop hl
 ld de,APD_BUF
 ld bc,max_item_types
 ldir                   ; Copy prices to APD buffer
 ld ix,APD_BUF
 pop hl                 ; HL -> Name of Shop string
 ld a,$FF               ; A != 0 means "Disable item quantity displays"
 ld (item_no_qty),a     ;
 call ITEM              ; (OUT: C=item choice A=latest key)
 cp G_ALPHA
 ret z
 ld a,c                 ; Disallow "---" as a choice
 or a                   ;
 ret z                  ;
 push bc                ; Save the item number ###
 call Sure              ; "Sure?"
 pop bc                 ; Restore the item number ###
 push bc                ; Re-save
 ld b,0                 ; Clear upper byte
 ld hl,Item_costs - first_item - first_item  ; [ see library.asm]
 ld a,c
 cp last_melee + 1
 jr nc,get_costs
 ld hl,weapon_costs - 4
get_costs:
 add hl,bc
 add hl,bc              ; HL -> Cost of Item
 ROM_CALL($0033-$1A)    ; LD HL,(HL)    ; HL = Cost of Item
 call amount_window
 pop bc                 ; Restore item number   ; C Flag is not affected
 ret c                  ; Return if transaction failed
 ld a,c                 ; A = C = Item chosen
 call Item_pickup         ; Add the item ; In: A = item chosen
 ret nc                 ; Return if no error occoured
 ld hl,(_X)             ; If an error happened, refund player's money
 ld (GOLD),hl           ;
 ret


Sure:
 call TALKBOX_WINDOW
 ld hl,str_sure
 ld de,$0102
 set 3,(IY+$05)         ; Enable white text
 call DMX2
 res 3,(IY+$05)         ; Enable normal text
 ret


;------------------------
;
; SELLER'S TRANSACTION
;
Transaction_sell:
;
; Display list of the
; gamer's items.
; Allow the gamer to
; select one and
; exchange it for
; half it's retail value
;
;------------------------
 ld hl,str_sell_item    ; Write "sell an item" on this list...
 ld ix,ITEM1            ; Point to item data
 call ITEM              ; (OUT: C=item choice A=latest key IX=pointer to item)
 cp G_ALPHA
 jr z,sell_declined
 ld a,c
 ld (sell_this),a       ; * Polymorph [see below]
 ld (deduct_this),a     ; * Polymorph [see event025]
 or a
 jr z,sell_declined     ; Duh!  You can't sell 'blank space'!
 cp first_game_item
 jr nc,sell_declined    ; Can't sell Gloves, Keys, Fins, Pass, etc...

 call TALKBOX_WINDOW    ; Draw a dialog window
 ld hl,msg_sell
 ld de,$0101
 set 3,(IY+$05)         ; Enable white text
 call PARAGRAPH         ; Write "Sell for ____ gold?

 ld hl,Item_costs - first_item - first_item
sell_this = $ + 1
 ld c,$00               ; * Polymorph ; Load item to sell
 ld a,c
 cp last_melee
 jr nc,normal_sell
 ld hl,weapon_costs - 2 - 2
normal_sell:
 ld b,0
 add hl,bc              ; Double-add because the prices are 16-bit words
 add hl,bc              ; HL -> Cost of Item
 call ld_de_hlPTR       ; DE == Cost of Item
 srl d
 rr e
 ld h,d
 ld l,e
 ld de,$011C
 push hl                ; Save refund amount
 call DMD2
 res 3,(IY+$05)         ; Restore text properties
 call CR_GRBCopy
 call no_yes_menu

 pop hl                 ; Restore refund amount

 jr z,sell_declined
 ld a,c
 cp 1
 jr z,sell_declined
 ld d,h
 ld e,l
 ld hl,(GOLD)           ; Give the gamer back 1/2 the original value
 add hl,de              ;
 ld (GOLD),hl           ;
 xor a
 ret
sell_declined:
 scf
 ret


; ld hl,str_sell_item    ; Write "sell an item" on this list...
; ld ix,ITEM1            ; Point to item data
; call ITEM              ; (OUT: C=item choice A=latest key IX=pointer to item)
; cp G_ALPHA
; jr z,sell_declined
; ld a,c
; ld (sell_this),a
; call Sure
; call no_yes_menu
; jr z,sell_declined
; ld a,c
; cp 1
; jr z,sell_declined
;sell_this = $ + 1
; ld a,$00               ; * Polymorph ; A = Item serial #
; or a
; jr z,sell_declined     ; Can't sell "blank space"
; ld hl,Item_costs - first_item - first_item
; ld b,0
; add hl,bc
; add hl,bc
; ROM_CALL($0033-$1A)
; push hl
; pop de
; ld hl,(GOLD)           ; Give the gamer back some GOLD
; add hl,de              ;
; ld (GOLD),hl           ;
; scf
;sell_declined:
; ccf
; ret

no_yes_menu:
 set 3,(IY+$05)         ; Enable white text
 ld hl,msg_no_yes       ; "No/Yes" Menu
 ld de,$0752            ;
 ld a,2                 ;
 call MENU              ;
 res 3,(IY+$05)         ; Enable normal text
 cp G_ALPHA
 ret


amount_window:
 push hl                ; Save cost amount
 ld hl,GRAPH_MEM+276    ; Current GOLD window
 ld de,$0C08            ;
 call WINDOW            ;

 ld hl,(GOLD)           ; Current GOLD Display
 ld de,$184A            ;
 call DMD2              ;

 ld hl,str_fee          ; Print "fee:"
 ld de,$1802            ;
 call DMX2              ;

 pop hl                 ; Restore cost amount
 push hl                ; Save amount
 call DM_HL_DECI        ; Display cost amount

 call no_yes_menu       ; Out: A=keypress  C= 1:No  2:Yes
 jr z,transaction_fail
 ld a,c
 cp 1
 jr z,transaction_fail
 pop bc                 ; Restore cost amount
payment:
 ld hl,(GOLD)           ; Charge it, man!
 call CP_HL_BC          ;
 jr c,insufficient_funds;
 sbc hl,bc              ;
 ld (GOLD),hl           ;
 xor a                  ; Clear Carry Flag      ; <<< Transaction passed
 ret

insufficient_funds:
 ld hl,msg_not_enough_gold
 call TALKBOX
 push bc                ; Fix the stack
transaction_fail:
 pop bc                 ; Trash the cost amount
 scf                    ; Set Carry Flag        ; <<< Transaction failed
 ret


boss_start:
 ld hl,msg_boss
 call TALKBOX
 ld c,4
 ret

;################################################
;################################################
;################################################
;------------------------
event001:               ; << TOUCHED SPIKES/WATER WITHOUT PROTECTION
 ld a,library_fins
 call Item_check        ; Are you carrying fins?
 ret z
 call GRAPH_INVERT
 call CR_GRBCopy
 ld a,100
 call DELAY
 ld hl,msg_faint        ; Display "Gwaaah!..."
 call TALKBOX 
 jp faint               ; [see battlent.asm]
;------------------------
event002:               ; << YOU TOUCHED A WHITE BUTTON
gateswap:
 ld hl,mapbuffer
 ld bc,1023
event002_loop:
 ld a,(hl)
 cp 6*8                 ; Is this an 'off' button?
 jr z,button_swap0
 cp 7*8                 ; Is this an 'on' button?
 jr z,button_swap1
 cp 3*8                 ; Is this an 'open' gate?
 jr z,gate_swap0
 cp 14*8                ; Is this a 'closed' gate
 jr z,gate_swap1
event002_lable0:
 inc hl                 ; Check next tile
 dec bc                 ; Decrement loop counter
 ld a,b                 ; Compare:  BC = 0000?
 or c                   ;
 jr nz,event002_loop   ; Loop again if not zero
 jr event003
button_swap0:           ; Change an 'off' button to an 'on'
 ld (hl),7*8            ; 7 is the 'on' button (the dark one)
 jr event002_lable0
button_swap1:           ; Change an 'on' button to an 'off'
 ld (hl),6*8            ; 6 is the 'off' button (the white one)
 jr event002_lable0
gate_swap0:             ; Change an 'open' gate to 'closed'
 ld (hl),14*8           ; 14 is the 'closed' gate
 jr event002_lable0
gate_swap1:             ; Change a 'closed' gate to 'open'
 ld (hl),3*8            ; 3 is the 'open' gate
 jr event002_lable0
;------------------------
event003:               ; << YOU TOUCHED A DARK BUTTON
 ld a,(FRAME)
 add a,gate_duration
 or %00000001           ; Ensure a non-zero sum
 ld (auto_gateswap_time),a
 ret
;------------------------
event004:                ; << YOU GET A FULL HEAL
 ld a,(MAP_NUM)         ; Declare this map as a safe check point
 ld (SAFEHOUSE),a

 ld e,first_city
 cp 2 + 1
 jr c,city_found

 ld e,first_city + 1
 cp 8 
 jr c,city_found

 ld e,first_city + 2
 cp 10 + 1
 jr c,city_found

 ld e,first_city + 3
 cp 16 + 1
 jr c,city_found

 ld e,first_city + 4
city_found:
 ld a,e
 ld (CITY),a

 ld ix,HP
 ld b,max_team          ; B  = (3) Maximum number of bots to repair 
 ld de,21               ; DE = Size of robot data blocks
healall_loop:
 ld a,(ix+6)            ; Copy HPmax (high byte) -> HP (high byte)
 ld (ix),a              ;

 ld a,(ix+7)            ; Copy HPmax (low byte) -> HP (low byte)
 ld (ix+1),a            ;

 ld a,(ix+3)            ; Copy EPmax -> EP
 ld (ix+2),a            ;

 ld (ix+5),0            ; Clear any ailments or shields
 add ix,de              ; Next robot...
 djnz healall_loop

 ld hl,msg_full_heal    ; Print "Repaired!"
 jp TALKBOX             ; Do the talkbox ; "ret" to main loop
;------------------------
event005:                ; << YOU ARE TALKING TO MARCUS (beginning of game)
 ld hl,STORY0           ; Check Story Mode flags
 bit 0,(hl)             ; "You have finished the CC tournament"
 jr nz,_event005_01     ; Go to "sub event"
 ld hl,ICON_MARCUS
 ld de,GRAPH_MEM
 ld bc,msg_marcus
 call TALKBOX_PIC
 ld hl,msg_marcus2      ; "Aren't you late..."
 jp TALKBOX       ; Do the talkbox        ; return included
_event005_01:            ; << MARCUS
 ld hl,STORY0           ; Allow player to TRAVEL now...
 set 5,(hl)             ;
 ld hl,ICON_MARCUS
 ld de,GRAPH_MEM
 ld bc,msg_marcus3
 call TALKBOX_PIC
 ld hl,msg_marcus4
 call TALKBOX
 ld hl,msg_marcus5
 call TALKBOX
 jp event056            ; "Avoid water and spikes!"
; ret
;------------------------
event006:               ; << TARGET RANGES IN GENERAL
; IN:   A  = bonus type
; OUT:  HL = if you 'won' the game
;       HL = OP1 (a dummy address) if you didn't

 ld (secret_bonus_type),a
 ld de,target_game_cost ; Cost to play?
 ld hl,msg_gilbert      ; "Try a target game?"
 call Transaction_toll  ; Charge player $10
 ld hl,OP1              ; < Dummy address
 ret c                  ; Return if the transaction fails
 ld hl,msg_gilbert2     ;
 call TALKBOX
 call load_targets
 ld a,(turns)           ; Get number of turns taken
 ld hl,OP1              ; < Dummy address
 cp 1
 ret nc
event006_lable1:        ; << YOU WON A SECRET ATTACK
 ld hl,msg_targets_win  ; "You won a new attack!"
 call TALKBOX
 ld hl,msg_who_to_upgrade
 call TALKBOX
upgrade_robot:
 ld ix,HP
 ld hl,NAME0
 call ROBOTS            ; Out: C = Robot Chosen
 call Robot_lookup2     ; IN: C = Robot Chosen ; Out: IX = Robot address

 ld a,(ix+18)           ; Check robot's image byte
 or a                   ; Reject blank selections
 jr z,upgrade_robot     ;

 ld hl,GRAPH_MEM+72                     ; Robot name/status window
 ld de,$0C1A                            ; 
 call WINDOW                            ;

 ld b,3
 ld de,$0A08
attacks_loop:
 push bc
 ld b,(ix+12)
 call LIBRARY
 ld a,d
 add a,6
 ld d,a
 pop bc
 inc ix
 djnz attacks_loop
 ld hl,menu_null
 ld de,$0A08
 ld a,3
 call MENU

 cp G_ALPHA
 jr z,upgrade_robot
 ld b,c
upgrade_attacks:
 inc ix
 djnz upgrade_attacks
secret_bonus_type = $ + 1
 ld a,$00               ; * Polymorph [see above]
 ld (ix+12-4),a
 ld hl,STORY2           ; Allow events 58-61 to alter story flags themselves
 ret

load_targets:
 ld b,enemy_target
 ld c,3
 call Opponent_loader
 ld b,enemy_target
 ld c,4
 call Opponent_loader
 ld b,enemy_target
 ld c,5
 call Opponent_loader
 ld a,$FF
 ld (gain_nullify),a    ; Disallow GOLD as a reward
 call BATTLE
 ret
; ret
;------------------------
event007:               ; << TALKING TO KID AT THE TOURNAMENT
 ld hl,msg_tournament_kid ; "Hey, Edgar!  I heard your rival, REED was..."
 jp TALKBOX       ; Do the talkbox        ; return included
; ret
;------------------------
event008:               ; << TALKING TO OTHER KID AT THE TOURNAMENT
 ld hl,STORY0
 bit 0,(hl)
 jr nz,event010                 ; "Strange weather today"
event008_lable0:
 ld hl,msg_tournament_kid3      ; "This will be quite a show"
 jp TALKBOX
; ret
;------------------------
event009:               ; << TALKING TO TOURNAMENT REGISTRATION
 ld hl,STORY0           ; Check story progress
 bit 0,(hl)             ; Already won the tournament?
 jp nz,event020         ; If so jump to Basic Item shop
 ld hl,msg_tournament_registration
 call TALKBOX
 ld hl,msg_tournament_registration2
 ld de,tournament_entry_cost
 call Transaction_toll
 ret c                  ; Return if the transaction fails
 pop hl                 ; Fix the stack
 ld b,3
 ld c,5
 call Opponent_loader
 call BATTLE
 ld b,21                ; Load Sphere opponent
 ld c,3                 ; Send sphere to cell #3
 call Opponent_loader
 call BATTLE

 ld hl,ICON_REED
 ld de,GRAPH_MEM
 ld bc,msg_reed_threat
 call TALKBOX_PIC

 ld b,15
 ld c,4
 call Opponent_loader
 ld a,3                                 ; Survive for 3 turns!
 ld (turn_limit),a                      ;
 ld hl,STORY0                           ; Prepare to change a STORY flag
 set 3,(hl)                             ; Set "automatic rain" bit
 set 4,(hl)                             ; Set "allow rain" bit
 call BATTLE
 ld hl,STORY0
 res 3,(hl)                             ; Clear "automatic rain bit
 ld hl,msg_tournament_registration3
 call TALKBOX
 ld hl,msg_tournament_registration4
 call TALKBOX
 ld hl,(GOLD)
 ld bc,tournament_winnings
 add hl,bc
 ld (GOLD),hl
 ld hl,STORY0           ; Update STORE flag...
 set 0,(hl)             ; "You finished the CC tournament" flag
 jp LOAD_MAP
; ret                   ; return is included
;------------------------
event010:               ; << STRANGE WEATHER
 ld hl,msg_strange_weather
 jp TALKBOX
; ret
;------------------------
event011:               ; << YOU OPENED A DOOR
 ld de,(G_POS)          ; Check current grid position.  This is 16 bits
 ld hl,(CURRENT_MAP)    ; Prepare to read permanent map data
 ld bc,1024             ; Search range = maximum map size
 xor a                  ; Search for terminating zero in encoded map data
 cpir                   ;
 ld de,10
 add hl,de              ; Skip over map settings data
doorway_search
 ld a,(G_POSL)
 ld c,255               ; C = Search range
 cpir                   ; Search for matching door location byte
 ld a,(G_POS)           ; Verify BOTH bytes!
 cp (hl)                ; Verify now
 jr z,open_door
 jr doorway_search
open_door:
 inc hl
 inc hl                 ; Point to mapXXX location
 ld a,$FF                       ; Prepare to place Edgar on the invisible...
 ld (find_entry_point),a        ; ..."entry point" tile in the next map.
 ld a,(hl)
 ld (MAP_NUM),a
 pop hl                 ; Trash the "Underfoot" call    ; Fix the stack
 jp LOAD_MAP
;------------------------
event012:               ; << YOU CAN'T USE THE BUILDER YET
 pop hl                 ; Trash the BUILDER call
_event012_001:
 ld hl,msg_cant_builder
 jp TALKBOX
; ret
;------------------------
event013:               ; << "The end is near!"
 ld hl,msg_weather_danger
 jp TALKBOX
; ret
;------------------------
event014:               ; << "FINS let you tread water"
 ld hl,msg_fins_hint
 jp TALKBOX
; ret
;------------------------
event015:
 jp event006
; ld de,madison_targets_cost
; jp event006_lable0             ; Do a target game
;------------------------
event016:               ; << "Meet me at RANK beta"
 ld a,(RANK)
 cp 3
 ret nc
 ld hl,msg_rank_beta
 call TALKBOX
 scf
 ret
; ret
;------------------------
event017:               ; << You picked up a key
 ld hl,G_AHEAD
 ROM_CALL($0033-$1A)
 ld (hl),0              ; Erase the key tile
 ld a,lib_key           ; A = Key [see Library.asm and robot.h]
 jp Item_pickup         ; Add item to stock
; ret
;------------------------
event018:               ; << "No one enters without..."
 ld a,pass
 call Item_check
 jr z,event018_000
 ld hl,msg_scram
 jp TALKBOX
event018_000:
 ld hl,msg_welcome_to_ess
 call TALKBOX
 ld a,13
 ld (MAP_NUM),a
 ld (find_entry_point),a
 pop hl
 jp LOAD_MAP
; ret
;------------------------
event019:               ; << HOW FINS WORK
 ld hl,msg_fins_hint
 jp TALKBOX
; ret
;------------------------
event020:               ; << BASIX SHOP
 ld hl,msg_what_to_purchase
 call TALKBOX
 ld ix,basix_shop
 ld hl,str_supply_shop
 jp Transaction_purchase
; ret
;------------------------
event021:               ; << "BETH: Can I join your quest?"
 ld hl,STORY1
 bit 7,(hl)                     ; Is Beth on your team already (no cheating!)
 jr nz,event021_001
 ld hl,ICON_BETH
 ld de,GRAPH_MEM
 ld bc,msg_beth_intro2
 call TALKBOX_PIC
 call no_yes_menu
 ret z                          ; Alpha cancels
 ld a,c
 cp 1
 ret z                          ; "No" cancels
 ld hl,ALLIES
 inc (hl)                       ; Add ally count
 ld hl,msg_builder_enabled
 call TALKBOX
 ld hl,STORY1
 set 7,(hl)                     ; Set "Beth is on your team now" bit
 pop hl                         ; Fix the stack
 jp BUILDER
event021_001:
 ld hl,ICON_BETH
 ld de,GRAPH_MEM
 ld bc,msg_good_team
 jp TALKBOX_PIC

; ret
;------------------------
event022:               ; << "I predict you will meet..."
 ld hl,msg_prediction
 jp TALKBOX
; ret
;------------------------
event023:               ; << ADVANCED ITEM SHOP
 ld hl,msg_what_to_purchase
 call TALKBOX
 ld ix,advanced_shop
 ld hl,str_supply_shop
 jp Transaction_purchase
; ret
;------------------------
event024:               ; << Mini-game for STASIS
; ld hl,msg_gilbert      ; "Hi, Edgar..."
; ld de,stasis_game_cost
; call Transaction_toll  ; Charge player $10
; ret c                  ; Return if the transaction fails
; ld hl,msg_gilbert2     ;
; call TALKBOX
; ld b,23                ; Load 3 Targets as opponents...
; ld c,3                 ;
; call Opponent_loader   ;
; ld b,23                ;
; ld c,4                 ;
; call Opponent_loader   ;
; ld b,23                ;
; ld c,5                 ;
; call Opponent_loader   ;
; ld a,$FF
; ld (gain_nullify),a    ; Disallow GOLD and ALLOY as rewards
; call BATTLE
; jp LOAD_MAP
; ret 
;------------------------
event025:               ; << SELL YOUR ITEMS HERE
 call Transaction_sell  ; Let the player sell something
deduct_this = $ + 1
 ld c,$00               ; * Polymorph ; From Transaction_sell [see above]
 call nc,Item_deduct4   ; If it didn't decline, deduct the item
 pop hl                 ; Fix the stack
 jp LOAD_MAP
; ret
;------------------------
event026:               ; << BREAKING OUT OF JAIL
 ld hl,ICON_EDMUND
 ld de,GRAPH_MEM
 ld bc,msg_quiet_prisoner
 call TALKBOX_PIC
 ld hl,msg_evil1
 call TALKBOX
 call gateswap
 call render_map
 ld hl,msg_quiet_prisoner2
 call TALKBOX
 ld b,3
 ld c,3
 call Opponent_loader
 ld b,3
 ld c,4
 call Opponent_loader
 ld b,3
 ld c,5
 call Opponent_loader
 call BATTLE
 ld hl,STORY0
 set 5,(hl)                     ; "You are allowed to travel again"
 set 7,(hl)                     ; "You have escape Edmund's prison"
 ld a,7                         ; Jump to GATE Colony map
 ld (MAP_NUM),a
 ld (find_entry_point),a
 pop hl                         ; Pop the click_event call
 jp LOAD_MAP
;------------------------
event027:               ; << LOCKED BOX REMOVAL
 ld a,library_key               ; [see library.asm] ; Item number of "KEY"
 call item_takeaway             ; Check to see if player has a KEY
 ret nz                         ; Cancel if he doesn't have one
 ld hl,G_AHEAD                  ; Get the block's address
 ROM_CALL($0033-$1A)            ;
 ld (hl),0                      ; Remove the block
 ret
;------------------------
event028:               ; << BOSS: LIEUTENANT ASH
 ld hl,STORY1
 bit 6,(hl)             ; Beat MADISON boss before?
 ret nz                 ; Disallow return visits to the boss
 ld a,1
 ld (BOSS),a            ; "This battle will upgrade your rank"
event028_000:
 call boss_start        ; Display text and set C = 4
 ld b,enemy_ash
 call Opponent_loader
 pop hl                 ; Fix the stack (Trash call)
 call BATTLE_000
 ld hl,STORY1
 set 0,(hl)             ; "Enemies can appear in large numbers" now
 set 6,(hl)             ; Defeated Lt. Ash
 call final_level_check
 jr nz,event028_001
 set 3,(hl)             ; "You beat mini-boss #1 in the final level
event028_001:
 jp LOAD_MAP
final_level_check:
 ld hl,STORY1
 ld a,(MAP_NUM)
 cp 21
 ret

;------------------------
event029:               ; << ALARM BUTTON
 ld a,library_key               ; A = Key ### [see library.asm]
 call Drop_all                  ; Drop all Keys
 ld hl,msg_alarm
 call TALKBOX             ; Show: "Touched an alarm!"
 ld b,enemy_titan
 ld c,4
 call Opponent_loader
 pop hl                         ; Trash the do_tile call
 call BATTLE
 jp LOAD_MAP
;------------------------
event030:               ; << INN FUNCTION
 ld hl,msg_inn                  ; "Need a repair?"
 ld de,inn_cost
 call Transaction_toll          ; Charge player $10
 ret c
 jp event004
;------------------------
event031:               ; << ARMORY SHOP
 ld hl,msg_what_to_purchase
 call TALKBOX
 ld ix,armory
 ld hl,str_armory_shop
 jp Transaction_purchase
;------------------------
event032:               ; << GET THE GLOVES
 ld hl,msg_base
 call TALKBOX
 call no_yes_menu
 ret z
 ld a,c
 cp 1
 ret z
 ld a,library_repair_dx
 call item_takeaway
 ret nz                 ; Didn't have Repair DX after all?
 ld a,gloves
 call Item_pickup
 ld hl,msg_thanks
 call TALKBOX
 ret 
;------------------------
event033:               ; << WEAPON SHOP
 ld hl,msg_what_to_purchase
 call TALKBOX
 ld ix,weapons
 ld hl,str_weapon_shop
 jp Transaction_purchase
;------------------------
event034:               ; << ENTERING A CITY
 ld hl,msg_enter_this   ; Display: "Enter this city?"
event034_000:
 call TALKBOX
 call no_yes_menu
 ret z                  ; Cancel on ALPHA
 ld a,c
 cp 1
 ret z                  ; Cancel if "NO"
 ld a,(Y_POS)
 srl a
 srl a
 srl a
 ld (ROW_MAP),a
 ld a,(X_POS)
 srl a
 srl a
 srl a
 ld (COL_MAP),a
 jp event011
;------------------------
event035:               ; << GABBIN' ABOUT GLOVES
 ld hl,msg_gloves        
 jp TALKBOX
;------------------------
event036:               ; << TOUCHED A FENCE
 ld a,gloves            ; [see library.asm]
 call Item_check        ; "Are you carrying gloves?"
 pop hl                 ; Trash the underfoot call ; Fix the stack
 jp z,game_loop         ; If so, act normal
 ld a,(Y_POS)
 and %00000111
 jp nz,game_loop    
 ld hl,msg_razor_wire
 call TALKBOX
 jp Trywalk_down       ; Walk down automatically (without a keypress)
;------------------------
event037:               ; << ENTERING EDMUND'S BASE
 ld hl,msg_enter_base
 jr event034_000
;------------------------
event038:               ; << BOSS: Lieutennant ION and MIRAGE
 ld hl,STORY2
 bit 5,(hl)             ; Beat LAKESHORE boss before?
 ret nz
 ld a,1
 ld (BOSS),a                    ; "This battle will upgrade your rank"
event038_000:
 call boss_start        ; Display text and set C = 4
 ld b,boss_ion
 call Opponent_loader
 ld b,boss_mirage
 ld c,5
 call Opponent_loader
 call BATTLE_000
 call final_level_check
 jr nz,event038_001
 set 4,(hl)             ; "You defeated mini-boss #2 in the final level"
event038_001:
 ld hl,STORY2
 set 5,(hl)             ; Set "You beat the LAKESHORE bosses" flag
 pop hl                 ; Fix the stack
 jp LOAD_MAP
;------------------------
event039:               ; << BOSS: Lieutennant ACE
 ld hl,STORY2
 bit 0,(hl)             ; Beat DESOTO boss before?
 ret nz                 ; Can't fight boss twice
 ld a,1
 ld (BOSS),a                    ; "This battle will upgrade your rank"
event039_000:
 call boss_start        ; Display text and set C = 4
 ld b,boss_ace
 call Opponent_loader
 call BATTLE_000
 call final_level_check
 jr nz,event039_001
 set 5,(hl)             ; "You defeated mini-boss #2 in the final level"
event039_001:
 ld hl,STORY2
 set 0,(hl)             ; Set "You beat the DESOTO boss" flag
 pop hl                 ; Fix the stack
 jp LOAD_MAP
;------------------------
event040:               ; << BATTLE FOR THE GLOVES!
 ld hl,msg_earn_gloves
 jp TALKBOX
;------------------------
event041:               ; << EARN THE FINS
 call event016          ; Check for stats >= epsilon
 ret c
 ld hl,msg_earn_fins
 call TALKBOX
 ld hl,msg_favor
 call TALKBOX
 ld b,torpid
 ld c,4
 call Opponent_loader
 call BATTLE
 ld a,fins
 call Item_pickup
 pop hl
 jp LOAD_MAP
;------------------------
event042:               ; << TOUCHED SOME SPIKES
; ld a,fins             ; [see library.asm]
; call Item_check        ; "Are you carrying BOOTS?" (Z=yes  NZ=no)
; jp z,game_loop         
; jp event001            ; "If not, you automatically loose"
;------------------------
event043:               ; << ESS Corp Heal
 ld hl,msg_welcome_to_ess
 call TALKBOX
 ld hl,msg_inn
 ld de,inn_cost
 call Transaction_toll
 ret c
 jp gateswap            ; Open the gates to the repair room
;------------------------
event044:               ; << Appointment with REED
 ld hl,STORY2           ; Check player's game progress
 bit 6,(hl)             ; "Already defeated REED on rooftop?"
 jp nz,event020         ; "If so, go to a basic item shop"
 ld hl,msg_welcome_to_ess
 call TALKBOX
 ld hl,msg_appointment
 call TALKBOX
 ld hl,msg_ready
 call TALKBOX
 call no_yes_menu
 jp z,map_return        ; Cancel on ALPHA
 ld a,c
 cp 1
 jp z,map_return        ; Cancel on "NO"
 call FADE
 ld a,14
 ld (MAP_NUM),a
 ld (find_entry_point),a        ; Put edgar on the entry tile
map_return:
 pop hl                 ; Fix stack
 jp LOAD_MAP
;------------------------
event045:               ; << ESS Corp Employee
 ld hl,msg_employee1
 call TALKBOX     ; "REED is testing his SCORP..."
 ld hl,msg_employee2
 jp TALKBOX       ; "But against who?"
;------------------------
event046:               ; << ROOFTOP BATTLE AGAINST REED
 ld hl,ICON_REED
 ld de,GRAPH_MEM
 ld bc,msg_reed
 call TALKBOX_PIC
 ld hl,msg_reed2
 call TALKBOX
 ld b,enemy_scorp2      ; [see data.asm]
 ld c,4
 call Opponent_loader
 ld a,1
 ld (BOSS),a            ; "This IS a boss battle"
 call BATTLE_000
 ld hl,ICON_REED
 ld de,GRAPH_MEM
 ld bc,msg_reed_join
 call TALKBOX_PIC
 ld hl,STORY2
 set 6,(hl)             ; Update player's game progress
 ld hl,ALLIES
 inc (hl)
 ld hl,msg_builder_enabled
 call TALKBOX
 ld a,13
 ld (MAP_NUM),a
 pop hl
 jp LOAD_MAP
;------------------------
event047:               ; << PRO SHOP
 ld hl,msg_what_to_purchase
 call TALKBOX
 ld ix,pro_shop
 ld hl,str_supply_shop
 jp Transaction_purchase
; ret
;------------------------
event048:               ; << MARIO WANTS A MUSHROOM
; ld hl,msg_elipse
; jp TALKBOX
;------------------------
event049:               ; << "...Afraid to go outside"
 ld hl,msg_afraid
 jp TALKBOX
;------------------------
event050:               ; << You battle Lurid
 ld hl,STORY2
 bit 7,(hl)             ; Beat lurid before?
 jr nz,event050_lable0  ; If so, don't battle Anti-Cronus again
 ld hl,msg_lurid
 call TALKBOX
 ld hl,msg_lurid2
 call TALKBOX
 ld b,enemy_anticronus  ; [see data.asm]
 ld c,5
 call Opponent_loader
 call BATTLE
 ld hl,STORY2
 set 7,(hl)
event050_lable0:
 call FADE
 ld a,21
 ld (MAP_NUM),a         ; Go to final base
 ld (find_entry_point),a
 pop hl
 jp LOAD_MAP
;------------------------
event051:               ; << You tried to travel from indoors
 ld hl,msg_go_outside
 call TALKBOX
 jp game_loop_entry
;------------------------
event052:               ; << Noah explains the darkstar project
 ld hl,ICON_NOAH
 ld de,GRAPH_MEM
 ld bc,msg_noah
 call TALKBOX_PIC
 ld hl,msg_noah2
 call TALKBOX
 ld hl,msg_noah3
 jp TALKBOX
;------------------------
event053:               ; << Final Boss
 ld a,(STORY1)
 and %00111000          ; Leave mini-boss flags remaining
 cp %00111000
 jr z,event053_000
 call gateswap
 ld hl,msg_edmund
 jp TALKBOX
event053_000:
 ld bc,msg_edmund2
 ld de,GRAPH_MEM
 ld hl,ICON_EDMUND
 call TALKBOX_PIC
 ld hl,msg_edmund3
 call TALKBOX
 ld b,14                ; Load EDMUND
 ld c,5
 call Opponent_loader
 call BATTLE

 ld hl,msg_evil1
 call TALKBOX
 ld b,17                ; Load EDMUND's final stage
 ld c,4
 call Opponent_loader
 call BATTLE

 call GRAPH_FILL
 ld bc,msg_final_fight
 ld de,GRAPH_MEM
 ld hl,ICON_EDMUND
 call TALKBOX_PIC

 call FADE

 ld hl,msg_final_fight2
 ld de,$0002
 call PARAGRAPH

 ld de,picture7                 ; End picture
 ld hl,GRAPH_MEM+564            ; Row 27
 call DecodeZCP                 ; Decode and display picture to screen

final_scroller_loop1:
 ld b,16                        ; Picture is 16 pixels high
 ld hl,GRAPH_MEM+755            ; Start at end of picture and move backwards
final_scroller_loop0:
 call RIGHT_ROTATE              ; Rotate the row by 1 pixel
 dec hl                         ; Move up one row...
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 dec hl
 push hl
 call GET_KEY
 pop hl
 cp G_NONE
 jr nz,the_end
 djnz final_scroller_loop0      ; Loop until all 16 row are rotated
 call CR_GRBCopy                ; Display the shifted picture
 ld a,ending_scroll_speed       ; <-- scrolling speed
 call DELAY                     ; Slow down
 jr final_scroller_loop1        ; Don't stop scrolling
the_end:
 pop hl                         ; Trash the call to game_click
 ld a,(RANK)                    ; Check ending rank
 cp 5                           ; Do you have ohmega?
 jp nz,Init_lable1              ; If not, restart the game

 ld hl,STORY1
 set 1,(hl)                     ; Set "you're an elite" flag

 jp SAVE2
;------------------------
event054:               ; << Get Ambushed
 ld hl,STORY0
 bit 7,(hl)             ; "Been ambushed before?"
 ret nz                 ; If so, do nothing
 res 5,(hl)             ; "Not allowed to travel!"
 ld hl,msg_ambushed
 call TALKBOX
 ld a,6
 ld (MAP_NUM),a
 ld (find_entry_point),a
 pop hl                 ; Trash the "underfoot" call    ; Fix the stack
 jp LOAD_MAP
;------------------------
event055:               ; EARN THE ESS CORP PASS
 call event016                  ; Check player's rank
 ret c
 ld a,pass
 call Item_pickup
 ld hl,msg_ess_corp_pass
 jp TALKBOX
;------------------------
event056:               ; AVOID THE SPIKES AND WATER!
 ld hl,msg_spikes_base
 jp TALKBOX
;------------------------
event057:               ; PRISONERS
 ld hl,msg_prisoners
 jp TALKBOX
;------------------------
event058:               ; CAPE CITY TARGET RANGE
 ld hl,STORY2
 bit 1,(hl)
 jp nz,event030                 ; Go to Inn function if this game was won
 ld a,22                        ; 22 = FORCE bonus
 call event006
 set 1,(hl)             ; HL always = STORY2
 jr secret_quest
;------------------------
event059:               ; MADISON TARGET RANGE
 ld hl,STORY2
 bit 2,(hl)
 jp nz,event030                 ; Go to Inn function if this game was won
 ld a,23                        ; 23 = TELEPORT bonus
 call event006
 set 2,(hl)             ; HL always = STORY2
 jr secret_quest
;------------------------
event060:               ; DESOTO TARGET RANGE
 ld hl,STORY2
 bit 3,(hl)
 jp nz,event030                 ; Go to Inn function if this game was won
 ld a,24                        ; 24 = STASIS bonus
 call event006
 set 3,(hl)             ; HL always = STORY2
 jr secret_quest
;------------------------
event061:               ; LAKESHORE TARGET RANGE
 ld hl,STORY2
 bit 4,(hl)
 jp nz,event030                 ; Go to Inn function if this game was won
 ld a,25                        ; 25 = FLUX bonus
 call event006
 set 4,(hl)             ; HL always = STORY2
secret_quest:
 ld a,(hl)
 and %00011110          ; Mask all but target game-related flags
 cp %00011110           ; Mastered all target games?
 pop hl                 ; Trash call to Game_click
 jp nz,LOAD_MAP
 ld hl,msg_rank_up
 call TALKBOX
 ld a,(RANK)
 inc a
 ld (RANK),a
 jp LOAD_MAP
;################################################
;################################################
;################################################
; EDGAR'S SPRITES (MASKED)
edgar_front:
.db %11100111    ;AND Mask
.db %11000011
.db %11000011
.db %11000011
.db %10000001
.db %10000001
.db %11000011
.db %11000011

.db %00011000    ; XOR Mask
.db %00111100
.db %00100100
.db %00111100
.db %01000010
.db %01100110
.db %00111100
.db %00111100

edgar_right_01:
;AND Mask
.db %11100111
.db %10000011
.db %11100011
.db %11000111
.db %11000111
.db %11000111
.db %11100111
.db %11100011
;XOR Mask
.db %00011000
.db %01111100
.db %00010100
.db %00111000
.db %00101000
.db %00111000
.db %00011000
.db %00011100


edgar_right_2367:
;AND Mask
.db %11100111
.db %10000011
.db %11100011
.db %11000111
.db %10000011
.db %11000111
.db %11100111
.db %11010111
;XOR Mask
.db %00011000
.db %01111100
.db %00010100
.db %00111000
.db %01110100
.db %00111000
.db %00011000
.db %00101000


edgar_right_45:
;AND Mask
.db %11100111
.db %10000011
.db %11100011
.db %11000111
.db %10001011
.db %10000111
.db %10000101
.db %10111011
;XOR Mask
.db %00011000
.db %01111100
.db %00010100
.db %00111000
.db %01110110
.db %10111000
.db %01111010
.db %01000100


edgar_rear:
;AND Mask
.db %11100111
.db %11000011
.db %11000011
.db %11000011
.db %10000001
.db %11000011
.db %11000011
.db %11000011
;XOR Mask
.db %00011000
.db %00111100
.db %00111100
.db %00111100
.db %01100110
.db %01100110
.db %00111100
.db %00111100


edgar_left_01:
;AND Mask
.db %11100111
.db %11000001
.db %11000111
.db %11100011
.db %11100011
.db %11100011
.db %11100111
.db %11000111
;XOR Mask
.db %00011000
.db %00111110
.db %00101000
.db %00011100
.db %00010100
.db %00011100
.db %00011000
.db %00111000


edgar_left_2367:
;AND Mask
.db %11100111
.db %11000001
.db %11000111
.db %11100011
.db %11000011
.db %11100011
.db %11100111
.db %11101011
;XOR Mask
.db %00011000
.db %00111110
.db %00101000
.db %00011100
.db %00101110
.db %00011100
.db %00011000
.db %00010100

edgar_left_45:
;AND Mask
.db %11100111
.db %11000001
.db %11000111
.db %11100011
.db %11010001
.db %11100001
.db %10100001
.db %11011101
;XOR Mask
.db %00011000
.db %00111110
.db %00101000
.db %00011100
.db %01101110
.db %00011101
.db %01011110
.db %00100010


edgar_animtable:
.dw edgar_right_01      ; direction = 0
.dw edgar_right_2367
.dw edgar_right_2367
.dw edgar_right_45
.dw edgar_right_45
.dw edgar_right_2367
.dw edgar_right_2367
.dw edgar_right_01
.dw edgar_left_01       ; directon = 8
.dw edgar_left_2367
.dw edgar_left_2367
.dw edgar_left_45
.dw edgar_left_45
.dw edgar_left_2367
.dw edgar_left_2367     
.dw edgar_left_01
.dw edgar_front         ; direction = 16
.dw edgar_rear          ; direction = 17

short_edgar:            ; <-- Use these graphics on the overworld
;AND Mask
 .db $FF,$FF,$E7,$E3,$E7,$E7,$FF,$FF
;XOR Mask
 .db $00,$00,$18,$1C,$18,$18,$00,$00
;AND Mask
 .db $FF,$FF,$E7,$E3,$E7,$DB,$FF,$FF
;XOR Mask
 .db $00,$00,$18,$1C,$18,$24,$00,$00


;################################################
;################################################
;################################################
CURRENT_TILESET:
.db 0                   ; <-- Always indicates the current graphic_set

find_entry_point:
.db 0

graphic_set:
;Tile data - Graphics Studio
tileData:
 ;Tile 0                                BLANK SQUARE
 .db $00,$00,$00,$00,$00,$00,$00,$00
 ;Tile 1				DOOR
 .db $3C,$42,$42,$42,$72,$42,$42,$C3
 ;Tile 2				WATER
water_tile:
 .db $00,$44,$BB,$00,$00,$22,$DD,$00
 ;Tile 3				OPEN GATE
 .db $AA,$01,$80,$01,$80,$01,$80,$55
 ;Tile 4				ENTRY POINT
 .db $AA,$55,$AA,$55,$AA,$55,$AA,$55
 ;Tile 5                                EVENT ACTIVATOR
 .db $00,$00,$00,$00,$00,$00,$00,$00
 ;Tile 6                                ON
 .db $00,$00,$18,$24,$24,$18,$00,$00
 ;Tile 7				OFF
 .db $00,$00,$18,$3C,$3C,$18,$00,$00
 ;Tile 8                                MALE CIVILIAN
 .db $18,$24,$24,$3C,$5A,$66,$3C,$3C
 ;Tile 9                                {KEY}
 .db $00,$00,$0E,$FA,$AE,$00,$00,$00
 ;Tile 10                               CHAIR
 .db $00,$00,$00,$04,$04,$1C,$24,$24
 ;Tile 11				SIGN
 .db $FF,$4B,$F9,$99,$89,$F9,$01,$39
 ;Tile 12                               FEMALE CIVILIAN
 .db $18,$3C,$66,$FF,$24,$18,$18,$18
 ;Tile 13				POTTED PLANT
 .db $28,$54,$BA,$54,$10,$38,$28,$38
 ;Tile 14				CLOSED GATE
 .db $FF,$A5,$FF,$A5,$A5,$FF,$A5,$FF
 ;Tile 15				LOCKED BLOCK
 .db $FF,$AA,$FF,$F1,$05,$51,$FF,$FF

temporary_set:
gsxxx:
.ds 128

gs001:
indoor_set:
 ;Tile 16                               CLOUD (for rooftop)
 .db $00,$00,$00,$00,$00,$08,$55,$AA
 ;Tile 17                               ALARM BRICK
 .db $00,$00,$3C,$24,$24,$3C,$00,$00
 ;Tile 18                               REPAIR TILE (not active)
 .db $C3,$99,$24,$00,$00,$24,$99,$C3
 ;Tile 19                               REPAIR TILE
 .db $C3,$81,$24,$42,$42,$24,$81,$C3
 ;Tile 20                               MUSHROOM [originally by Bill Nagel]
 .db $3C,$66,$D3,$B9,$FF,$6A,$42,$3C
 ;Tile 21                               SPIKES
 .db $04,$04,$04,$4A,$4A,$40,$A0,$A0
 ;Tile 22                               MARIO'S HAT [originally by Bill Nagel]
 .db $00,$00,$00,$00,$00,$00,$18,$3E
 ;Tile 23                               GHOST BRICK
 .db $18,$24,$4A,$85,$AF,$5E,$2C,$18
 ;Tile 24                               STONE BRICK
 .db $FF,$81,$B3,$A3,$83,$83,$BF,$FF
 ;Tile 25                               TABLE TOP
 .db $00,$FF,$00,$00,$00,$FF,$AA,$55
 ;Tile 26				GLASS BARRIER
 .db $18,$24,$4A,$85,$AF,$5E,$2C,$18
 ;Tile 27                               MARIO [originally by Bill Nagel]
 .db $64,$D0,$C6,$3C,$7E,$95,$7E,$E7
 ;Tile 28				COMPUTER
 .db $7E,$42,$42,$42,$7E,$18,$24,$7E
 ;Tile 29                               IRON BRICK
 .db $FF,$81,$AB,$83,$AB,$83,$BF,$FF
 ;Tile 30				BOSS
 .db $18,$DB,$FF,$BD,$99,$24,$66,$C3
 ;Tile 31				INVISIBLE WALL
 .db $00,$00,$00,$00,$00,$00,$00,$00

gs002:
outdoor_set:
 ;Tile 32				RIGHT WALL
 .db $80,$88,$AA,$80,$80,$A0,$A8,$80
 ;Tile 33                               GRASS
 .db $00,$08,$2A,$00,$00,$20,$A8,$00
 ;Tile 34                               EVERGREEN TREETOP
 .db $28,$54,$AA,$54,$BA,$54,$BA,$54
 ;Tile 35                               EVERGREEN TREETOP 2
 .db $14,$2A,$55,$2A,$55,$2A,$5D,$2A
 ;Tile 36                               RAZORED FENCE
 .db $80,$FF,$AA,$D5,$AA,$D5,$AA,$D5
 ;Tile 37				BENCH
 .db $00,$00,$00,$FF,$81,$FF,$42,$00
 ;Tile 38                               LEFT WALL
 .db $01,$05,$15,$01,$01,$11,$55,$01
 ;Tile 39				SIDEWALK
 .db $00,$FF,$01,$2B,$55,$2B,$55,$FF
 ;Tile 40                               AIR CONDITIONER
 .db $80,$F8,$A8,$F8,$88,$88,$FC,$80
 ;Tile 41				SHOP SIGN
 .db $24,$7E,$66,$6E,$76,$66,$7E,$FF
 ;Tile 42                               PRACTICE TARGET
 .db $38,$44,$BA,$AA,$BA,$44,$BA,$82
 ;Tile 43                               MALE (LYING DOWN)
 .db $00,$00,$30,$EE,$D9,$D9,$EE,$30
 ;Tile 44				TREETRUNK
 .db $10,$14,$35,$10,$18,$10,$B8,$1C
 ;Tile 45				ROOFTOP
 .db $FF,$00,$4D,$33,$4D,$33,$4D,$FF
 ;Tile 46				BRICK WALL
 .db $44,$FF,$11,$FF,$44,$FF,$11,$FF
 ;Tile 47                               WINDOW
 .db $00,$3C,$34,$2C,$24,$3C,$00,$FF

gs003:
outdoor2_set:
 ;Tile 48                               SAND
 .db $00,$02,$10,$00,$00,$42,$10,$00
 ;Tile 49                               CITY'S ENTRANCE
 .db $00,$00,$6C,$44,$00,$44,$6C,$00
 ;Tile 50                               ENTRY POINT BIB
 .db $00,$08,$2A,$00,$00,$20,$A8,$00
 ;Tile 51                               RIGHT SHORE LINE
 .db $00,$04,$19,$00,$00,$10,$64,$00
 ;Tile 52                               THE WHITE HOUSE
 .db $00,$10,$10,$28,$C6,$82,$AA,$FE
 ;Tile 53                               BASE ENTRANCE
 .db $00,$00,$6C,$44,$10,$44,$6C,$00
 ;Tile 54                               RIGHT SHORE LINE 2
 .db $00,$02,$4D,$00,$08,$00,$04,$00
 ;Tile 55                               LEFT SHORE LINE
 .db $00,$40,$B1,$00,$04,$40,$00,$10
 ;Tile 56                               PALM TREE
 .db $00,$6C,$92,$38,$54,$10,$10,$38
 ;Tile 57                               FOREST
 .db $44,$EA,$11,$3F,$54,$88,$FA,$23
 ;Tile 58                               HILLS
 .db $10,$38,$4C,$A6,$00,$04,$4E,$F3
 ;Tile 59                               RESIDENTIAL
 .db $00,$08,$3C,$42,$81,$52,$56,$7E
 ;Tile 61                               COMMERCIAL
 .db $00,$E2,$A7,$E5,$BF,$E5,$AF,$FF
 ;Tile 62                               EDMUND'S FACTORY (LEFT)
 .db $00,$0A,$0A,$7F,$43,$D7,$D7,$FF
 ;Tile 63                               WATER TOWER
 .db $30,$48,$48,$30,$48,$78,$C8,$C8
 ;Tile 64 <left out>

map_load_table:
.dw map000      ; Starting map
.dw map001      ; CAPE CITY
.dw map002      ; World
.dw map003      ; MADISON
.dw map004      ; Madison Item Shop
.dw map005      ; Beth's House
.dw map006      ; Jail
.dw map007      ; Gate Colony
.dw map008      ; DESOTO
.dw map009      ; Big Item Shop (desoto)
.dw map010      ; Spikes & Diamonds colony
.dw map011      ; LAKESHORE
.dw map012      ; Lakeshore Advanced Shop
.dw map013      ; Ess Corp Sales
.dw map014      ; Ess Corp Roof
.dw map015      ; Pro Shop
.dw map016      ; Spikes Colony
.dw map017      ; SABLE
.dw map018      ; Shelter
.dw map019      ; Sable One Stop Shop
.dw map020      ; Get the key here
.dw map021      ; Alcatraz Colony

;--------------------------------
;                               -
; MAPS                          -
;                               -
; Map Data order:               -
; 1st = byte = map_width        -
; 2nd = byte = map_height       -
; 3rd = byte = tileset for map  -
; 4th = word = fight background -
;     = word = event position   -
;     = word = event address    -
;     = word = event position   -
;     = word = event address    -
;     = 0000 = *End of events   -
;     = word = "attached" map   -
;--------------------------------
map000:                         ; EDGAR's HOUSE
; Compression = 32% ; Saved = 30 bytes
.db $E0,$B0,$98,$8C,$86,$83,$81,$C0,$E0,$B0,$98,$8C,$86,$83,$80,$E8,$29,$E8
.db $E0,$B0,$0A,$E0,$B0,$3F,$13,$E0,$B0,$12,$F0,$A6,$9C,$2B,$E0,$B0,$3F,$13
.db $E0,$B0,$13,$C0,$A8,$31,$E0,$B0,$8D,$2C,$E8,$E0,$B0,$0A,$E0,$B0,$98,$8C
.db $86,$83,$81,$C0,$E0,$B0,$98,$8C,$81,$83,$80,$00
.db 12          ; <- map width
.db 8           ; <- map height
.db 1           ; <- graphics set (indoors)
.dw picture6    ; <- Fight background: "Mechanical Archives"
.db 1           ; <- Connected map: CAPE CITY
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (very common)
.db 0           ; <- Enemy 2 (common)
.db 0           ; <- Enemy 3 (very rare)
.dw 63          ; At tile 62...
.dw event005    ;       ...Marcus stands here
.dw $0000       ; * End of event data

map001:                         ; CAPE CITY
; Compression = 25% ; Saved = 63 bytes
.db $CC,$3F,$12,$CC,$B8,$9C,$88,$C4,$E3,$C0,$27,$C8,$A2,$93,$3B,$F0,$3F,$12
.db $F0,$A2,$92,$88,$C7,$82,$A0,$27,$F0,$A4,$9C,$88,$C4,$E0,$1E,$CC,$A6,$91
.db $88,$C5,$C3,$D1,$E8,$F4,$BA,$90,$88,$C4,$A2,$B1,$88,$F0,$A4,$91,$8E,$84
.db $C2,$91,$90,$CC,$A4,$91,$8E,$84,$C2,$91,$E0,$C4,$90,$88,$88,$C7,$83,$C1
.db $88,$C4,$AC,$9D,$8E,$C7,$A3,$D1,$80,$CC,$A6,$9C,$88,$C4,$C3,$C1,$88,$C8
.db $B8,$91,$8E,$87,$83,$C1,$90,$C4,$B8,$92,$33,$CC,$A2,$91,$8B,$87,$E0,$91
.db $F8,$FC,$B0,$9C,$8E,$84,$E2,$91,$E0,$C8,$A2,$9C,$89,$C4,$A2,$A1,$88,$C4
.db $B8,$92,$88,$C7,$82,$B0,$1F,$CC,$B8,$24,$C0,$27,$F0,$A2,$92,$8E,$84,$A2
.db $A1,$E0,$C4,$B8,$91,$88,$C4,$C3,$C1,$90,$C8,$B8,$91,$8A,$C2,$82,$91,$E0
.db $3F,$19,$CC,$A2,$9C,$89,$84,$A3,$C1,$88,$E8,$90,$9A,$88,$C7,$82,$A1,$E0
.db $F0,$3F,$3F,$0B,$F0,$A2,$91,$8E,$3F,$83,$C0,$3F,$00
.db 32          ; map width
.db 8           ; map height
.db 2           ; graphics set
.dw picture1    ; Background: "The Space Coast"
.db 0           ; <- Connected map: none!
.db 1           ; <- Enemy Difficulty Level
.db 2           ; <- Enemy 1 "Hunter"
.db 2           ; <- Enemy 2 "Hunter"
.db 11          ; <- Enemy 3 "Spy"
.dw 211
.dw event058    ; < Cape City Target Range (secret prize: "FORCE")
.dw 93
.dw event008    ; < Tournament KID #1
.dw 92
.dw event007    ; < Tournament KID #2
.dw 189
.dw event009    ; < Tournament REGISTRATION
.dw 133
.dw $0000       ; < Doorway to map000
.dw $0000       ; * End of event data

map002:                         ; WORLD MAP
; Compression = 66% ; Saved = 674 bytes!!!!
.db $3F,$19,$E8,$31,$F0,$B6,$9C,$8D,$C7,$A3,$E1,$98,$88,$AE,$3F,$3F,$3F,$83
.db $C1,$88,$F0,$AA,$92,$81,$85,$E0,$3F,$2F,$E8,$29,$E8,$21,$EC,$A4,$90,$89
.db $80,$C2,$F1,$D8,$3F,$3F,$3F,$1B,$CC,$84,$97,$8A,$1C,$E4,$39,$E8,$31,$E8
.db $21,$E4,$29,$E4,$B6,$91,$89,$84,$C3,$90,$3F,$3F,$19,$E4,$29,$E4,$11,$C8
.db $A6,$99,$2B,$E8,$A0,$9A,$8D,$86,$C3,$A0,$3F,$E4,$29,$E4,$BA,$9E,$13,$E8
.db $29,$E4,$B4,$90,$8D,$87,$A3,$E1,$D0,$38,$E4,$29,$E4,$A6,$95,$82,$C1,$A3
.db $A0,$27,$E4,$09,$E8,$A0,$9A,$8D,$C6,$C3,$A1,$C0,$E8,$B4,$9A,$8C,$C6,$A3
.db $90,$27,$E8,$21,$E4,$29,$E8,$11,$E8,$A0,$90,$88,$C6,$C3,$90,$3F,$18,$E8
.db $21,$E4,$29,$E4,$19,$E4,$B4,$9A,$8D,$86,$A0,$3F,$17,$E4,$21,$E4,$29,$E4
.db $3F,$3F,$1B,$E4,$21,$E4,$29,$E4,$3F,$3F,$1B,$E4,$29,$E4,$3F,$3F,$3F,$0C
.db $E4,$29,$E4,$3F,$3F,$3F,$0C,$E4,$31,$E8,$3F,$3F,$3F,$86,$A0,$36,$E4,$3F
.db $32,$E4,$3F,$0A,$E4,$39,$E8,$3F,$3F,$33,$E4,$39,$E8,$09,$E4,$3F,$0A,$E4
.db $29,$E8,$21,$E4,$39,$E8,$19,$E4,$3F,$3F,$13,$E4,$39,$E8,$29,$E4,$3F,$12
.db $E8,$39,$E4,$29,$E4,$3F,$3F,$3F,$14,$E8,$19,$E4,$3F,$3F,$2B,$E4,$21,$E4
.db $29,$E0,$3F,$3F,$3F,$86,$C0,$36,$E0,$21,$E0,$3F,$1A,$E8,$29,$E4,$3F,$96
.db $81,$80,$C2,$B1,$C0,$EC,$B0,$3F,$3B,$E4,$B6,$93,$8E,$86,$E3,$C0,$27,$D8
.db $84,$82,$89,$84,$A2,$A0,$37,$E8,$31,$E4,$BA,$9E,$8E,$86,$E3,$C0,$37,$D8
.db $84,$82,$89,$84,$E0,$3F,$2F,$C0,$AA,$9B,$88,$C4,$C0,$3F,$0F,$E0,$3F,$3F
.db $0B,$C8,$A0,$92,$3F,$24,$E0,$3F,$3F,$3F,$3F,$83,$90,$3F,$3F,$3F,$3F,$8C
.db $C0,$3F,$3F,$3F,$3F,$B2,$1A,$00
.db 32          ; map width
.db 32          ; map height
.db 3           ; graphics set
bg7_pointer:
.dw picture7    ; Background: "Lo-rez Mirage"
.db 0           ; <- Connected map: none!
.db 9           ; <- Enemy Difficulty Level
.db 2           ; <- Enemy 1 (very common)
.db 2           ; <- Enemy 2 (common)
.db 2           ; <- Enemy 3 (very rare)
.dw 291
.dw $1111       ; << Entering SABLE
.dw 52
.dw $0B0B       ; << Entering LAKESHORE
.dw 152
.dw $0303       ; << Entering MADISON
.dw 876
.dw $0808       ; << Entering DESOTO
.dw 825
.dw $0101       ; << Entering CAPE CITY
.dw 245
.dw $0707       ; << Edmund's MADISON BASE
.dw 874
.dw $0A0A       ; << Edmund's DESOTO BASE
.dw 54
.dw $1010       ; << Edmund's LAKESHORE BASE
;.dw 321          
;.dw $1515       ; << Edmund's SABLE BASE (final level)
.dw $0000       ; * End of event data

map003:                         ; MADISON
; Compression = 47% ; Saved = 118 bytes
.db $3F,$3F,$1A,$D8,$BA,$9D,$8E,$C4,$82,$E1,$E8,$F4,$BA,$90,$33,$D8,$BE,$99
.db $8F,$C4,$82,$E1,$F8,$FC,$BE,$90,$34,$D8,$FC,$82,$9F,$8C,$85,$C3,$F1,$F8
.db $FC,$B0,$3F,$82,$84,$A1,$81,$90,$E8,$B4,$9A,$3B,$D8,$BA,$9D,$8E,$C4,$83
.db $C0,$E0,$3F,$09,$D8,$BA,$9D,$8E,$C4,$2D,$C8,$12,$C0,$10,$D8,$BE,$9F,$8F
.db $C6,$2D,$F0,$B6,$0A,$F4,$BA,$9D,$88,$84,$C1,$3F,$17,$F4,$BA,$9D,$88,$87
.db $3F,$1E,$FC,$82,$9F,$8C,$84,$C0,$3F,$2F,$C8,$B8,$3F,$2B,$C8,$B8,$2A,$C8
.db $AC,$9D,$8E,$C7,$A2,$81,$90,$10,$F0,$31,$F0,$AC,$9D,$8E,$C7,$A2,$81,$E0
.db $3F,$11,$D8,$BE,$9F,$8F,$C4,$81,$3F,$3F,$08,$00
.db 16          ; map width
.db 16          ; map height
.db 2           ; graphics set
.dw picture2    ; fight background: "Mr. Gallagher goes to Washington"
.db 0           ; <- Connected map: none!
.db 12          ; <- Enemy Difficulty Level
.db 2           ; <- Enemy 1: Hunter
.db 18          ; <- Enemy 2: Spy
.db 4           ; <- Enemy 3: Spitfire
.dw 112
.dw event025    ; << SELL YOUR ITEMS HERE
.dw 89
.dw event059    ; << Madison Target Game (secret prize: "TELEPORT")
.dw 133
.dw event010    ; << "Odd weather today"
.dw 53
.dw $0404       ; << DOORWAY to Item Shop
.dw 161
.dw $0505       ; << DOORWAY to BETH's House
.dw 126
.dw event032    ; << HEX's CHALLENGE
.dw 71
.dw event033    ; << WEAPONS SHOP
.dw 239
.dw event031    ; << ARMORY SHOP
.dw $0000                       ; * End of event data

map004:                         ; Madison Item Shop
.db $3F,$3F,$3F,$3F,$3F,$3F,$81,$C0,$E0,$B0,$98,$8C,$3C,$E0,$98,$0B,$C0,$E0
.db $39,$E0,$B2,$99,$8C,$C6,$3D,$E0,$0B,$C0,$80,$E0,$39,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture2    ; fight background: "Mr. Gallagher goes to Washington"
.db 3           ; <- Connected map: MADISON
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (very common)
.db 0           ; <- Enemy 2 (common)
.db 0           ; <- Enemy 3 (very rare)
.dw 73
.dw event020    ; << Basic items shop
.dw 75
.dw event030    ; << Inn
.dw $0000


map005:                         ; NOAH'S HOUSE
; Compression = 45% ; Saved = 43 bytes
.db $3F,$3F,$3F,$0B,$E0,$B0,$98,$8C,$86,$83,$81,$C0,$E0,$B0,$98,$8C,$0C,$E0
.db $B8,$0A,$F0,$2A,$E8,$E0,$09,$E0,$0A,$E0,$A8,$31,$E0,$09,$E0,$22,$C0,$20
.db $E0,$09,$E0,$9A,$3B,$E8,$E0,$09,$E0,$B0,$1C,$C0,$17,$E0,$B0,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives (indoor)
.db 3           ; <- Connected map: MADISON
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (very common)
.db 0           ; <- Enemy 2 (common)
.db 0           ; <- Enemy 3 (very rare)
.dw 51
.dw event021    ; << BETH joins your quest
.dw 66
.dw event052    ; << NOAH explains the Darkstar project
.dw $0000


map006:                         ; Jail
; Compression = 29% ; Saved = 27 bytes
.db $E0,$B0,$98,$8C,$86,$83,$81,$C0,$E0,$B0,$98,$8C,$86,$81,$3F,$3F,$3F,$3F
.db $0B,$C0,$10,$E0,$B0,$8E,$8C,$86,$83,$80,$F0,$E0,$B0,$8E,$8C,$86,$83,$1E
.db $E0,$AC,$0A,$E0,$0B,$C0,$80,$E0,$B0,$0A,$D0,$09,$E0,$B6,$0A,$E0,$19,$E0
.db $B0,$98,$8C,$86,$83,$81,$C0,$E0,$B0,$98,$8C,$86,$83,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives (indoor)
.db 3           ; <- Connected map: MADISON
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (very common)
.db 0           ; <- Enemy 2 (common)
.db 0           ; <- Enemy 3 (very rare)
.dw 57
.dw event026    ; << "Quiet prisoner!"
.dw $0000

map007:                         ; GATE COLONY
; Compression = 26% ; Saved = 67 bytes
.db $F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA
.db $9D,$8E,$87,$C3,$C1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E
.db $C7,$A1,$90,$3F,$10,$C4,$1A,$F0,$09,$F8,$09,$F0,$10,$C4,$19,$C4,$12,$C8
.db $F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$83,$84,$A0,$26,$F4,$8C,$8F
.db $83,$87,$A0,$1E,$C4,$13,$E0,$0F,$F4,$BA,$3F,$8E,$C7,$A3,$D1,$E8,$B8,$BA
.db $9D,$8E,$C7,$A1,$F1,$E8,$F4,$BA,$9D,$87,$87,$A3,$D1,$E8,$F4,$BA,$9D,$8E
.db $C7,$A3,$D0,$80,$F4,$BA,$9D,$8E,$C0,$82,$90,$20,$F0,$3F,$1A,$F8,$30,$F4
.db $BA,$22,$F4,$0B,$E1,$88,$F4,$BA,$0A,$F4,$3F,$1A,$F4,$31,$F4,$BA,$1B,$C8
.db $F4,$A2,$2A,$F4,$21,$C4,$8C,$91,$23,$F4,$31,$F4,$BA,$9D,$8E,$C7,$A3,$D1
.db $E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4
.db $BA,$9D,$8E,$C7,$A0,$C0,$20,$C8,$F4,$80,$00
.db 32          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives" (indoor)
.db 3           ; <- Connected map: MADISON
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <-                    
.db 0           ; <- 
.db 0           ; <- 
.dw 18
.dw event028    ; << BOSS: Lieutenant ASH
.dw $0000




map008:                         ; DESOTO
; Compression = 28% ; Saved = 70 bytes
.db $FC,$BE,$9F,$8C,$84,$82,$E1,$E8,$F4,$BA,$90,$88,$C4,$83,$A0,$C0,$E8,$1A
.db $C0,$B4,$A0,$96,$8F,$C6,$A3,$F1,$80,$C4,$A0,$22,$D0,$A8,$94,$8A,$84,$81
.db $B1,$F8,$84,$BE,$98,$89,$85,$82,$C1,$A0,$D0,$A8,$3C,$C0,$0F,$F0,$90,$22
.db $DC,$AE,$97,$8B,$C5,$E2,$F1,$B8,$DC,$AE,$97,$8B,$C5,$E2,$F1,$B8,$DC,$AE
.db $3F,$3F,$3F,$3F,$81,$B8,$DC,$AE,$97,$8B,$C5,$E2,$F1,$B8,$DC,$AE,$97,$8B
.db $C5,$E3,$D1,$E8,$F4,$98,$33,$C0,$D4,$19,$D8,$BA,$9D,$8E,$C0,$25,$D8,$BA
.db $9D,$8E,$C7,$A2,$81,$90,$D8,$BE,$9F,$8F,$C5,$C3,$D1,$E8,$F4,$A0,$96,$8E
.db $C7,$A3,$D1,$E8,$C0,$B8,$96,$8F,$C7,$E3,$F1,$B0,$F4,$BA,$9D,$88,$85,$C3
.db $E1,$F0,$F8,$BC,$90,$2B,$D8,$BE,$9F,$8F,$C4,$82,$E1,$F0,$F8,$BC,$9E,$8C
.db $2C,$D8,$BE,$9F,$8F,$C6,$82,$90,$C0,$3F,$09,$C8,$1A,$E0,$A0,$A2,$88,$3F
.db $87,$81,$3F,$30,$C0,$00
.db 16          ; map width
.db 16          ; map height
.db 2           ; graphics set
.dw picture3    ; fight background: "The West Begins"
.db 0           ; <- Connected map: none
.db 22          ; <- Enemy Difficulty Level
.db hunter      ; <- Enemy 1
.db spy         ; <- Enemy 2
.db tomahawk    ; <- Enemy 3
.dw 39
.dw $0909       ; << DOORWAY to Item Shop
.dw 13
.dw event060    ; << DESOTO Target Game (secret prize: "STASIS")
.dw 135
.dw event025    ; << SELL YOUR ITEMS HERE
.dw 241
.dw event031    ; << ARMORY SHOP
.dw 255
.dw event033    ; << WEAPONS SHOP
.dw 231                
.dw event055    ; << Get ESS Corp Pass (need high rank)
.dw 59
.dw event035    ; << "You need gloves to clear that fence"
.dw 214
.dw event010    ; << "Odd weather today"
.dw 229
.dw event022    ; << "Bots kick ASCII"
.dw 228           
.dw event014    ; << "FINS let you tread water"
.dw 128
.dw event040    ; << Getting the GLOVES
.dw 18
.dw event041    ; << Getting FINS (need high rank)
.dw $0000

map009:                         ; Item Shop (DESOTO)
; Compression = 58% ; Saved = 55 bytes
.db $3F,$3F,$3F,$3F,$3F,$3F,$2E,$E0,$B0,$98,$8C,$86,$83,$81,$C0,$28,$E0,$98
.db $8A,$84,$0D,$C0,$E0,$29,$E0,$B2,$99,$8C,$C6,$A3,$91,$C0,$28,$E0,$13,$C0
.db $0F,$E0,$80,$00
.db 12		; map width
.db 8		; map height
.db 1		; graphics set
.dw picture6    ; fight background: "Lo-rez Mirage"
.db 8           ; <- Connected map: DESOTO
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (very common)
.db 0           ; <- Enemy 2 (common)
.db 0           ; <- Enemy 3 (very rare)
.dw 78
.dw event020    ; << Basic items shop
.dw 80
.dw event030    ; << Inn
.dw 82
.dw event023    ; << Advanced items shop
.dw $0000


map010:                         ; Diamonds Colony
; Compression = 31% ; Saved = 78 bytes
.db $E8,$B8,$9E,$8E,$86,$C3,$A1,$D0,$E8,$B4,$9A,$8D,$86,$C3,$A1,$D0,$E8,$0B
.db $C0,$80,$E8,$B4,$9A,$8D,$86,$C3,$A1,$D0,$E8,$B4,$9A,$8D,$86,$C3,$A1,$D0
.db $E8,$B4,$8F,$0B,$E8,$92,$91,$3F,$86,$C0,$E1,$D0,$3F,$29,$E8,$B4,$9A,$87
.db $C1,$C1,$E1,$88,$0A,$E0,$37,$E8,$09,$E8,$3F,$2A,$E8,$B4,$9A,$87,$C6,$C3
.db $A0,$0F,$E8,$31,$E8,$19,$E8,$3F,$0A,$E8,$11,$E8,$B4,$2A,$E8,$09,$E8,$21
.db $E8,$09,$DC,$09,$DC,$09,$E8,$39,$E8,$92,$9A,$0B,$E8,$B4,$22,$DC,$09,$C4
.db $09,$E8,$12,$F0,$18,$E8,$1A,$F0,$28,$E8,$92,$13,$F8,$E8,$B4,$2A,$E8,$A4
.db $9A,$13,$E8,$21,$E8,$21,$E8,$23,$E1,$D0,$A4,$B4,$0A,$E8,$B4,$9A,$8D,$86
.db $C3,$A1,$D0,$E8,$B4,$9A,$8D,$86,$C3,$A1,$D0,$E8,$B4,$9A,$8D,$86,$C3,$A1
.db $D0,$E8,$B4,$9A,$8D,$86,$C3,$A1,$D0,$E8,$B4,$9A,$8D,$86,$C0,$00
.db 32          ; map width
.db 8           ; map height
.db 1		; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 8           ; <- Connected map: DESOTO
.db 16          ; <- Enemy Difficulty Level
.db titan       ; <- Enemy 1 (very common)
.db titan       ; <- Enemy 2 (common)
.db tomahawk    ; <- Enemy 3 (very rare)
.dw 2  
.dw event039    ; << BOSS: Lieutennant ACE
.dw $0000

map011:                         ; LAKESHORE
; Compression = 28% ; Saved = 71 bytes
.db $18,$D8,$BE,$9F,$8F,$C4,$81,$81,$B0,$FC,$BE,$9F,$8F,$C4,$81,$1E,$D8,$BE
.db $99,$8F,$C4,$82,$A1,$B0,$FC,$BE,$9F,$8F,$C4,$25,$D8,$BE,$81,$8F,$C6,$83
.db $C1,$B0,$FC,$BE,$9F,$8F,$C4,$15,$D8,$BA,$9D,$8E,$C4,$17,$C1,$B0,$FC,$BE
.db $9F,$8F,$C5,$82,$C0,$80,$D8,$BA,$9D,$8E,$C5,$82,$C1,$A0,$D0,$A0,$88,$33
.db $D8,$BE,$9F,$8F,$C4,$08,$C0,$A1,$90,$C0,$32,$E0,$D8,$BE,$9F,$8F,$C6,$81
.db $C1,$A8,$F0,$A0,$3F,$1B,$D0,$A8,$94,$8A,$85,$82,$C1,$A0,$C0,$3F,$32,$D8
.db $BA,$9D,$8E,$C4,$3F,$1E,$D8,$BA,$9D,$8E,$C4,$3F,$1E,$D8,$BE,$9F,$8F,$C4
.db $82,$E1,$E8,$F4,$BA,$90,$8D,$84,$A3,$A0,$17,$D8,$BE,$9F,$8F,$C4,$82,$E1
.db $E8,$F4,$BA,$90,$88,$C2,$82,$91,$D0,$10,$D8,$BE,$9F,$8F,$C4,$82,$E1,$F8
.db $E4,$BE,$90,$33,$D8,$BE,$81,$8F,$C4,$81,$B1,$F8,$84,$BE,$98,$3C,$E8,$A0
.db $9A,$3F,$3F,$0C,$00
.db 16          ; map width
.db 16          ; map height
.db 2           ; graphics set
.dw picture4    ; fight background: "Vertical Village"
.db 0           ; <- Connected map: none
.db 27          ; <- Enemy Difficulty Level
.db sea_lance   ; <- Enemy 1
.db enemy_enfor ; <- Enemy 2
.db enemy_ispy  ; <- Enemy 3
.dw 226
.dw $0C0C       ; << DOORWAY to Advanced Shop
.dw 8
.dw event031    ; << ARMOR SHOP
.dw 15
.dw event033    ; << WEAPONS SHOP
.dw 171
.dw event040    ; << Battle for GLOVES
.dw 237
.dw event018    ; << Need a pass
.dw 37
.dw $0F0F       ; << DOORWAY to PRO SHOP
.dw 74
.dw event025    ; << ITEM Sell-back guy
.dw 198
.dw event061    ; << LAKESHORE target game (secret prize: "FLUX")
.dw 96
.dw event056    ; << "Avoid water and spikes"
.dw 102
.dw event041    ; << Getting FINS (need high rank)
.dw $0000

map012:                         ; ADVANCED SHOP
.db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$80,$E0,$B0,$98,$8C,$3F,$83,$80,$E0,$A8,$B0
.db $3F,$8C,$86,$A3,$91,$C0,$3F,$B0,$0C,$C1,$C0,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set (indoors)
.dw picture6    ; fight background: "Mechanical Archives" (indoor)
.db 11          ; <- Connected map: LAKESHORE
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (most common)
.db 0           ; <- Enemy 2 (less common)
.db 0           ; <- Enemy 3 (least common)
.dw 81
.dw event023    ; << ADVANCED ITEM SHOP
.dw $0000

map013:                         ; ESS Corp sales floor
; Compression = 30% ; Saved = 28 bytes
.db $F4,$BA,$9D,$8E,$C7,$A3,$81,$C0,$E0,$B0,$98,$8C,$86,$83,$D1,$E0,$CC,$8C
.db $9D,$8E,$82,$16,$E0,$A8,$B0,$9D,$1B,$F4,$B2,$99,$8C,$C6,$A3,$91,$C8,$E0
.db $BA,$8E,$87,$83,$C3,$D0,$E8,$11,$C0,$09,$E8,$E0,$B0,$3F,$13,$E0,$B0,$3F
.db $13,$E0,$B0,$3F,$13,$E0,$B0,$8D,$3D,$C0,$E8,$E0,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set (indoors)
.dw picture6    ; fight background: "Mechanical Archives" (indoor)
.db 11          ; <- Connected map: LAKESHORE
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (most common)
.db 0           ; <- Enemy 2 (less common)
.db 0           ; <- Enemy 3 (least common)
.dw 30
.dw event043    ; << ESS Corp repair
.dw 33
.dw event044    ; << Edgar's appointment with REED
.dw 44
.dw event045    ; << ESS Corp employee
.dw $0000

map014:                         ; Ess Corp Roof
; Compression = 40% ; Saved = 38 bytes
.db $3F,$3F,$3F,$0B,$C0,$A0,$12,$C0,$A0,$12,$C0,$A0,$1A,$C0,$19,$C0,$11,$C0
.db $19,$C0,$11,$C0,$19,$C0,$21,$FC,$BE,$9F,$8F,$C7,$E3,$F1,$F8,$FC,$BE,$9F
.db $0B,$F4,$88,$3F,$84,$87,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4
.db $BA,$9D,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set (indoors)
.dw picture4    ; fight background: "Vertical Village"
.db 0           ; <- Connected map: none
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (most common)
.db 0           ; <- Enemy 2 (less common)
.db 0           ; <- Enemy 3 (least common)
.dw 82
.dw event046    ; << Rooftop battle against REED
.dw $0000

map015:                         ; Lakeshore Item Shop (Basic 2)
.db $3F,$3F,$3F,$3F,$3F,$3F,$3E,$E0,$B0,$98,$8C,$86,$3D,$E0,$90,$8A,$8E,$86
.db $3D,$E0,$B2,$99,$8C,$C6,$3D,$E0,$0B,$C0,$80,$E0,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set (indoors)
.dw picture6    ; fight background: "Mechanical Archives" (indoor)
.db 11          ; <- Connected map: none
.db 0           ; <- Enemy Difficulty Level
.db 0           ; <- Enemy 1 (most common)
.db 0           ; <- Enemy 2 (less common)
.db 0           ; <- Enemy 3 (least common)
.dw 80
.dw event047    ; << PRO SHOP
.dw $0000

map016:                         ; Spikes Colony
; Compression = 19% ; Saved = 48 bytes
.db $D4,$AA,$95,$8A,$C5,$A2,$D0,$27,$D4,$AA,$95,$8A,$C5,$A2,$D1,$A8,$D4,$AA
.db $95,$8A,$C5,$A2,$D1,$A8,$D4,$AA,$95,$8A,$C5,$A2,$D1,$A8,$D4,$21,$D4,$AA
.db $1A,$D4,$19,$D4,$19,$D4,$0A,$F0,$98,$AA,$2A,$D4,$B8,$95,$8A,$C5,$A0,$0F
.db $F0,$B8,$19,$D4,$9C,$95,$0B,$D4,$09,$D4,$8C,$95,$1B,$D4,$19,$D4,$11,$D4
.db $BC,$0A,$F8,$AA,$95,$0B,$D4,$13,$C0,$80,$D4,$0B,$E1,$A8,$08,$D4,$09,$D4
.db $09,$D4,$21,$D4,$AA,$95,$13,$D4,$AA,$9C,$88,$C7,$82,$D1,$A8,$98,$AA,$0A
.db $D4,$8C,$0A,$D4,$09,$D4,$0A,$F0,$D4,$8C,$95,$0B,$D4,$9C,$95,$8A,$C0,$1D
.db $D4,$12,$F8,$98,$11,$D4,$AA,$83,$8A,$C5,$A0,$81,$A8,$D4,$11,$D4,$09,$D4
.db $11,$D4,$AA,$0A,$D4,$AA,$95,$8A,$C5,$A0,$0E,$D4,$12,$F0,$18,$D4,$AA,$2A
.db $D4,$8C,$1A,$D4,$3F,$1A,$D4,$09,$D4,$AA,$95,$1B,$D4,$AA,$95,$8A,$C5,$A2
.db $D0,$C8,$D4,$AA,$95,$8A,$C5,$A2,$D1,$A8,$D4,$AA,$95,$8A,$C5,$A2,$D1,$A8
.db $D4,$AA,$95,$2B,$D4,$AA,$95,$8A,$C0,$00
.db 32          ; map width
.db 8           ; map height
.db 1		; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 11          ; <- Connected map: LAKESHORE
.db 30          ; <- Enemy Difficulty Level
.db titan       ; <- Enemy 1 (very common)
.db titan       ; <- Enemy 2 (common)
.db sea_lance   ; <- Enemy 3 (very rare)
.dw 92
.dw event038    ; BOSS: ION and MIRAGE 
.dw 94
.dw event038    ; BOSS: ION and MIRAGE
.dw $0000

map017:                         ; SABLE
; Compression = 32% ; Saved = 71 bytes
.db $13,$C0,$A0,$90,$D8,$BC,$9E,$8F,$84,$08,$C0,$A0,$90,$29,$C0,$88,$84,$96
.db $8F,$87,$C3,$E1,$0A,$C0,$A0,$E0,$3B,$C2,$E1,$F0,$F8,$BC,$90,$81,$3F,$15
.db $D8,$BC,$81,$8F,$84,$3F,$21,$C0,$A0,$0A,$C0,$A0,$0F,$D8,$BA,$9D,$8E,$C7
.db $A3,$D1,$E8,$C0,$14,$C0,$09,$C1,$16,$D8,$BC,$81,$8F,$87,$A3,$D1,$E8,$D0
.db $A8,$94,$8A,$85,$82,$C1,$A0,$D0,$A8,$90,$1B,$F4,$BA,$9E,$8C,$1E,$C0,$17
.db $D8,$BA,$9D,$8E,$C7,$A3,$E1,$F0,$C0,$3F,$96,$8F,$87,$C3,$E1,$F0,$D8,$BA
.db $9D,$8E,$C4,$3F,$1E,$D8,$BA,$9D,$8E,$C4,$35,$D8,$BA,$9D,$8E,$C4,$82,$E1
.db $F0,$E4,$BC,$90,$33,$D8,$BA,$9D,$8E,$C4,$82,$E1,$F0,$84,$BC,$90,$33,$D8
.db $BC,$9E,$8F,$84,$17,$C0,$3F,$28,$00
.db 16          ; map width
.db 16          ; map height
.db 2           ; graphics set
.dw picture5    ; fight background: "Sable City"
.db 0           ; <- Connected map: none 
.db 35          ; <- Enemy Difficulty Level
.db enemy_redeye; <- Enemy 1
.db enemy_sphere; <- Enemy 2
.db spy         ; <- Enemy 3
.dw 94
.dw $1212       ; << DOORWAY to Shelter
.dw 194
.dw $1313       ; << DOORWAY to One Stop Shop
.dw 55
.dw $1414       ; << DOORWAY to Get Key Here
.dw 28
.dw event023    ; << Advance Shop
.dw 18
.dw event047    ; << Pro Shop
.dw 89
.dw event030    ; << Inn function
.dw $0000

map018:                         ; Shelter
; Compression = 45% ; Saved = 43 bytes
.db $3F,$3F,$3F,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA
.db $1B,$C0,$29,$C0,$F4,$BA,$33,$E0,$18,$F4,$BA,$88,$3F,$0C,$F4,$BA,$3F,$0C
.db $C0,$F4,$BA,$84,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3,$D0,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 17          ; <- Connected map: SABLE
.db 0           ; <- Enemy Difficulty Level (no enemies)
.db 0           ; <- Enemy 1
.db 0           ; <- Enemy 2
.db 0           ; <- Enemy 3
;.dw 50
;.dw event048    ; << Mario wants a mushroom
.dw 40
.dw event031    ; << ARMOR SHOP
.dw 82
.dw event033    ; << WEAPON SHOP
.dw 55
.dw event025    ; << SELL BACK SHOP
.dw 46
.dw event013    ; << "The end is near!"
.dw 61
.dw event049    ; << "...Afraid to go outdoors"
.dw $0000


map019:                         ; SABLE shop
; Compression = 59% ; Saved = 56 bytes
.db $3F,$3F,$3F,$3F,$3F,$3F,$2E,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$28,$F4,$0A
.db $D0,$A0,$9A,$0A,$F4,$29,$F4,$B2,$99,$8C,$C6,$A3,$91,$E8,$28,$F4,$13,$C0
.db $0F,$F4,$80,$00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 17          ; <- Connected map: SABLE
.db 0           ; <- Enemy Difficulty Level (no enemies)
.db 0           ; <- Enemy 1
.db 0           ; <- Enemy 2
.db 0           ; <- Enemy 3
.dw 80
.dw event049    ; << "Edmund declared a curfew..."
.dw $0000

map020:                         ; Get the key here
; Compression = 62% ; Saved = 59 bytes
.db $3F,$3F,$3F,$1B,$F4,$BA,$9D,$8E,$C7,$A0,$2E,$F4,$29,$F4,$29,$F4,$12,$C0
.db $10,$F4,$29,$F4,$29,$F4,$29,$F4,$29,$F4,$31,$F4,$BA,$84,$8E,$C7,$A0,$1E
.db $00
.db 12          ; map width
.db 8           ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 17          ; <- Connected map: SABLE
.db 0           ; <- Enemy Difficulty Level (no enemies)
.db 0           ; <- Enemy 1
.db 0           ; <- Enemy 2
.db 0           ; <- Enemy 3
.dw 53
.dw event050    ; << You battle Lurid
.dw $0000


map021:                         ; Alcatraz Colony
; Compression = 30% ; Saved = 160 bytes
.db $D4,$AA,$95,$8A,$C0,$25,$D4,$09,$D4,$AA,$1A,$F4,$BA,$9D,$8E,$C7,$A3,$D1
.db $E8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$95,$1B,$D4,$19,$D4,$39,$F4
.db $39,$F4,$12,$F8,$18,$F4,$AA,$9E,$8A,$C0,$82,$D0,$80,$D4,$19,$D4,$11,$D4
.db $AA,$95,$8E,$C4,$A3,$D1,$E8,$20,$F4,$11,$F4,$09,$F8,$09,$F4,$AA,$95,$13
.db $D4,$11,$D4,$09,$D4,$11,$D4,$19,$F4,$19,$F4,$31,$F4,$13,$E1,$E8,$D4,$11
.db $D4,$09,$D4,$11,$D4,$11,$D4,$8C,$0A,$D4,$09,$F4,$21,$F4,$BA,$2A,$F4,$BA
.db $8E,$8E,$C5,$A0,$81,$A8,$08,$D4,$11,$D4,$11,$D4,$11,$D4,$11,$F4,$31,$F4
.db $BA,$32,$F4,$AA,$1A,$D4,$09,$D4,$AA,$0A,$D4,$11,$D4,$AA,$8E,$8E,$C7,$C3
.db $D0,$F0,$20,$F4,$8C,$9D,$2B,$F4,$AA,$0A,$D4,$21,$D4,$19,$D4,$AA,$8E,$2C
.db $F0,$28,$F4,$22,$C8,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7
.db $A3,$D1,$E8,$38,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D,$8E,$C7,$A3
.db $A1,$D0,$20,$E8,$21,$E8,$B4,$8E,$2C,$F0,$21,$F0,$18,$F4,$11,$F4,$B4,$0A
.db $E8,$11,$E8,$09,$DC,$11,$DC,$11,$E8,$9C,$9D,$89,$C7,$A1,$E0,$20,$C8,$F4
.db $1A,$F0,$A0,$09,$F4,$B4,$12,$E8,$B4,$1A,$E8,$AE,$97,$13,$E8,$09,$F4,$88
.db $9D,$33,$F4,$19,$F4,$11,$F4,$09,$F8,$09,$DC,$AE,$9A,$8B,$C5,$E3,$A1,$B8
.db $E8,$B4,$86,$8D,$85,$E3,$D0,$80,$F4,$8C,$9D,$8E,$C7,$A3,$D1,$E8,$98,$21
.db $E0,$B0,$9D,$8D,$14,$E8,$AE,$1A,$E8,$B4,$9A,$13,$E8,$09,$F4,$09,$F4,$31
.db $F4,$19,$F4,$11,$F4,$B4,$0A,$E8,$11,$E8,$09,$DC,$11,$E8,$09,$DC,$11,$F4
.db $09,$F4,$31,$F4,$1A,$F0,$A0,$09,$F4,$B4,$9A,$23,$E8,$21,$E8,$19,$F4,$09
.db $F4,$32,$F8,$18,$F4,$11,$F4,$B4,$9A,$8D,$86,$C3,$A1,$D0,$E8,$B4,$9A,$8D
.db $86,$C3,$D1,$E8,$F4,$BA,$1A,$F4,$BA,$9D,$8E,$C7,$A3,$D1,$E8,$F4,$BA,$9D
.db $8E,$C7,$A3,$D1,$E8,$00
.db 32          ; map width
.db 17          ; map height
.db 1           ; graphics set
.dw picture6    ; fight background: "Mechanical Archives"
.db 17          ; <- Connected map: none
.db 30          ; <- Enemy Difficulty Level
.db enemy_titan ; <- Enemy 1
.db enemy_redeye; <- Enemy 2
.db enemy_titan ; <- Enemy 3
.dw 93
.dw event028_000; << Lt. ASH revived
.dw 385
.dw event039_000; << Lt. ACE revived
.dw 65
.dw event038_000; << Lt. ION and MIRAGE revived
.dw 349
.dw event047    ; << PRO SHOP
.dw 477
.dw event057    ; << Prisoner
.dw 208
.dw event053    ; << Final boss
.dw $0000
